import fs from 'fs';
import kue from 'kue';
import path from 'path';
import config from './config';
import TAB from './modules/TAB'
import emlformat from 'eml-format';
import parse from 'emailjs-mime-parser'
import { json } from 'body-parser';
import Dashboard from './modules/Dashboard';
var readTextFile = require('read-text-file');

var MailParser = require("mailparser-mit").MailParser;
var mailparser = new MailParser();

export var queue = kue.createQueue({
    prefix: 'scanner',
    redis: {
        host: config.redis.host,
        port: config.redis.port,
        auth: config.redis.password
    }
});

var storeFile = function (data, fileName, type) {
    return new Promise((resolve, reject) => {
        try {
            var filePath = path.join(appRoot, '..', '..', 'Content', 'Unprocessed', fileName + '-' + type + '.txt');
            var fileStream = fs.createWriteStream(filePath, { flags: 'w' })
            if (data) fileStream.write(data)
            console.log('File successfully stored', fileName, type);
            resolve(true);
            // for reporting button changes
            // if (type == "header") {
            //     var header = data.join('\r\n')
            //     if (header && header.length) fileStream.write(header)
            //     console.log('File successfully stored', fileName, type);
            //     resolve(true);
            // }
            // else {
            //     if (data) fileStream.write(data)
            //     console.log('File successfully stored', fileName, type);
            //     resolve(true);
            // }
        } catch (err) {
            reject(new Error("Error creating Eml file ") + err);
        }
    })
}


var base64ToFile = function (data, fileName) {
    return new Promise((resolve, reject) => {
        try {
            if (typeof (data) == 'string') data = JSON.parse(data);
            var attachment = [];
            for (var i = 0; data && i < data.length; i++) {
                var filemime = (data[i].Key).split('.')[(data[i].Key).split('.').length - 1];
                var filename = fileName + '-attachment-' + i + '.' + filemime;
                var filePath = path.join(appRoot, '..', '..', 'Content', 'Attachments', filename);
                var writeStream = fs.createWriteStream(filePath);
                writeStream.write(Buffer.from(data[i].Value, "base64"))
                attachment.push({ name: filename, originalName: data[i].Key, type: filemime });
            }
            resolve(attachment);
        } catch (err) {
            console.log(err)
            reject(new Error("Error creating Eml file ") + err);
        }
    })
}

export default {
    init: () => {
        // queue.process("process-request", (job, done) => {
        //     console.log('Request Processing started for file', job.data.fileName)
        //     storeFile(job.data.header, job.data.fileName, 'header')
        //         .then(r => {
        //             return storeFile(job.data.body, job.data.fileName, 'body')
        //         })
        //         .then(r => {
        //             return base64ToFile(job.data.attachments, job.data.fileName);
        //         })
        //         .then(attachment => {
        //             console.log('Sending Response to TAB');
        //             return new TAB().processFile({ fileName: job.data.fileName, attachments: attachment, companyCode: job.data.token, to: job.data.to, from: job.data.from , subject : job.data.subject});
        //         })
        //         .then(r => {
        //             return new Dashboard().sendFcmToAdmin({ companyCode: job.data.token, to: job.data.to, from: job.data.from })
        //         })
        //         .then(r => {
        //             console.log("Request successfully Sent");
        //             done();
        //         })
        //         .catch(err => {
        //             console.log('Error processing request', err)
        //             done(new Error('Error processing Request'));
        //         })
        // })
        queue.process("process-request", (job, done) => {
            console.log('Request Processing started for file', job.data.fileName)
            storeFile(job.data.header, job.data.fileName, 'header')
                .then(r => {
                    return storeFile(job.data.body, job.data.fileName, 'body')
                })
                .then(r => {
                    return base64ToFile(job.data.attachments, job.data.fileName);
                })
                .then(attachment => {
                    console.log('Sending Response to TAB');
                    return new TAB().processFile({ fileName: job.data.fileName, attachments: attachment, companyCode: job.data.token, to: job.data.to, from: job.data.from, subject: job.data.subject });
                })
                .then(r => {
                    console.log("Request successfully Sent");
                    done();
                })
                .catch(err => {
                    console.log('Error processing request', err)
                    done(new Error('Error processing Request'));
                })
        })
        queue.process("convert-eml-to-header-body", (job, done) => {
            console.log("ndkn")
            var eml = fs.readFileSync(job.data.path, "utf-8");
            new Promise((resolve, reject) => {
                emlformat.read(eml, function (error, data) {
                    if (error) return reject(error);
                    var body = data.html;
                    var header = parse(eml).header;
                    var from;
                    if (data.from.email) from = data.from.email;
                    else from = '';
                    var to = data.to.email;
                    var attachment = [];
                    if (!(data.attachments == undefined)) {
                        for (var i = 0; i < data.attachments.length; i++) {
                            attachment.push({ Key: data.attachments[i].id.split('<')[1].split('>')[0], Value: data.attachments[i].data })
                        };
                    } else var attachment = [];
                    resolve({ header: header, body: body, to: to, from: from, attachments: attachment, token: job.data.companyCode })
                });

            }).then(r => {
                console.log("1")
                if (r) return new TAB().create(r)
                else throw new Error("Coudnt send mail for parsing")
            })
                .then(r => {
                    console.log("2")
                    if (r) done();
                    else throw new Error("Coudnt send mail for parsing")
                })
                .catch(err => {
                    console.log("0")
                    console.log(err)
                    done(err);
                })

        })

        queue.process("convert-txt-file-to-header-body", (job, done) => {
            new Promise((resolve, reject) => {
                resolve(readTextFile.readSync(job.data.path))
            })
                .then(r => {
                    if (r) {
                        var data = { header: JSON.parse(r).header, body: JSON.parse(r).body, to: JSON.parse(r).to, from: JSON.parse(r).from, attachment: JSON.parse(r).attachments, token: job.data.companyCode };
                        return new TAB().create(data);
                    } else throw new Error("Coudnt parse Txt File" + job.data.filePath)
                })
                .then(r => {
                    if (r) done();
                    else throw new Error("Coundt send mail for parsing");
                })
                .catch(err => {
                    console.log(err);
                    done(err);
                })
        })
    }
}