import { generateQueryString } from '../functions';
import request from 'request'
export default class API {

    constructor(baseUrl = null) {
        this.baseUrl = baseUrl;
        this.get = this.get.bind(this);
    }

    setBaseUrl(baseUrl) {
        this.baseUrl = baseUrl;
    }

    get(url, queryParams = {}) {
        return new Promise((resolve, reject) => {
            request.get(url + '?' + generateQueryString(queryParams), (error, response, body) => {
                if (error) { return reject(error); }
                if (response && response.statusCode >= 400) {
                    try {
                        if (isJson(body) && JSON.parse(body) && JSON.parse(body).error) {
                            return reject(new RequestError(JSON.parse(body).error, response.statusCode));
                        } else if (body && body.error) {
                            return reject(new RequestError(body.error, response.statusCode));
                        } else {
                            return reject(new RequestError(body, response.statusCode));
                        }
                    } catch (err) {
                        return reject(err);
                    }
                } else if (response && body) {
                    resolve(isJson(body) ? JSON.parse(body) : body);
                } else {
                    return reject(new RequestError("Unexpected Response.."));
                }
            });
        });
    }
    post(url, data) {
        // console.log("Sending post request to", url, data);
        return new Promise((resolve, reject) => {
            request.post(url, { json: data }, (error, response, body) => {
                if (error) { return reject(error); }
                if (response && response.statusCode >= 400) {
                    try {
                        if (isJson(body) && JSON.parse(body) && JSON.parse(body).error) {
                            return reject(new RequestError(JSON.parse(body).error, response.statusCode));
                        } else if (body && body.error) {
                            return reject(new RequestError(body.error, response.statusCode));
                        } else {
                            return reject(new RequestError(body, response.statusCode));
                        }
                    } catch (err) {
                        return reject(err);
                    }
                } else if (response && body) {
                    resolve(isJson(body) ? JSON.parse(body) : body);
                } else {
                    return reject(new RequestError("Unexpected Response.."));
                }
            });
        });
    }
    patch(url, data) {
        return new Promise((resolve, reject) => {
            request.patch(url, { json: data }, (error, response, body) => {
                if (error) { return reject(error); }
                if (response && response.statusCode >= 400) {
                    try {
                        if (isJson(body) && JSON.parse(body) && JSON.parse(body).error) {
                            return reject(new RequestError(JSON.parse(body).error, response.statusCode));
                        } else if (body && body.error) {
                            return reject(new RequestError(body.error, response.statusCode));
                        } else {
                            return reject(new RequestError(body, response.statusCode));
                        }
                    } catch (err) {
                        return reject(err);
                    }
                } else if (response && body) {
                    resolve(isJson(body) ? JSON.parse(body) : body);
                } else {
                    return reject(new RequestError("Unexpected Response.."));
                }
            });
        });
    }
}