import spfParser from "spf-parse";
import {Netmask} from "netmask";
import dig from 'dns-node-dig';
var domainIps = {};

class SPFValidator {
	constructor(options) {
		if (typeof options === "string" || options instanceof String) {
			options = {
				domain: options
			};
		}

		this.options = options || {};
	}

	getDNSTxt(domain) {
		return new Promise(function (resolve, reject) {
			if (domain.startsWith('v=spf1')) {
				resolve([{ value: domain }]);
			}
			else {
				dig([domain, 'TXT'])
					.then(r => {
						
						console.log("r is",r)
						if (r && r.answer && r.answer.length) resolve(r.answer);
						else resolve([]);
					})
					.catch(e => {
						console.log("Error occured",e);
						resolve([]);
					})
			}
		});
	}
	getMX(domainName) {
		return dig([domainName, 'MX'])
			.then(records => {
				if (records && records.answer && records.answer.length) resolve(records.answer);
				else resolve([])
			})
			.catch(e => {
				console.log(e);
				resolve([]);
			})
	}
	getCloudTxt(domain) {
		if (domain.endsWith('.spf.kdmarc.com')) domain = domain.replace('.spf.kdmarc.com', '');
		return new Promise(function (resolve, reject) {
			cloudns.findRecord('spf.kdmarc.com', domain, 'TXT')
				.then(r => {
					resolve(r)
				})
				.catch(e => {
					console.log(e);
					reject(e);
				})
		});
	}
	getRecords(domain, config) {
		// console.log("domain",domain)
		if (domain === this.options.domain) domainIps = {};
		if (!domain) domainIps[this.options.domain] = null;
		domain = domain || this.options.domain;
		var domainName = this.options.domain;
		let dnsPromise = this.getDNSTxt(domain);
		let myInstance = this;
			return dnsPromise.then(function (entries) {
					for (let i = 0; i < entries.length; i++) {
						let records, parsedRecord, recordsLength;

						if (config && config.record) {
							domainIps = {};

							records = spfParser(config.record);
							parsedRecord = spfParser(config.record);
						} else {
							records = spfParser(entries[i].value);
							parsedRecord = spfParser(entries[i].value);
						}
						if (records.valid) {
							if (!domainIps[domainName])
								domainIps[domainName] = { page: 0, lookup: 0, record: [], lookups: 0, netblocks: 0, ipsCount: 0, ip4: [], ip6: [], ip4Count: 0, ext: [] };

							domainIps[domainName]["record"].push({ name: domain, record: entries[i].value, });

							parsedRecord.mechanisms.reduce((prev, currVal) => {
								var otherRecords = 0;

								if (currVal.type == "a" || currVal.type == "mx") { otherRecords++; }


								if (currVal.type == "ip4" && currVal.value) {
									var range = new Netmask(currVal.value);
									domainIps[domainName]["ip4Count"] = domainIps[domainName]["ip4Count"] + range.size;
									domainIps[domainName]["ip4"].push(currVal.value);
									domainIps[domainName]["ipsCount"] = range.size + domainIps[domainName]["ipsCount"];
									domainIps[domainName]["netblocks"] = domainIps[domainName]["netblocks"] + 1;
								}
								if (currVal.type == "ip6" && currVal.value) { domainIps[domainName]["ip6"].push(currVal.value); }
								if (currVal.type == "mx" ) { domainIps[domainName]["ext"].push(`mx:${currVal.value}`); }
								if (currVal.type == "ptr" && currVal.value) { domainIps[domainName]["ext"].push(`ptr:${currVal.value}`); }
								if (currVal.type == "a" && currVal.value) { domainIps[domainName]["ext"].push(`a:${currVal.value}`); }

								if (currVal.type == "ptr" || currVal.type == "redirect" || currVal.type == "include" || currVal.type == "a" || currVal.type == "mx" || currVal.type == "exists")
									domainIps[domainName]["lookups"] =
										domainIps[domainName]["lookups"] + 1;

								if (currVal.type == "ptr" || currVal.type == "ip4" || currVal.type == "ip6" || currVal.type == "a" || currVal.type == "mx" || currVal.type == "exists") {
									if (domainIps[domainName]["netblocks"] + otherRecords >= 24 && (domainIps[domainName]["netblocks"] + otherRecords) % 24 == 0) {
										var name = "";

										if (!domainIps[domainName]["page"]) name = domainName;
										else name = "p" + domainIps[domainName]["page"] + "." + domainName;

										domainIps[domainName][domainIps[domainName].page]["record"] = domainIps[domainName][domainIps[domainName].page]["record"] + " include:" + name + " ~all";
										domainIps[domainName].page++;
									}

									if (!domainIps[domainName][domainIps[domainName].page]) {
										var name = "";
										if (!domainIps[domainName]["page"]) name = domainName;
										else name = "p" + domainIps[domainName]["page"] + "." + domainName;
										domainIps[domainName][domainIps[domainName].page] = { record: "v=spf1", domain: name, };
									}

									if (currVal.value != undefined && currVal.value) {
										domainIps[domainName][domainIps[domainName].page]["record"] = domainIps[domainName][domainIps[domainName].page]["record"] + " " + currVal.type + ":" + currVal.value;
									} else {
										domainIps[domainName][domainIps[domainName].page]["record"] = domainIps[domainName][domainIps[domainName].page]["record"] + " " + currVal.type;
									}
								}
							}, Promise.resolve());
							let haveIncludes = myInstance.haveIncludes(records);
							if (haveIncludes && myInstance.options.expandIncludes) {								
								domainIps[domainName].lookup++;
								return myInstance.expandIncludes(records);
								// expandPromise.then(resolve).catch(reject);
								// console.log("haveIncludes",haveIncludes)
								// return;
							} else if (!haveIncludes && !domainIps[domainName].page) {
								return domainIps;
							} else {
								return records;
							}
						}
					}

					return null;
				})
	}
	deleteRecords(domain, config) {
		domain = domain || this.options.domain;
		var domainName = this.options.domain;
		let cloudPromise = this.getCloudTxt(domain);
		let myInstance = this;

		return new Promise(function (resolve, reject) {
			cloudPromise
				.then(async txtRecord => {
					var keys = Object.keys(txtRecord), prom = [];
					for (let i = 0; i < keys.length; i++) {
						var key = keys[i];
						if (txtRecord && txtRecord[key].host.toLowerCase().endsWith(domainName.toLowerCase())) {
							prom.push(cloudns.deleteRecord("spf.kdmarc.com", txtRecord[key].id))
						}
						var records = spfParser(txtRecord[key].record);
						let haveIncludes = myInstance.haveIncludes(records);
						if (haveIncludes && myInstance.options.expandIncludes) {
							let expandPromise = myInstance.expandCloudIncludes(records);
							prom.push(expandPromise)
						}
					}
					Promise.all(prom).then(resolve)
				})
				.catch(function (e) {
					console.log(e);
					reject(e);
				});
		});
	}
	haveIncludes(records) {
		for (let j = 0; j < records.mechanisms.length; j++) {
			if (
				records.mechanisms[j].type === "include" ||
				records.mechanisms[j].type === "redirect" 
			) {
				return true;
			}
		}

		return false;
	}

	expandIncludes(records) {
		let dnsPromises = [];

		for (let i = 0; i < records.mechanisms.length; i++) {
			if (
				records.mechanisms[i].type === "include" ||
				records.mechanisms[i].type === "redirect"
			) {
				// domainIps[n].lookup++;
				dnsPromises.push(this.expandInclude(records.mechanisms, i));
			}
			// else if ( records.mechanisms[i].type === "mx" ) {
			// 	dnsPromises.push(this.expandMx(records.mechanisms, i, d));
			// }
			// else if ( records.mechanisms[i].type === "a" ) {
			// 	dnsPromises.push(this.expandA(records.mechanisms, i, d));
			// }
			// else if ( records.mechanisms[i].type === "ptr" ) {
			// 	dnsPromises.push(this.expandPtr(records.mechanisms, i, d));
			// }
		}

		let metaPromise = Promise.all(dnsPromises);

		return new Promise(function (resolve, reject) {
			metaPromise
				.then(function () {
					resolve(domainIps);
				})
				.catch(function (e) {
					console.log(e);
					reject(e);
				});
		});
	}
	expandCloudIncludes(records) {
		let dnsPromises = [];
		for (let i = 0; i < records.mechanisms.length; i++) {
			if (records.mechanisms[i].type === "include" || records.mechanisms[i].type === "redirect") {
				dnsPromises.push(this.expandCloudInclude(records.mechanisms, i));
			}
		}
		let metaPromise = Promise.all(dnsPromises);
		return new Promise(function (resolve, reject) {
			metaPromise
				.then(function () {
					resolve();
				})
				.catch(function (e) {
					reject(e);
				});
		});
	}

	expandCloudInclude(mechanisms, i) {
		let recordPromise = this.deleteRecords(mechanisms[i].value);

		return new Promise(function (resolve, reject) {
			recordPromise
				.then(function (records) {
					resolve(null);
				})
				.catch(function (e) {
					reject(e);
				});
		});
	}
	expandInclude(mechanisms, i) {
		let recordPromise = this.getRecords(mechanisms[i].value);

		return new Promise(function (resolve, reject) {
			recordPromise
				.then(function (records) {

					mechanisms[i].expanded = records;

					resolve(null);
				})
				.catch(function (e) {
					reject(e);
				});
		});
	}
	expandMx(mechanisms, i, d) {
		var domain = mechanisms[i].value ? mechanisms[i].value: d;
		let recordPromise =this.getMX(domain);

		return new Promise(function (resolve, reject) {
			recordPromise
				.then(function (records) {
					mx.reduce( ( p, a, index)=>{
					}, Promise.resolve())
					for(var j=0; j< records.length; i++){
						domainIps =  domainIps[d].ip4.push(records[i])
					}
					resolve(null);
				})
				.catch(function (e) {
					reject(e);
				});
		});
	}
	expandPtr(mechanisms, i) {
		let recordPromise = this.getRecords(mechanisms[i].value);

		return new Promise(function (resolve, reject) {
			recordPromise
				.then(function (records) {
					mechanisms[i].expanded = records;

					resolve(null);
				})
				.catch(function (e) {
					reject(e);
				});
		});
	}
	expandA(mechanisms, i) {
		let recordPromise = this.getRecords(mechanisms[i].value);

		return new Promise(function (resolve, reject) {
			recordPromise
				.then(function (records) {
					mechanisms[i].expanded = records;

					resolve(null);
				})
				.catch(function (e) {
					reject(e);
				});
		});
	}

}
module.exports.SPFValidator = SPFValidator;