import 'dotenv/config';
import { db } from './models';
import { restRouter } from './api';
import { webRouter } from './web';
// import socket from './socket';
import config from './config';
import appManager from './app';
// import './passport';
import kue from './kue';
import './errors';
import cors from 'cors';
import scheduler from './scheduler';
import path from 'path';


global.appRoot = path.resolve(__dirname);


const PORT = config.app.port;

const app = appManager.setup(config, path.join(__dirname, 'web', 'resources', 'views', 'layouts', 'main.hbs'));


app.use(cors());

/* Route handling */
app.use('/api', restRouter);

app.use('/', webRouter);
/* app.get('*', function(req, res) {
    res.sendFile(path.resolve(__dirname) + '/public/index.html');
}); */
app.use((req, res, next) => {
    const error = new Error('Not found');
    error.message = 'Invalid route';
    error.status = 404;
    next(error);
});

app.use((error, req, res, next) => {
    if (!(error instanceof RequestError)) {
        error = new RequestError('Some Error Occurred', 500, error.message);
    }
    // console.log("Request Url is",req.url);
    res.status(error.status || 500);
    let contype = req.headers['content-type'];
    var json = !(!contype || contype.indexOf('application/json') !== 0);
    if (json) {
        return res.json({ errors: error.errorList });
    } else {
        if (error.errorCode == 401) {
            return res.render('401', { message: error.message })
        } else if (error.errorCode == 500) res.send(error.message)
        else {
            return res.render(error.errorList.toString(), { layout: null })
        }
    }
});


// kue.init();

/* Database Connection */
db.sequelize.authenticate().then(function() {
    console.log('Nice! Database looks fine - ' + config.app.name);
    // scheduler.init();
    // socket.init(io);
}).catch(function(err) {
    console.log(err, "Something went wrong with the Database Update!")
});

/* Start Listening service */
app.listen(PORT, () => {
    console.log(`Server is running at PORT http://localhost:${PORT}`);
    // startServer();
});