import { db } from '../../../models';
import { Tab } from '../../../modules/Tab';
import { Gctx } from '../../../modules/Gctx';
import { validateMetaKeys } from '../../../functions';
import fs from 'fs';
import csvParser from 'csv-parser';
import { Exchange } from '../../../modules/Exchange';
import request from 'request'
import { Outlook } from '../../../modules/OutlookSearch';
import Audit from '../../../modules/Audit';
var parser = require('ua-parser-js');

const getOutlookUsers = async(accessToken, companyId) => {
    console.log("Inside getOutlookUsers", accessToken, companyId);
    const options = {
        url: "https://graph.microsoft.com/v1.0/users/?$select=id",
        headers: {
            'Authorization': accessToken,
            'Content-Type': 'application/json'
        }
    };

    console.log("Url formed to fetch IP", options.url);
    const data = await getDataFromUrl(options);
    for (let i = 0; i < data.value.length; i++) {
        await saveUser(data.value[i].id, companyId);
        console.log("User saved");
    }
    return data;
}

const getDataFromUrl = (options) => {
    return new Promise(resolve => {
        request.get(options, (error, response, body) => {
            resolve(JSON.parse(body));
        });
    });
}

const saveUser = (outlookId, companyId) => {
    console.log("saveUser", outlookId, companyId);
    return new Promise(resolve => {
        db.MetaAuth.findOne({ where: { companyId: companyId, value: outlookId } })
            .then(r => {
                if (!r) {
                    db.MetaAuth.create({ companyId: companyId, key: "outlookId", value: outlookId })
                        .then(userSaved => {
                            if (userSaved)
                                resolve(1)
                            else
                                throw new Error("Some Error occured");
                        })
                } else
                    resolve(1);
            })
            .catch(err => {
                throw new Error("Some Error occured");
            })
    })
}

export default {
    async index(req, res, next) {
        var { senderID, startDate, endDate } = req.body;
        var data = { senderID: senderID, startDate: startDate, endDate: endDate }
        var options = await validateMetaKeys(req.user)
        return new Tab().advanceScan({ data: data, options: options })
            .then(result => {
                if (result) res.status(200).json({ success: result.success })
                else throw new Error("Coudnt send Mail for Advance Scanning !")
            })
            .catch(err => {
                console.log(err)
                next(err);
            })

    },
    async index_v2_old(req, res, next) {
        var { senderID, startDate, endDate } = req.body;
        var data = { senderID: senderID, startDate: startDate, endDate: endDate }
        var options = await validateMetaKeys(req.user)
        return db.MetaAuth.findOne({ where: { companyId: req.user.companyId, key: "outlookAccessToken" } })
            .then(r => {
                if (r && r.key)
                    return true;
                else
                    return false;
            }).then(graph_api => {
                db.MetaAuth.findAll({ where: { companyId: req.user.companyId, key: "outlookId" } })
                    .then(r => {
                        return r;
                    }).then(outlookIds => {
                        new Tab().advanceScan({ data: data, options: options, graph_api: graph_api, companyId: req.user.companyId, outlookIds: outlookIds })
                            .then(result => {
                                if (!graph_api) {
                                    if (result) res.status(200).json({ success: result.success })
                                    else throw new Error("Coudnt fetch Mail!")
                                } else {
                                    console.log("result.value", result);
                                    if (result) res.status(200).json({ success: result.result })
                                    else throw new Error("Coudnt fetch Mail!")
                                }
                            })
                            .catch(err => {
                                console.log("Error while getting mails", err)
                                next(err);
                            })
                    })
            })

    },
    async index_v2(req, res, next) {
        console.log("Inside index_v2 advance search", req.body)
        var { senderID, startDate, endDate } = req.body;
        var options = await validateMetaKeys(req.user)
        console.log("options", JSON.parse(JSON.stringify(options)));
        if (options.userName && options.password) {
            var outlook = new Outlook();
            outlook.searchMail({ username: options.userName, password: options.password, sender: senderID, startDate, endDate, subject: req.body.mailSubject ? req.body.mailSubject : req.body.subject, senderIP: req.body.senderIP })
                .then(r => {
                    r = JSON.parse(JSON.stringify(r));
                    console.log("Results is", r);
                    res.send(r);
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: r, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} searched - ${senderID} using advanced search`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                })
        } else {
            res.send({ msg: "Exchange Credentials Not Found" })
            new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: r, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to search - ${senderID} using advanced search (Exchange credentials not exits)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
        }
    },
    async advanceScanResponse_old(req, res, next) {
        var filePath = req.body.success;
        var results = [];
        const result = [];
        var gctx;
        return new Promise((resolve, reject) => {
            fs.access(filePath, fs.F_OK, (err) => {
                if (err) return resolve("File not Found Please wait!")
                else resolve(filePath)
            })
        }).then(r => {
            if (r) {
                if (r == 'File not Found Please wait!') res.status(200).json({ success: "Please Wait!" })
                else {
                    fs.createReadStream(filePath).pipe(csvParser())
                        .on('data', function(data) { results.push(data) })
                        .on('error', function(error) { console.log(error) })
                        .on('end', async function() {
                            for (var i = 0; i < results.length; i++) {
                                await new Gctx().checkIpv4({ ip: results[i].FromIP }).then(r1 => {
                                    result.push({ SenderAddress: results[i]['SenderAddress'], RecipientAddress: results[i].RecipientAddress, Subject: results[i].Subject, Size: results[i].Size, FromIP: results[i].FromIP, Status: results[i].Status, Received: results[i].Received, gctx: r1.IP })
                                })

                            }
                            console.log(result, "----------------");
                            res.status(200).json({ success: result })
                        })
                }
            } else throw new Error("Coudnt parse CSV file")
        }).catch(err => {
            console.log(err)
            next(err);
        })
    },
    async advanceScanResponse(req, res, next) {
        console.log("Inside advanceScanResponse", req.body);

    },
    async blockSender(req, res, next) {
        var options = await validateMetaKeys(req.user)
        return new Exchange().blockSender({ sender: req.body.senderId, options: options })
            .then(r => {
                if (r) res.status(200).json({ success: true })
                else throw new Error("coundt block senderId!")
            }).catch(err => {
                console.log(err)
                next(err)
            })
    },

    async blockDomain(req, res, next) {
        var options = await validateMetaKeys(req.user)
        console.log(req.body.senderId.split('@')[1], "------------------")
        return new Exchange().blockDomain({ domain: req.body.senderId.split('@')[1], options: options })
            .then(r => {
                if (r) res.status(200).json({ success: true })
                else throw new Error("coundt block senderId!")
            }).catch(err => {
                console.log(err)
                next(err)
            })
    },

    async deleteMail(req, res, next) {
        console.log("Inside deleteMail", req.body, JSON.parse(JSON.stringify(req.user)).Company.MetaAuths);
        var options = await validateMetaKeys(req.user)
        var { sender, receiver, date, subject } = req.body;
        date = date.split(" ")[0]
        return db.MetaAuth.findOne({ where: { companyId: req.user.companyId, key: "outlookAccessToken" } })
            .then(r => {
                if (r && r.key)
                    return true;
                else
                    return false;
            }).then(graph_api => {
                if (!graph_api) {
                    var fileName = Math.random();
                    console.log(fileName);
                    return new Exchange().deleteMail({ subject: subject, startDate: date, endDate: date, mail: receiver, from: sender, filename: fileName, options: options })
                        .then(r => {
                            if (r) console.log(r)
                            else throw new Error("Coudnt send mail for deletion")
                        }).catch(err => {
                            console.log(err)
                            next(err)
                        })
                } else {
                    console.log("Graph API delete")
                }
            });
    },
    async updateToken(req, res, next) {
        console.log("Inside updateToken", req.body, JSON.parse(JSON.stringify(req.user)));
        console.log("req.user.companyId", req.user.companyId, req.body.accessToken);
        getOutlookUsers(req.body.accessToken, req.user.companyId)
            .then(r => {
                console.log("Result is", r);
            })
        return db.MetaAuth.findOne({ where: { companyId: req.user.companyId, key: "outlookAccessToken" } })
            .then(r => {
                if (r)
                    return true;
                else
                    return false;
            })
            .then(check => {
                if (check) {
                    db.MetaAuth.update({ value: req.body.accessToken }, { where: { companyId: req.user.companyId, key: "outlookAccessToken" } })
                        .then(r => {
                            if (r)
                                res.send("Token Updated");
                            else
                                throw new Error("Access Token Not Updated");
                        }).catch(err => {
                            console.log("Error occured", err);
                            next(err)
                        })
                } else {
                    db.MetaAuth.create({ companyId: req.user.companyId, key: "outlookAccessToken", value: req.body.accessToken })
                        .then(r => {
                            if (r)
                            //         return true;
                            //     else
                            //         return false;
                            // })
                            // .then(r => {
                            //     if (r) {
                            //         db.MetaAuth.create({ companyId: req.user.companyId, key: "outlookId", value: req.body.uniqueId })
                            //             .then(userSaved => {
                            //                 if (userSaved)
                                res.send("Token Updated");
                            else
                                throw new Error("Some Error occured");
                            //         })
                            // } else
                            //     throw new Error("Access Token Not saved");
                        }).catch(err => {
                            console.log("Error occured", err);
                            next(err)
                        })
                }
            });
    },
    async deleteToken(req, res, next) {
        console.log("Inside deleteToken", req.body, JSON.parse(JSON.stringify(req.user)));
        return db.MetaAuth.destroy({ where: { companyId: req.user.companyId, key: "outlookAccessToken" } })
            .then(r => {
                console.log("r is", JSON.parse(JSON.stringify(r)));
                if (r)
                    res.send("Token deleted");
                else
                    throw new Error("Access Token Not Deleted");
            })
    }
}