import { db } from '../../../models';
import dns from 'dns';
import whois from 'whois';
import iplocation from 'iplocation';
import psl from 'psl';
import Tab from '../../../modules/Tab';

Date.prototype.customFormat = function (formatString) {
    var YYYY, YY, MMMM, MMM, MM, M, DDDD, DDD, DD, D, hhhh, hhh, hh, h, mm, m, ss, s, ampm, AMPM, dMod, th;
    YY = ((YYYY = this.getFullYear()) + "").slice(-2);
    MM = (M = this.getMonth() + 1) < 10 ? ('0' + M) : M;
    MMM = (MMMM = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"][M - 1]).substring(0, 3);
    DD = (D = this.getDate()) < 10 ? ('0' + D) : D;
    DDD = (DDDD = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"][this.getDay()]).substring(0, 3);
    th = (D >= 10 && D <= 20) ? 'th' : ((dMod = D % 10) == 1) ? 'st' : (dMod == 2) ? 'nd' : (dMod == 3) ? 'rd' : 'th';
    formatString = formatString.replace("#YYYY#", YYYY).replace("#YY#", YY).replace("#MMMM#", MMMM).replace("#MMM#", MMM).replace("#MM#", MM).replace("#M#", M).replace("#DDDD#", DDDD).replace("#DDD#", DDD).replace("#DD#", DD).replace("#D#", D).replace("#th#", th);
    h = (hhh = this.getHours());
    if (h == 0) h = 24;
    if (h > 12) h -= 12;
    hh = h < 10 ? ('0' + h) : h;
    hhhh = h < 10 ? ('0' + hhh) : hhh;
    AMPM = (ampm = hhh < 12 ? 'am' : 'pm').toUpperCase();
    mm = (m = this.getMinutes()) < 10 ? ('0' + m) : m;
    ss = (s = this.getSeconds()) < 10 ? ('0' + s) : s;
    return formatString.replace("#hhhh#", hhhh).replace("#hhh#", hhh).replace("#hh#", hh).replace("#h#", h).replace("#mm#", mm).replace("#m#", m).replace("#ss#", ss).replace("#s#", s).replace("#ampm#", ampm).replace("#AMPM#", AMPM);
};

function fetchIp(ipAddress) {
    return new Promise((resolve, reject) => {
        iplocation(ipAddress)
            .then(r => {
                resolve(r)
            })
            .catch(e => {
                resolve({ countryCode: "unKnown" })
            })
    })
}

function getIPDetails(ipAddress, domain) {
    console.log("IP address", ipAddress);
    return new Promise((resolve, reject) => {
        dns.lookupService(ipAddress, 80, (err, hostname, service) => {
            if (err || !hostname) {
                whois(ipAddress, { follow: 3, verbose: true })
                    .then(result => {
                        if (result && result.data && result.data.domainName) {
                            resolve({
                                hostname: result.data.domainName,
                                domain: "*." + result.data.domainName
                            })
                        } else {
                            resolve({
                                hostname: "unKnown",
                                domain: "unKnown"
                            })
                        }
                    }).catch(err => {
                        resolve({
                            hostname: "unKnown",
                            domain: "unKnown"
                        })
                    })
            } else {
                // let domain = null; //parseDomain(hostname);
                if (domain && domain.domain) domain = "*." + domain.domain + '.' + domain.tld;
                else {
                    var dataDomain = psl.parse(hostname)
                    domain = dataDomain && dataDomain.domain ? "*." + dataDomain.domain : 'unKnown';
                }
                resolve({
                    hostname: hostname,
                    domain: domain
                })
            }
        });
    })
        .then(ipDetails => {
            if (ipDetails) {
                return fetchIp(ipAddress).then(r => [r, ipDetails])
            } else throw "Error parsing sources";
        })
        .then(([location, ipDetails]) => {
            if (location) {
                ipDetails.countryCode = location.countryCode;
            }
            console.log("IP Detail parse successfully.");
            return ipDetails;
        })
        .catch(err => {
            throw err;
        })
}

function checkExistingAggregateForDate(companyId, forDate, source) {
    let d = new Date(forDate);
    let startDate = d.getFullYear() + '-' + (d.getMonth() + 1) + '-' + d.getDate() + ' 00:00:00';
    let endDate = d.getFullYear() + '-' + (d.getMonth() + 1) + '-' + d.getDate() + ' 23:59:59';
    console.log("Start and end date", startDate, endDate);
    console.log("Inside checkExistingAggregateForDate", companyId, forDate, source)
    return db.tab_aggregateReports.findOne({
        where: {
            companyId: companyId,
            // forDate: { $gte: forDate },
            // forDate: forDate,
            forDate: {
                $between: [startDate, endDate]
            },
            source: source
        },
        logging: console.log
    })
        .then(r => {
            if (r)
                return r;
            else
                return false;
        }).catch(err => {
            console.log("Error while checking existing report for the date", err);
            return false;
        })
}

function updateReport(report, dataToUpdate) {
    console.log("updateReport", report.id);
    return db.tab_aggregateReports.update(dataToUpdate, { where: { id: report.id } })
        .then(r => {
            console.log("updateReport", JSON.parse(JSON.stringify(r)))
            return report.id;
        }).catch(err => {
            console.log("Error while updating values", err);
            return false;
        })

}

function createNewReportForDate(req) {
    return db.tab_aggregateReports.create({
        companyId: req.body.companyCode,
        forDate: req.body.forDate,
        source: req.body.source,
        totalMails: req.body.totalMails,
        spfPassCount: req.body.spfPassCount,
        dkimPassCount: req.body.dkimPassCount,
        dmarcPassCount: req.body.dmarcPassCount,
        totalUrlScanned: req.body.totalUrlScanned,
        urlVirusFound: req.body.urlVirusFound,
        totalAttachmentScanned: req.body.totalAttachmentScanned,
        attachmentsVirusFound: req.body.attachmentsVirusFound,
        totalUrlScannedGctx: req.body.totalUrlScannedGctx,
        urlVirusFoundGctx: req.body.urlVirusFoundGctx,
        totalAttachmentScannedGctx: req.body.totalAttachmentScannedGctx,
        attachmentVirusFoundGctx: req.body.attachmentVirusFoundGctx,
        totalDomainScanned: req.body.totalDomainScanned,
        domainVirusFound: req.body.domainVirusFound,
        totalIpScannedGctx: req.body.totalIpScannedGctx,
        ipVirusFoundGctx: req.body.ipVirusFoundGctx,
        createdAt: req.body.createdAt,
        updatedAt: req.body.updatedAt
    })
        .then(r => {
            console.log("aggregateReport", JSON.parse(JSON.stringify(r)).id);
            return r.id;
        }).catch(err => {
            console.log("error while creating new report", err);
            return false;
        })
}

function getWhoElseCounts(companyId, startDate, endDate) {
    let condition = { companyId: companyId };
    if (startDate != undefined && endDate != undefined && startDate != null && endDate != null)
        condition["forDate"] = { $gte: startDate, $lte: endDate };

    console.log("Condition formed", condition);
    return db.tab_aggregateReports.findAll({
        attributes: [
            [db.sequelize.fn('sum', db.sequelize.literal('totalMails +totalWhoElseCount')), 'totalMails'],
            [db.sequelize.fn('sum', db.sequelize.col('deletedCountWhoElse')), 'mailboxSecured']
        ],
        where: condition,
        logging: console.log
    }).then(r => {
        console.log("Result is", JSON.parse(JSON.stringify(r)));
        return r[0];
    })
        .catch(err => {
            console.log("Error while getting who else counts", err);
            throw new RequestError('Something went wrong!', 500)
        })
}

function getHeaderAnalyticsCount(companyId, startDate, endDate) {
    let condition = { companyId: companyId };
    if (startDate != undefined && endDate != undefined && startDate != null && endDate != null)
        condition["forDate"] = { $gte: startDate, $lte: endDate };

    return db.tab_aggregateReports.findAll({
        attributes: [
            [db.sequelize.fn('sum', db.sequelize.literal('totalMails - spfPassCount')), 'spfFailCount'],
            [db.sequelize.fn('sum', db.sequelize.literal('totalMails - dkimPassCount')), 'dkimFailCount'],
            [db.sequelize.fn('sum', db.sequelize.literal('totalMails - dmarcPassCount')), 'dmarcFailCount'],
            [db.sequelize.fn('sum', db.sequelize.col('totalMails')), 'totalMails']
        ],
        where: condition,
        // logging: console.log
    }).then(r => {
        console.log("Result is", JSON.parse(JSON.stringify(r)));
        return r[0];
    })
        .catch(err => {
            console.log("Error while getting source details", err);
            throw new RequestError('Something went wrong!', 500)
        })
}

async function getMailsWithFailedHeaders(companyId, startDate, endDate) {
    let condition = { companyId: companyId };
    if (startDate != undefined && endDate != undefined && startDate != null && endDate != null)
        condition["forDate"] = { $gte: startDate, $lte: endDate };

    return db.tab_aggregateReports.findAll({
        attributes: [
            'source', [db.sequelize.fn('sum', db.sequelize.col('totalMails')), 'totalMails'],
            [db.sequelize.fn('sum', db.sequelize.literal('totalMails - spfPassCount')), 'spfFailCount'],
            [db.sequelize.fn('sum', db.sequelize.literal('totalMails - dkimPassCount')), 'dkimFailCount'],
            [db.sequelize.fn('sum', db.sequelize.literal('totalMails - dmarcPassCount')), 'dmarcFailCount'],
        ],
        where: condition,
        group: 'source',
        // logging: console.log
    }).then(r => {
        console.log("Result is", JSON.parse(JSON.stringify(r)));
        return r;
    }).catch(err => {
        console.log("Error while getting source details", err);
        throw new RequestError('Something went wrong!', 500)
    })
}

function getAttachmentScannedCount(companyId, startDate, endDate) {
    let condition = { companyId: companyId };
    if (startDate != undefined && endDate != undefined && startDate != null && endDate != null)
        condition["forDate"] = { $gte: startDate, $lte: endDate };

    return db.tab_aggregateReports.findAll({
        attributes: [
            [db.sequelize.fn('sum', db.sequelize.col('totalAttachmentScanned')), 'totalAttachmentScanned'],
            [db.sequelize.fn('sum', db.sequelize.col('attachmentsVirusFound')), 'attachmentsVirusFound']
        ],
        where: condition,
        // logging: console.log
    }).then(r => {
        // console.log("Result is", JSON.parse(JSON.stringify(r)));
        return r[0];
    }).catch(err => {
        console.log("Error while getting attachment scanned", err);
        throw new RequestError('Something went wrong!', 500)
    })
}

function getUrlScannedCount(companyId, startDate, endDate) {
    let condition = { companyId: companyId };
    if (startDate != undefined && endDate != undefined && startDate != null && endDate != null)
        condition["forDate"] = { $gte: startDate, $lte: endDate };

    return db.tab_aggregateReports.findAll({
        attributes: [
            [db.sequelize.fn('sum', db.sequelize.col('totalUrlScanned')), 'totalUrlScanned'],
            [db.sequelize.fn('sum', db.sequelize.col('urlVirusFound')), 'urlVirusFound']
        ],
        where: condition,
        // logging: console.log
    }).then(r => {
        // console.log("Result is", JSON.parse(JSON.stringify(r)));
        return r[0];
    }).catch(err => {
        console.log("Error while getting urls scanned", err);
        throw new RequestError('Something went wrong!', 500)
    })
}

function gctxScannedCount(companyId, startDate, endDate) {
    let condition = { companyId: companyId };
    if (startDate != undefined && endDate != undefined && startDate != null && endDate != null)
        condition["forDate"] = { $gte: startDate, $lte: endDate };

    return db.tab_aggregateReports.findAll({
        attributes: [
            [db.sequelize.fn('sum', db.sequelize.col('totalUrlScannedGctx')), 'totalUrlScannedGctx'],
            [db.sequelize.fn('sum', db.sequelize.col('totalAttachmentScannedGctx')), 'totalAttachmentScannedGctx'],
            [db.sequelize.fn('sum', db.sequelize.col('totalDomainScanned')), 'totalDomainScanned'],
            [db.sequelize.fn('sum', db.sequelize.col('totalIpScannedGctx')), 'totalIpScannedGctx'],
            [db.sequelize.fn('sum', db.sequelize.col('urlVirusFoundGctx')), 'urlVirusFoundGctx'],
            [db.sequelize.fn('sum', db.sequelize.col('attachmentVirusFoundGctx')), 'attachmentVirusFoundGctx'],
            [db.sequelize.fn('sum', db.sequelize.col('domainVirusFound')), 'domainVirusFound'],
            [db.sequelize.fn('sum', db.sequelize.col('ipVirusFoundGctx')), 'ipVirusFoundGctx'],
        ],
        where: condition,
        // logging: console.log
    }).then(r => {
        // console.log("Result is", JSON.parse(JSON.stringify(r))[0]);
        return JSON.parse(JSON.stringify(r))[0];
    })
}

function getDailyReportedMailCount(companyId, startDate, endDate) {
    let condition = { companyId: companyId };
    if (startDate != undefined && endDate != undefined && startDate != null && endDate != null)
        condition["forDate"] = { $gte: startDate, $lte: endDate };

    return db.tab_aggregateReports.findAll({
        attributes: [
            [db.sequelize.fn('sum', db.sequelize.col('totalMails')), 'totalMails'],
            'forDate'
        ],
        where: condition,
        group: 'forDate',
        // logging: console.log
    }).then(r => {
        // console.log("Result is", JSON.parse(JSON.stringify(r)));
        return r;
    }).catch(err => {
        console.log("Error while daily reported counts", err);
    })
}

function scanUrlVirusTotal(companyId, urls) {
    console.log("Inside scanUrlVirusTotal", urls)
    var tab = new Tab();
    return tab.fetchLinkScanResult_aggregate({ companyCode: companyId, url: urls[0] });
}

function scanAttachmentVirusTotal(req, res, next) {
    db.Company.findOne({ where: { id: req.user.companyId } })
        .then(company => {
            var tab = new Tab();
            if (company) return tab.scanAttachment({ companyCode: companyId, name: req.body.fileName, type: req.body.type });
            else throw new RequestError('Server error!');
        })
        .then(scan => {
            if (scan) return res.status(200).json({ 'success': true });
            else throw new RequestError('Server error!');
        })
        .catch(err => {
            console.log(err);
            next(err);
        })
}

function seperateDatesAndCounts(data) {
    // console.log("Inside seperateDatesAndCounts", data);
    let dates = [];
    let counts = [];
    if (data.length) {
        for (let i = 0; i < data.length; i++) {
            counts.push(Number(data[i].totalMails));
            dates.push(data[i].forDate);
        }
    }
    return ({ counts, dates })
}

async function getSource(req) {
    let ipString = req.body.parsedEmail.headerLines.filter(r => r.key == "authentication-results")[0].line;
    let ipAddress = ipString.match(/\b\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\b/);
    if (ipAddress != undefined && ipAddress != null) {
        let ipDetails = await getIPDetails(ipAddress[0], req.body.domain);
        req.body.source = ipDetails.domain;
    } else {
        req.body.source = "Unknown";
    }

    return req;
}

function mapReportedMailWithAggregateReport(reportId, reportedMailId) {
    return db.reportedMailReportMappings.create({
        reportedMailId,
        aggregateReportId: reportId
    }).catch(err => {
        console.log("Error while mapping report with mail", err);
        return null;
    })
}

export default {
    async saveReport(req, res, next) {
        // console.log("Inside saveReport", req.body, req.body.parsedEmail.headerLines);
        req = await getSource(req);
        if (req.body.source != null && req.body.source != undefined) {
            // console.log("IP source", req.body.source);
            // req.body.urlVirusFound = await scanUrlVirusTotal(req.body.totalUrls);
            req.body.totalIpScannedGctx = 1;
            req.body.ipVirusFoundGctx = 0;
            req.body.forDate = new Date(req.body.forDate).customFormat('#YYYY#-#MM#-#DD#') //.toLocaleString('en-US', { timeZone: 'Asia/Kolkata' })
            if (req.body.domain) {
                req.body.totalDomainScanned = 1;
            }
            let reportId = null;
            let existingReport = await checkExistingAggregateForDate(req.body.companyCode, req.body.forDate, req.body.source);
            // console.log("existingReport", existingReport);
            if (existingReport) {
                let dataToUpdate = {
                    totalMails: existingReport.totalMails + req.body.totalMails,
                    spfPassCount: existingReport.spfPassCount + req.body.spfPassCount,
                    dkimPassCount: existingReport.dkimPassCount + req.body.dkimPassCount,
                    dmarcPassCount: existingReport.dmarcPassCount + req.body.dmarcPassCount,
                    totalUrlScanned: existingReport.totalUrlScanned + req.body.totalUrlScanned,
                    urlVirusFound: existingReport.urlVirusFound + req.body.urlVirusFound,
                    totalAttachmentScanned: existingReport.totalAttachmentScanned + req.body.totalAttachmentScanned,
                    attachmentsVirusFound: existingReport.attachmentsVirusFound + req.body.attachmentsVirusFound,
                    totalUrlScannedGctx: existingReport.totalUrlScannedGctx + req.body.totalUrlScannedGctx,
                    urlVirusFoundGctx: existingReport.urlVirusFoundGctx + req.body.urlVirusFoundGctx,
                    totalAttachmentScannedGctx: existingReport.totalAttachmentScannedGctx + req.body.totalAttachmentScannedGctx,
                    attachmentVirusFoundGctx: existingReport.attachmentVirusFoundGctx + req.body.attachmentVirusFoundGctx,
                    totalDomainScanned: existingReport.totalDomainScanned + req.body.totalDomainScanned,
                    domainVirusFound: existingReport.domainVirusFound + req.body.domainVirusFound,
                    totalIpScannedGctx: existingReport.totalIpScannedGctx + req.body.totalIpScannedGctx,
                    ipVirusFoundGctx: existingReport.ipVirusFoundGctx + req.body.ipVirusFoundGctx,
                }
                reportId = await updateReport(existingReport, dataToUpdate);
            } else {
                reportId = await createNewReportForDate(req);
            }

            if (reportId) {
                await mapReportedMailWithAggregateReport(reportId, req.body.reportedMailId)
                res.send("Success");
            }
            else
                throw new RequestError('Something went wrong!', 500)
        } else {
            throw new RequestError('Something went wrong!', 500)
        }
    },
    async mailboxAnalysis(req, res, next) {
        console.log("Inside mailboxAnalysis", req.body);
        let data = await getWhoElseCounts(req.user.Company.alias, req.body.startDate, req.body.endDate);
        res.send(data);
    },
    async dailyReports(req, res, next) {
        // console.log("Inside dailyReports", req.body);
        let data = await getDailyReportedMailCount(req.user.Company.alias, req.body.startDate, req.body.endDate);
        data = await seperateDatesAndCounts(data);
        res.send(data);
    },
    async headersAnalyticsCount(req, res, next) {
        // console.log("Inside headersAnalytics", req.body);
        let data = await getHeaderAnalyticsCount(req.user.Company.alias, req.body.startDate, req.body.endDate);
        res.send(data);
    },
    async failedHeadersMails(req, res, next) {
        // console.log("Inside failedHeadersMails", req.body);
        let failedMails = await getMailsWithFailedHeaders(req.user.Company.alias, req.body.startDate, req.body.endDate);
        res.send(failedMails);
    },
    async scannedAttachmentsAnalysis(req, res, next) {
        console.log("Inside scannedAttachmentsAnalysis", req.body);
        let data = await getAttachmentScannedCount(req.user.Company.alias, req.body.startDate, req.body.endDate);
        res.send(data);
    },
    async scannedUrlsAnalysis(req, res, next) {
        console.log("Inside scannedAttachmentsAnalysis", req.body);
        let data = await getUrlScannedCount(req.user.Company.alias, req.body.startDate, req.body.endDate);
        res.send(data);
    },
    async gctxScannedItems(req, res, next) {
        console.log("Inside gctxScannedItems", req.body);
        let { totalUrlScannedGctx, totalAttachmentScannedGctx, totalDomainScanned, totalIpScannedGctx, urlVirusFoundGctx, attachmentVirusFoundGctx, domainVirusFound, ipVirusFoundGctx } = await gctxScannedCount(req.user.Company.alias, req.body.startDate, req.body.endDate);
        res.send({ totalUrlScannedGctx, totalAttachmentScannedGctx, totalDomainScanned, totalIpScannedGctx, urlVirusFoundGctx, attachmentVirusFoundGctx, domainVirusFound, ipVirusFoundGctx });
    },
    async updateWhoElseCount(req, res, next) {
        console.log("Insdie updateReport", req.body);
        req = await getSource(req);
        console.log("Req.domain", req.body.source)
        req.body.forDate = new Date(req.body.forDate).customFormat('#YYYY#-#MM#-#DD#')
        let existingReport = await checkExistingAggregateForDate(req.body.companyCode, req.body.forDate, req.body.source);
        console.log("existingReport", JSON.parse(JSON.stringify(existingReport)));
        if (existingReport) {
            let dataToUpdate = {
                totalWhoElseCount: existingReport.totalWhoElseCount + req.body.whoElseCount,
            }
            await updateReport(existingReport, dataToUpdate);
        }
        res.send("Done")
    }
}