import JWT from 'jsonwebtoken';
import { db } from '../../../models';
import mailer from '../../../mailer';
import config from '../../../config';
import bcrypt from 'bcrypt-nodejs';
import iplocation from "iplocation";
import timeDifference from 'date-timestamp-diff';
import authenticator from 'authenticator';
import redis from "redis";
var parser = require('ua-parser-js');
import { validateEmail } from './../../../functions';
import Blowfish from 'egoroof-blowfish';
import Audit from '../../../modules/Audit';

const client = redis.createClient();
var bf = new Blowfish(toString(process.env.APP_SECRET), Blowfish.MODE.ECB, Blowfish.PADDING.NULL);
const fromHexString = hexString => new Uint8Array(hexString.match(/.{1,2}/g).map(byte => parseInt(byte, 16)));

var JWTSign = function (user, date) {
    return JWT.sign({
        iss: config.app.name,
        sub: user.id,
        iam: user.type,
        iat: date.getTime(),
        exp: new Date().setMinutes(date.getMinutes() + 30)
    }, config.app.secret);
}

var makeid = () => {
    var text = "";
    var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
    for (var i = 0; i < 7; i++)
        text += possible.charAt(Math.floor(Math.random() * possible.length));
    return text;
}
var makenumerickey = () => {
    var text = "";
    var possible = "0123456789";
    for (var i = 0; i < 7; i++)
        text += possible.charAt(Math.floor(Math.random() * possible.length));
    return text;
}
var makealphakey = () => {
    var text = "";
    var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
    for (var i = 0; i < 7; i++)
        text += possible.charAt(Math.floor(Math.random() * possible.length));
    return text;
}

var createCompanyUser = function (firstName, lastName, email, password) {
    return new Promise((resolve, reject) => {
        var alias = makealphakey();
        var key = makenumerickey();
        var companyId;
        db.Company.create({ alias, key })
            .then((company) => {
                if (company) return db.User.create({
                    firstName: firstName,
                    lastName: (lastName ? lastName : null),
                    email: email,
                    password: password,
                    status: 'active',
                    isNewUser: 0,
                    attempt: 0,
                    valid: 1,
                    twoFA: 0,
                    companyId: company.id
                })
                throw new RequestError('Server error', 401);
            })
            .then(r => {
                if (r) {
                    companyId = r.companyId
                    return db.MetaAuth.create({ companyId: r.companyId, key: 'tablite', value: true })
                } else throw new RequestError('Server error', 401);
            })
            .then(r => {
                if (r) {
                    var values = {
                        hackButton: true,
                        spf: true,
                        dkim: true,
                        dmarc: true,
                        content: true,
                        dkimalign: true,
                        spfalign: true,
                        gctx: true,
                        reportedby: false,
                        country: true,
                        attachments: true,
                        score: false
                    }
                    for (const [key, value] of Object.entries(values)) {
                        db.MetaAuth.create({ value: `${value}`, key: `${key}`, companyId: companyId })
                    }
                    return true
                } else throw new Error("Coundt make tablite in MetaAutj")
            })
            .then(user => {
                if (user) return resolve(true);
                else reject();
            })
    })
}

function verifyAuthCode(email, otp, r) {
    return new Promise(resolve => {
        client.get(email + ":" + "g-auth", (err, reply) => {
            console.log("data from redis", reply);
            if (err)
                throw new RequestError('Something went wrong!');
            else {
                reply = JSON.parse(reply).key;
                console.log("Reply", reply);
                var formattedKeyDecrypted = bf.decode(fromHexString(reply));
                var verify = reply ? (authenticator.verifyToken(formattedKeyDecrypted, otp) ? 200 : 401) : 401;
                console.log("verify", verify);
                r.status = verify;
                resolve(r);
            }
        })
    });
}

function verifyRecoveryCode(email, otp, data) {
    return db.recoveryCodes.findOne({ where: { email } })
        .then(r => {
            console.log(" r.codes.indexOf(otp)", r.codes.indexOf(otp));
            if (r && r.codes.indexOf(otp) != -1) {
                data.status = 200;
                r.codes.splice(r.codes.indexOf(otp), 1);
                db.recoveryCodes.update({
                    codes: JSON.stringify(r.codes)
                }, {
                    where: { email }
                })
            } else {
                data.status = 401;
            }
            return data;
        }).catch(err => {
            console.log("Error while fetching recovery codes");
            data.status = 500;
            return data;
        })
}

function verifyLogin(body) {
    return db.User.findOne({
        where: { email: body.email }
    }).then(r => {
        if (r.twoFA == 1 && (r.otp != body.otp)) {
            return false
        } else {
            return true;
        }
    }).catch(err => {
        console.log("Error while user verification", err);
        throw new RequestError('Something went wrong!', 500);
    })
}

export default {
    async register(req, res, next) {
        const { firstName, lastName, email, password } = req.body;
        var alias = makealphakey();
        var key = makenumerickey();
        var passwordHash = bcrypt.hashSync(password);
        db.User.findOne({ where: { email } })
            .then(user => {
                if (user) {
                    throw new RequestError('Email is already in use', 409);
                }
                return db.Company.create({ alias, key });
            })
            .then(company => {
                if (company) {
                    return db.User.findOrCreate({ where: { email: email }, defaults: { firstName, lastName, password: passwordHash, companyId: company.id } })
                }
                throw new RequestError('Server error', 401);
            }).then(r => {
                console.log("R is", JSON.parse(JSON.stringify(r)));
                if (r)
                    return db.MetaAuth.create({ companyId: r[0].id, key: 'tablite', value: 1 });
            })
            .then(result => {
                if (result) {
                    return mailer.sendRegisterationLink(email, key);
                } else {
                    throw new RequestError('Email is already in use', 409);
                }
            })
            .then(r => {
                return res.status(200).json({ email: email, key });
            })
            .catch(err => {
                console.log(err);
                next(err);
            });
    },
    async login(req, res) {
        console.log("Inside login", req.body);
        let verification = await verifyLogin(req.body);
        if (verification) {
            try {
                var date = new Date();
                var token = JWTSign(req.user, date);
                res.cookie('XSRF-token-tab', token, {
                    expire: new Date().setMinutes(date.getMinutes() + 30),
                    httpOnly: true,
                    secure: config.app.secure
                });

                return db.Company.findOne({ where: { id: req.user.id } })
                    .then(r => {
                        res.status(200).json({ success: true, token: r.alias })
                        new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: r.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} successfully logged in`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    })
            } catch (err) {
                console.log("login -> err", err)
            }
        } else {
            console.log("OTP verification not done trying to bypass")
            res.status(401).json({ success: false, token: null })
        }
    },
    async rootUserCheck(req, res) {
        // new Audit().saveAuditLog({ email: req.body.email, userId: "", companyId: "", EmployeeId: "", oldData: {}, newData: {}, metaData: req.headers, activity: `anonymous user with ${req.body.email} attempts login`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
        if (validateEmail(req.body.email)) {
            db.User.findOne({ where: { email: req.body.email } })
                .then(user => {
                    if (user) {
                        new Audit().saveAuditLog({ email: req.body.email, userId: "", companyId: "", EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.body.email} successfully verified as admin`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                        return res.status(200).json({ success: true, redirect: false, email: req.body.email, isNewUser: user.isNewUser });
                    }
                    new Audit().saveAuditLog({ email: req.body.email, userId: "", companyId: "", EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `anonymous user ${req.body.email} not exists in our database`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    return res.status(401).json({ success: false, redirect: false, msg: "A TAB account with that sign-in information does not exist. Try again or create a new account." })
                })
        } else {
            db.Company.findOne({ where: { alias: req.body.email } })
                .then(company => {
                    console.log("Company found", JSON.parse(JSON.stringify(company)));
                    if (company) {
                        new Audit().saveAuditLog({ email: req.body.email, userId: "", companyId: "", EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.body.email} successfully verified as employee`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                        return res.status(200).json({ success: false, alias: req.body.email, redirect: true })
                    }
                    new Audit().saveAuditLog({ email: req.body.email, userId: "", companyId: "", EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `anonymous user ${req.body.email} not exists in our database`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    return res.status(401).json({ success: false, alias: req.body.email, redirect: false, msg: "A TAB account with that sign-in information does not exist. Try again or create a new account." })
                })
        }
    },

    async changePassword(req, res, next) {
        var hash = bcrypt.hashSync(req.body.newPass);
        if (req.user.type == "root") {
            db.User.update({ password: hash, verf_key: null, attempt: 0, valid: 1, isNewUser: 0 }, { where: { id: req.user.id } })
                .then(userUpdate => {
                    if (userUpdate) return res.status(200).json({ success: true })
                    throw new RequestError('Server error', 500);
                })
                .catch(err => {
                    next(err);
                })
        } else {
            db.Employee.update({ password: hash, verf_key: null, attempt: 0, valid: 1, isNewUser: 0, resetPassword: 0 }, { where: { id: req.user.id } })
                .then(userUpdate => {
                    if (userUpdate[0]) return res.status(200).json({
                        success: true
                    })
                    throw new RequestError('Server error', 500);
                })
                .catch(err => {
                    next(err);
                })
        }
    },

    async changeEmployeePassword(req, res, next) {
        // var email = req.body.email;
        // var newPass2=req.body.confirmPass;
        // var newPass=req.body.newPass;
        // var paswd=  /^(?=.*[0-9])(?=.*[!@#$%^&*])[a-zA-Z0-9!@#$%^&*]{7,15}$/;
        // if (newPass.match(paswd)) {
        //     if (newPass == newPass2) {
        // var hash = bcrypt.hashSync(req.body.newPass);
        // db.Employee.update({ password: hash, verf_key: null, attempt: 0, valid: 1, isNewUser: 0 , resetPassword:0 }, { where: { id: req.user.id } })
        //     .then(userUpdate => {
        //         if (userUpdate) 
        return res.status(200).json({
            success: true
        })
        //     throw new RequestError('Server error', 500);
        // })
        // .catch(err=>{
        //     next(err);
        // })
        //     } else {
        //         throw new RequestError('Password does not match', 500);
        //     }
        // } else {
        //     throw new RequestError('Password has to be between 8 to 15 characters which contain at least one numeric digit, one uppercase and one lowercase letter', 500);
        // }

    },

    async resetPassword(req, res) {
        console.log("Inside resetPassword", req.body)
        const { email, verificationCode, password } = req.body;
        db.User.findOne({
            where: { email: email, verf_key: verificationCode }
        })
            .then(result => {
                if (result) {
                    var hash = bcrypt.hashSync(password);
                    db.User.update({ password: hash, verf_key: null, attempt: 0, isVerify: 1 }, { where: { email: email } });
                    return res.status(200).json({ success: true });
                } else {
                    return res.status(500).json({ errors: ['Invalid verification code!'] });
                }
            })
            .catch(err => {
                console.log(err);
                return res.status(500).json({ errors: ['Error Updating Password!'] });
            })

    },
    async resendOtp(req, res, next) {
        const { email } = req.body;
        const otp = Math.random().toString(36).replace('0.', '');
        var date = new Date();
        var otpExpiry = new Date().setMinutes(date.getMinutes() + 10);
        db.User.findOne({ where: { email: email, isNewUser: 1 } })
            .then((user) => {
                if (user) {
                    var cur = new Date();
                    var creation = new Date(user.createdAt);
                    var lastUpdate = new Date((user.updatedAt));
                    var fromUpdate = new Date((user.updatedAt).getTime() + 30000);
                    var timeleft = new Date(fromUpdate).getTime() - cur.getTime();
                    timeleft = Math.round(timeleft / 1000);
                    if (fromUpdate < cur || creation.getTime() == lastUpdate.getTime()) {
                        return db.User.update({ otp, otpExpiry, attempt: null }, { where: { id: user.id } }).then(r => [r, 0])
                    }
                    return [0, timeleft];
                }
                throw new RequestError('Email is invalid');
            })
            .then(([up, timeleft]) => {
                if (up && !timeleft) {
                    mailer.sendNewUserLink(email, otp);
                    return res.status(200).json({ success: 1, msg: "Otp sent to your provided email.", timeleft: 30 })
                } else if (timeleft && !up) {
                    return res.status(200).json({ success: 0, msg: timeleft + " seconds remaining .", timeleft })
                }
                throw new RequestError('Server error');
            })
    },
    async otpGen(req, res, next) {
        var isNewUser = 1;
        const { firstName, lastName, email, phone } = req.body;
        const otp = Math.random().toString(36).replace('0.', '');
        var alias = makealphakey();
        var key = makenumerickey();
        var date = new Date();
        var otpExpiry = new Date().setMinutes(date.getMinutes() + 10);
        db.User.findOne({ where: { email: email, isNewUser: 1 } })
            .then(user => {
                if (user) {
                    var cur = new Date();
                    var creation = new Date(user.createdAt);
                    var lastUpdate = new Date(user.updatedAt);
                    var fromUpdate = new Date((user.updatedAt).getTime() + 30000);
                    var timeleft = new Date(fromUpdate).getTime() - cur.getTime();
                    timeleft = Math.round(timeleft / 1000);
                    if (fromUpdate < cur || creation.getTime() == lastUpdate.getTime()) {
                        return db.User.update({ otp: otp, otpExpiry: otpExpiry }, { where: { id: user.id, isNewUser: 1 } }).then(r => [r, "a", "Otp sent to your provided email."])
                    }
                    return [1, timeleft, "Resend OTP in " + timeleft + " seconds  ."];
                } else {
                    return db.Company.create({ alias, key }).then(r => [r, "b", "Otp sent to your provided email."])
                }
            })
            .then(([created, timeleft, msg]) => {
                if (created && timeleft == "b") {
                    mailer.sendNewUserLink(email, otp);
                    return db.User.create({ firstName: firstName, otpExpiry: otpExpiry, lastName: lastName, email, otp: otp, phone, valid: 0, isNewUser, companyId: created.id }).then(r => [r, -1, msg])
                } else if (created && timeleft == "a") {
                    mailer.sendNewUserLink(email, otp);
                    return [created, 0, msg]
                } else if (created) {
                    return [created, timeleft, msg]
                }
                throw new RequestError('Server error');
            })
            .then(([result, timeleft, msg]) => {
                if (result) {
                    return res.status(200).json({ status: "200", email, msg, timeleft });
                }
                throw new RequestError('Server error');
            })
            .catch((err) => {
                next(err);
            });

    },
    async otpVerify(req, res, next) {
        db.User.update({ verf_key: null, attempt: 0, valid: 1 }, { where: { id: req.user.id } })
            .then(User => {
                if (User) {
                    return res.status(200).json({ success: true, otp: "OTP VERFIED", "firstName": req.user.firstName + ' ' + req.user.lastName, phone: req.user.phone });
                }
                throw new RequestError('Server error', 401);
            }).catch(err => {
                next(err);
            })
    },
    async create(req, res, next) {
        let { email, firstName, lastName, phone, code, source, ref } = req.body;
        if (!code) {
            code = '';
        }
        var info = {};
        info = parser(req.headers['user-agent']);
        var userAgent = info;
        info.ip = req.headers["x-real-ip"] || req.ip;
        iplocation(info.ip)
            .then(detail => {
                info.userAgent = req.headers['user-agent'];
                var details = {
                    details: req.body.details,
                    info: info,
                    location: detail
                }
                var data = {
                    firstname: firstName,
                    lastname: lastName,
                    email: email,
                    phone: code + "-" + phone,
                    source: source,
                    referral: ref,
                    details: JSON.stringify(details)
                }
                return mailer.sendDomain(data, userAgent);
            })
            .then(r => {
                res.status(200).json({ success: true });
            })
            .catch((err) => {
                console.log(err);
                next(err)
            })
    },

    async sendReset(req, res) {
        const { email } = req.body;
        console.log("Inside sendReset", email)
        client.get(email + ":reset-password", (err, reply) => {
            console.log("Reply", reply, Date.now(),((Date.now() - reply) / 1000),Math.round((Date.now() - reply) / 1000))
            if (!reply || (reply && (Math.round((Date.now() - reply) / 1000) > 120))) {
                mailer.sendResetPassword(email)
                    .then(r => {
                        client.set(email + ":reset-password", Date.now())
                        return res.status(200).json({ success: true });
                    })
                    .catch(err => {
                        console.log(err);
                        return res.status(500).json({ errors: ['Error Sending Email!'] });
                    });
            } else {
                console.log("Mail not sent")
                return res.status(500).json({ errors: ['Mail Not Sent! Please try after 2 mins'] });
            }
        });
    },

    async userCheck(req, res, next) {
        var { email } = req.body;
        db.User.findOne({ where: { email: email }, include: [{ model: db.Order, where: { status: "success" }, required: false }] })
            .then(user => {
                if (user) {
                    if (user.Orders.length) {
                        return res.status(200).json({ isNewUser: user.isNewUser, firstName: user.firstName, lastName: user.lastName, email: user.email, success: 1, phone: user.phone, paid: 1 });
                    }
                    return res.status(200).json({ isNewUser: user.isNewUser, firstName: user.firstName, lastName: user.lastName, email: user.email, success: 1, phone: user.phone, paid: 0 });

                }
                return res.status(200).json({ success: 0 })

            })
            .catch((err) => {
                console.log(err);
                next(err);
            })
    },

    async otpLink(req, res, next) {
        var { email } = req.body;
        const newPassword = Math.random().toString(36).replace('0.', '');
        var password = bcrypt.hashSync(newPassword);
        db.User.update({ password }, { where: { email, isNewUser: 1 } })
            .then(up => {
                if (up) {
                    mailer.sendOtpLink(email, newPassword);
                    return res.status(200).json({ success: true, isNewUser: 1, msg: "Login Link has been sent!" })
                }
                throw new RequestError('Unable to update');
            })
            .catch(err => {
                next(err);
            })
    },

    async otpCheck(req, res, next) {
        var { email } = req.body;
        db.User.findOne({ where: { email: email } })
            .then(user => {
                if (user) {
                    return res.status(200).json({ isNewUser: user.isNewUser })
                }
                throw new RequestError(' ' + email + ' is not associated with any kdmarc account.', 401);
            })
            .catch((err) => {
                console.log(err);
                next(err);
            })
    },

    async isValidDomain(req, res, next) {
        console.log(req.params, req.query, req.body)
        var domainName = req.query.domainName;
        db.Company.findOne({ where: { alias: req.query.companyCode }, include: [{ model: db.MetaAuth, where: { value: domainName } }] })
            .then(r => {
                console.log(r)
                if (r) res.status(200).json({ 'success': r })
                else throw new RequestError("This Domain is Not Authorized to report Mails")
            })
            .catch(err => {
                console.log(err)
                next(err);
            })
    },
    async checkOTPforlogin(req, res, next) {
        var email = req.body.email;
        db.User.findOne({ attributes: ['twoFA'], where: { email: email } })
            .then(r => {
                // console.log("User found", json(r))
                if ((json(r).twoFA) == 1) return res.status(200).json({ success: 1, twoFA: 1 })
                if ((json(r).twoFA) == 2) return res.status(200).json({ success: 1, twoFA: 2 })
                else if ((json(r).twoFA) == 0) return res.status(200).json({ success: 0, twoFA: 0 })
                else throw new RequestError("Coudnt find if the User has 2FA")
            })
            .catch(err => {
                console.log(err)
                next(err);
            })
    },

    async sendOtp(req, res, next) {
        var email = req.body.email;
        var OTP = makenumerickey();
        var currentTimeStamp = Date.now();
        db.User.update({ otp: OTP, otpExpiry: currentTimeStamp }, { where: { email: email } })
            .then(r => {
                if (r) return mailer.sendOTP(email, OTP)
                else throw new RequestError("OTP coudnt be generated and Mail coudnt be send to the User")
            })
            .then(r => {
                if (r) res.status(200).json({ success: 1 })
                else throw new RequestError("Coudnt send Mail")
            })
            .catch(err => {
                console.log(err);
                next(err);
            })

    },

    async loginwithotp(req, res, next) {
        var { email, otp } = req.body;
        db.User.findOne({ attributes: ['otp', 'otpExpiry', 'twoFA'], where: { email: email } })
            .then(async r => {
                console.log("r.twoFA", r.twoFA);
                if (r.twoFA == 1) {
                    if (r.otp == otp) return r;
                    else throw new RequestError("Wrong OTP entered");
                } else {
                    console.log("Inside else");
                    console.log("Google authenticator code", email, otp);
                    if (!req.body.recovery_code)
                        r = await verifyAuthCode(email, otp, r);
                    else
                        r = await verifyRecoveryCode(email, otp, r);

                    console.log("R is", JSON.parse(JSON.stringify(r)));
                    return r;
                }
            })
            .then(r => {
                console.log("r", JSON.parse(JSON.stringify(r)));
                if (r.twoFA == 1) {
                    var databaseOtp = r.otpExpiry;
                    var currentDate = new Date;
                    const Age = timeDifference.convertDateToAge(databaseOtp.toUTCString(), currentDate.toUTCString());
                    console.log(Age.minute)
                    if (Age.minute == undefined) Age.minute = 0;
                    if (Age.minute < 2) return res.status(200).json({ success: 1 });
                    else throw new RequestError("OTP is Expired kindly request a New OTP");
                } else if (r.twoFA == 2) {
                    return res.status(r.status).json({ success: r.status == 200 ? 1 : 0, message: r.status != 200 ? "Wrong OTP entered!" : "" });
                } else throw new RequestError("Wrong OTP entered!")
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },

    async otpResend(req, res, next) {
        var email = req.body.email;
        var OTP = makenumerickey();
        var currentTimeStamp = Date.now();
        db.User.update({ otp: OTP, otpExpiry: currentTimeStamp }, { where: { email: email } })
            .then(r => {
                if (r) return mailer.sendOTP(email, OTP)
                else throw new RequestError("OTP coudnt be generated and Mail coudnt be send to the User")
            })
            .then(r => {
                if (r) res.status(200).json({ success: 1 })
                else throw new Error("Coudnt send Mail")
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },

    async threatcop(req, res, next) {
        var { firstName, lastName, email, password } = req.body;
        return db.User.findOne({ where: { email: email } })
            .then(userexist => {
                if (!userexist) return createCompanyUser(firstName, lastName, email, password);
                else return true;
            })
            .then(r => {
                return res.status(200).json({ success: true });
            })
            .catch(err => {
                console.log(err)
                next(err);
            })
    },
    async updateProfileImage(req, res, next) {
        console.log("Inside updateProfileImage", req.body, JSON.parse(JSON.stringify(req.user)));
        db.User.update({ profileImage: req.body.profileImage }, { where: { id: req.user.id } })
            .then(updated => {
                console.log("Profile image updated");
                res.status(200).send("Profile Image Updated Successfully.")
            }).catch(err => {
                console.log("Error while updating profile image", err);
                // return new RequestError("Something went wrong! Please try again later")
                res.status(500).send("Something went wrong! Please try again later.")
            })
    }
};