import express from 'express';
import authController from './auth.controller';
import { localStrategy, jwtStrategy, otpLoginStrategy ,rootLoginStrategy} from '../../../middleware/strategy';
import { sanitize } from '../../../middleware/sanitizer';
import { validateBody, schemas } from '../../../middleware/validator';
import { userPermission } from '../../../middleware/userPermission';


export const authRouter = express.Router();
authRouter.route('/register').post(sanitize(), validateBody(schemas.registerSchema), authController.register);
authRouter.route('/lead').post(authController.create);
authRouter.route('/login').post(sanitize(), validateBody(schemas.loginSchema), localStrategy, authController.login);
authRouter.route('/rootLogin').post(sanitize(), validateBody(schemas.rootLoginSchema), rootLoginStrategy, authController.login);
authRouter.route('/rootCheck').post(sanitize(), authController.rootUserCheck);
authRouter.route('/changePassword','/changeEmployeePassword').post(sanitize(), validateBody(schemas.changePassword), jwtStrategy,authController.changePassword);
authRouter.route('/changeEmployeePassword').post(sanitize(), validateBody(schemas.changePassword),jwtStrategy, authController.changeEmployeePassword);
authRouter.route('/sendReset').post(sanitize(), validateBody(schemas.sendResetPassword), authController.sendReset);
authRouter.route('/reset').post(sanitize(), validateBody(schemas.resetPassword), authController.resetPassword);
authRouter.route('/resendOTP').post(sanitize(), validateBody(schemas.otpSchema), authController.resendOtp);
authRouter.route('/otpGen').post(sanitize(), validateBody(schemas.otpSchema), authController.otpGen);
authRouter.route('/otpVerify').post(sanitize(), validateBody(schemas.rootLoginSchema), otpLoginStrategy, authController.otpVerify);
authRouter.route('/userCheck').post(sanitize(), validateBody(schemas.userCheckSchema),authController.userCheck);
authRouter.route('/otpLink').post(sanitize(),validateBody(schemas.otpLinkSchema),authController.otpLink);
authRouter.route('/otpCheck').post(sanitize(),validateBody(schemas.sendResetPassword),authController.otpCheck);
authRouter.route('/isvaliddomain').get(authController.isValidDomain);
authRouter.route('/checkOTPforlogin').post(sanitize(),validateBody(schemas.check2FA),authController.checkOTPforlogin);
authRouter.route('/sendOtp').post(sanitize(),authController.sendOtp);
authRouter.route('/loginwithotp').post(sanitize(),validateBody(schemas.otp),authController.loginwithotp);
authRouter.route('/otpresend').post(sanitize(),authController.otpResend);
authRouter.route('/threatcop').post(sanitize(),authController.threatcop);
authRouter.route('/updateProfileImage').post(sanitize(),jwtStrategy,authController.updateProfileImage);