import { db } from '../../../models';
import { Tab } from '../../../modules/Tab';
import { validateMetaKeys } from '../../../functions';
import Audit from '../../../modules/Audit';
var parser = require('ua-parser-js');

export default {
    async index(req, res, next) {
        console.log("Inside blacklist index", req.body);
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                console.log("Company", JSON.parse(JSON.stringify(company)));
                if (company) return tab.blacklist({ companyCode: company.alias, id: req.body.reportedMailId });
                else throw new RequestError('Server error!');
            })
            .then(inbox => {
                if (inbox) return res.status(200).json(inbox)
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                console.log("Error inside blacklist index", err);
                next(err)
            })
    },
    async unBlockSender(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company) {
                    return tab.unBlockSender({ companyCode: company.alias, id: req.body.senderId, options: validateMetaKeys(req.user) })
                        .then(r => {
                            new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} unblocked the sender having sender id - ${req.body.senderId}`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                            return r;
                        });
                } else {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to unblock the sender having sender id - ${req.body.senderId} (Some error occured)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    throw new RequestError('Server error!')
                }
            })
            .then(inbox => {
                if (inbox && !inbox.errors) return res.status(200).json(inbox)
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                next(err)
            })
    },
    async unBlockDomain(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company) return tab.unBlockDomain({ companyCode: company.alias, id: req.body.senderDomainId, options: validateMetaKeys(req.user) })
                    .then(r => {
                        new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} unblocked the domain having domain id - ${req.body.senderDomainId}`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                        return r;
                    });
                else throw new RequestError('Server error!');
            })
            .then(inbox => {
                if (inbox && !inbox.errors) return res.status(200).json(inbox)
                else {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to unblock the domain having domain id - ${req.body.senderDomainId} (Some error occured)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    throw new RequestError('Server error!')
                };
            })
            .catch(err => {
                next(err)
            })
    },
}