import express from 'express';
import clientController from './client.controller';
import { sanitize } from '../../../middleware/sanitizer';
import path from 'path';
import multer from 'multer';
import { jwtStrategy } from '../../../middleware/strategy'


var attachmentDir = path.join(path.dirname(require.main.filename), 'public', 'logo')

var storage = multer.diskStorage({
    destination: function (req, file, cb) {
        cb(null, attachmentDir)
    },
    filename: function (req, file, cb) {
        cb(null, Date.now() + path.extname(file.originalname))
    }
})
var uploadAttachment = multer({ storage: storage, limits: { fileSize: 10485760 } })

export const clientRouter = express.Router();
clientRouter.route('/').get(sanitize(), clientController.index);
clientRouter.route('/product-list').get(clientController.productList);
clientRouter.route('/client-list').get(clientController.clientList);
clientRouter.route('/client-product/:clientId/:productId').get(sanitize(), clientController.productDetail);
clientRouter.route('/create-product').post(sanitize(), clientController.updateClientProduct);
clientRouter.route('/').post(sanitize(), uploadAttachment.single('logo'), clientController.store);
clientRouter.route('/edit/:id').post(sanitize(), clientController.fetch);
clientRouter.route('/').patch(sanitize(), uploadAttachment.single('logo'), clientController.update);
clientRouter.route('/:id').get(sanitize(), clientController.action);
clientRouter.route('/status-update/:id').post(sanitize(), clientController.statusUpdate);