import express from 'express';
import contentController from './content.controller';
import { sanitize } from '../../../middleware/sanitizer';
import path from 'path';
import multer from 'multer';
import { jwtStrategy } from '../../../middleware/strategy'

var attachmentDir = path.join(path.dirname(require.main.filename), 'public', 'logo')

var storage = multer.diskStorage({
    destination: function (req, file, cb) {
        cb(null, attachmentDir)
    },
    filename: function (req, file, cb) {
        cb(null, Date.now() + path.extname(file.originalname))
    }
})
var uploadAttachment = multer({ storage: storage, limits: { fileSize: 10485760 } })

export const contentRouter = express.Router();
contentRouter.route('/').get(sanitize(), contentController.index);
contentRouter.route('/file').get(sanitize(), contentController.getFile);
contentRouter.route('/product-list').get(sanitize(), contentController.clientProductsList);
contentRouter.route('/').post(sanitize(), uploadAttachment.fields([{'name': 'file', 'maxCount': 1}, {'name' :'thumbnail', 'maxCount': 1}]), contentController.store);
contentRouter.route('/fetch/:id').get(sanitize(), contentController.fetch);
contentRouter.route('/').patch(sanitize(),  uploadAttachment.single('thumbnail'), contentController.update);
contentRouter.route('/:id').get(sanitize(), contentController.action);