import { db } from '../../../models'
import { Tab } from '../../../modules/Tab';
import cloudinary from 'cloudinary';
import config from '../../../config';

function getCloudinarySignature(publicId) {
    let ts = Math.round((new Date()).getTime() / 1000);
    cloudinary.config({
        cloud_name: config.cloudinary.cloud_name,
        api_secret: config.cloudinary.api_secret,
        api_key: config.cloudinary.api_key
    });

    let serialized_params = {
        'public_id': publicId,
        'timestamp': ts,
    };

    let signature = cloudinary.utils.api_sign_request(serialized_params, cloudinary.config().api_secret);

    let signatureObj = {
        'timestamp': ts,
        'public_id': publicId,
        'api_key': config.cloudinary.api_key,
        'signature': signature,
        'cloud_name': config.cloudinary.cloud_name,
        'cloudinary_url': config.cloudinary.cloudinary_url
    };
    return signatureObj;
}

export default {
    async updateFcmToken(req, res, next) {
        console.log("Insdie updateFcmToken", req.query, req.user.id);
        db.User.update({
                fcmKey: req.query.token
            }, { where: { companyId: req.user.id } })
            .then(r => {
                console.log("Token updated");
            })
        res.send("Success");
    },
    async totalReport(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company)
                    return tab.totalReport({ companyCode: company.alias, startDate: req.query.startDate, endDate: req.query.endDate });
                else throw new RequestError('Server error!');
            })
            .then(inbox => {
                if (inbox) return res.status(200).json(inbox)
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                next(err)
            })

    },
    async pendingCount(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company)
                    return tab.pendingReport({ companyCode: company.alias, startDate: req.query.startDate, endDate: req.query.endDate });
                else throw new RequestError('Server error!');
            })
            .then(inbox => {
                if (inbox) return res.status(200).json(inbox)
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                next(err)
            })

    },
    async topReportedMail(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company)
                    return tab.topReportedMail({ companyCode: company.alias, startDate: req.query.startDate, endDate: req.query.endDate });
                else throw new RequestError('Server error!');
            })
            .then(inbox => {
                if (inbox) return res.status(200).json(inbox)
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                next(err)
            })

    },
    async approveReport(req, res, next) {
        console.log("Inside approvedReport", req.query)
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company)
                    return tab.approveReport({ companyCode: company.alias, startDate: req.query.startDate, endDate: req.query.endDate });
                else throw new RequestError('Server error!');
            })
            .then(inbox => {
                if (inbox) return res.status(200).json(inbox)
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                next(err)
            })

    },
    async declinedReport(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company)
                    return tab.declinedReport({ companyCode: company.alias, startDate: req.query.startDate, endDate: req.query.endDate });
                else throw new RequestError('Server error!');
            })
            .then(inbox => {
                if (inbox) return res.status(200).json(inbox)
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                next(err)
            })

    },
    async monthReport(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company)
                    return tab.monthReport({ companyCode: company.alias });
                else throw new RequestError('Server error!');
            })
            .then(inbox => {
                if (inbox) return res.status(200).json(inbox)
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                console.log("Error while getting monthly report", err);
                next(err)
            })

    },

    async spamOrHamp(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company)
                    return tab.spamOrHamp({ companyCode: company.alias });
                else throw new RequestError('Server error!');
            })
            .then(inbox => {
                if (inbox) return res.status(200).json(inbox)
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                next(err)
            })

    },

    async recentReportedMails(req, res, next) {
        return new Tab().recentReportedMails({ companyCode: req.user.Company.alias })
            .then(r => {
                if (r) res.status(200).json({ data: r.success })
                else throw new Error("Coudnt get ReportedMails from TAB")
            })
            .catch(err => {
                console.log(err, "eror")
            })
    },
    async getImageSignature(req, res, next) {
        console.log("Inside getImageSignature", req.body);
        let signature = await getCloudinarySignature(req.body.public_id);
        res.status(200).send(signature);
    }
}