import bcrypt from 'bcrypt-nodejs';
import { db } from '../../../models';
import mailer from '../../../mailer';
import Audit from '../../../modules/Audit';
var parser = require('ua-parser-js');

export default {
    async index(req, res, next) {
        db.Employee.findAll({ attributes: ['id', 'firstName', 'lastName', 'email'], where: { companyId: req.user.companyId } })
            .then(re => {
                if (re) return res.status(200).json({ 'data': re });
                else throw new RequestError("list is not found");
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async showUser(req, res, next) {
        if (req.params.userId) {
            db.Employee.findOne({ where: { id: req.params.userId, companyId: req.user.companyId } }, { attributes: ['id', 'firstName', 'lastName', 'email'] })
                .then(re => {
                    if (re)
                        return res.status(200).json({ 'User': re });
                    else
                        throw new RequestError('User is not found', 401);
                });
        } else {
            next("Invalid User Id");
        }
    },
    async showUserPolicy(req, res, next) {
        db.Employee.findOne({ where: { id: req.params.userId, companyId: req.user.companyId } })
            .then(user => {
                if (user) {
                    return db.UserPolicy.findAll({ attributes: ['id'], where: { userId: user.id }, include: [{ model: db.Policy }], })
                }
                throw new RequestError('User is not found', 401);
            })
            .then(re => {
                if (re) return res.status(200).json({ 'data': re });
                throw new RequestError('User is not found');
            })
            .catch(err => {
                console.log(err);
                next(err)
            })
    },
    async policyUserDelete(req, res) {
        var date = new Date();
        db.Employee.findOne({ where: { id: req.params.userId, companyId: req.user.companyId } })
            .then(user => {
                if (user) {
                    return db.UserPolicy.update({
                        deletedAt: date
                    }, {
                        where: {
                            userId: req.params.userId,
                            policyId: req.params.policyId
                        },
                    }).then(async r => {
                        new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} deleted a policy of an employee with ID - ${req.params.userId} and policy ID - ${req.params.policyId}`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                        return r;
                    })
                }
                throw new RequestError('Access denied !', 401);
            })
            .then(re => {
                if (re) return res.status(200).json({ 'status': "delete User Seccessfully" });
                throw new RequestError('Unable to delelte employee policy', 401);
            });
    },
    async userDelete(req, res, next) {
        var date = new Date();
        db.Employee.findOne({ where: { companyId: req.user.companyId, id: parseInt(req.params.userId) } })
            .then(user => {
                if (user && req.user.type == "root") {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `${req.user.type} user ${req.user.email} deleted an Iam user - ${req.params.userId}`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    return db.Employee.destroy({ where: { id: user.id }, })
                } else if (user && req.user.type == "employee" && req.user.id != user.id) {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `${req.user.type} user ${req.user.email} delete an Iam user - ${req.params.userId}`, IP: req.connection.remotedeleteress.split(`:`).pop() })
                    return db.Employee.destroy({ where: { id: user.id }, })
                }
                new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `${req.user.type} user ${req.user.email} tried to delete an Iam user - ${req.params.userId} (User didn't have permission to delete user)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                throw new RequestError('Access denied. ', 401);
            })
            .then(re => {
                if (re) return res.status(200).json({ 'status': "deleted User Seccessfully" });
                else throw new RequestError('User is not found');
            }).catch(err => {
                console.log(err);
                next(err);
            })
    },

    async addEmployee(req, res, next) {
        const { email, firstName, lastName, phone } = req.body;
        var otp = Math.random().toString(36).replace('0.', '');
        var date = new Date();
        var otpExpiry = new Date().setMinutes(date.getMinutes() + 2880);
        return db.Employee.findOne({ where: { email }, paranoid: false })
            .then(user => {
                if (user) {
                    if (user.deletedAt != null) {
                        new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `${req.user.type} user ${req.user.email} has updated a deleted Iam user - ${email}`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                        return db.Employee.update({ deletedAt: null, otp, otpExpiry }, { where: { id: user.id }, paranoid: false }).then(r => [r[0], " Employee added. Login Link has been sent to employee's email."])
                    }
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: r, newData: {}, metaData: parser(req.headers['user-agent']), activity: `${req.user.type} user ${req.user.email} has tried to add an existing Iam user - ${email}`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    throw new RequestError("User already exist", 409);
                }
                return db.Employee.create({ email, phone, otp, otpExpiry, firstName, lastName, valid: 1, companyId: req.user.companyId, resetPassword: 1, isNewUser: 1 }).then(r => {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: r, newData: {}, metaData: parser(req.headers['user-agent']), activity: `${req.user.type} user ${req.user.email} has added a new Iam user - ${email}`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    return [r, " New Employee added. Login credentials has been sent to employee's email."]
                })
            })
            .then(([alias, msg]) => {
                if (alias) {
                    mailer.sendEmployeePassword(email, otp, req.user.Company.alias);
                    return res.status(200).json({ success: true, key: otp, firstName, lastName, email, msg });
                }
                throw new RequestError("Server Error!");
            }).catch(err => {
                console.log(err);
                next(err);
                new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: r, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} has tried to add an Iam user - ${email} (Something went wrong)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            })
    },

    async employeePolicyList(req, res, next) {
        db.Employee.findOne({ where: { id: req.params.id, companyId: req.user.companyId } })
            .then(user => {
                if (user) {
                    return db.Policy.findAll({
                        where: { $or: [{ type: "managed" }, { type: "custom", companyId: req.user.companyId }] },
                        include: [{ model: db.UserPolicy, where: { userId: user.id }, required: false }]
                    })
                }
                throw new RequestError('User doent exist', 401)
            })
            .then(UserPolicy => {
                return res.status(200).json({ "data": UserPolicy });
            })
            .catch(err => {
                next(err);
            })
    },

    async employeePolicyUpdate(req, res, next) {
        const { list } = req.body;
        db.Employee.findOne({ where: { companyId: req.user.companyId, id: req.params.id } })
            .then(user => {
                if (user) {
                    return db.UserPolicy.destroy({ where: { userId: req.params.id } })
                }
                throw new RequestError('User doent exist', 401)
            })
            .then(des => {
                var UserPolicies = [];
                for (var i = 0; i < list.length; i++) {
                    UserPolicies.push({
                        policyId: parseInt(list[i]),
                        userId: req.params.id
                    })
                }
                return db.UserPolicy.bulkCreate(UserPolicies);
            })
            .then(UserP => {
                if (UserP) {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: UserP, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} added or updated policy(ies) of an employee with ID - ${req.params.id}`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    return res.status(200).json({ 'status': "true" });
                }
                new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: UserP, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to create or updated policy(ies) of an employee with ID - ${req.params.id} but failed (Some error occured)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                throw new RequestError("Server Error!");
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },

    async employeeResetPassword(req, res, next) {
        const { id } = req.body
        const Password = Math.random().toString(36).replace('0.', '');
        var passwordHash = bcrypt.hashSync(Password);
        db.Employee.findOne({ where: { id, companyId: req.user.companyId } })
            .then(user => {
                if (user) {
                    return db.Employee.update({ password: passwordHash, resetPassword: 1, isNewUser: 1 }, { where: { id } }).then(r => [r, user])
                }
                throw new RequestError('User doent exist', 401)
            })
            .then(([re, user]) => {
                if (re) {
                    return db.Company.findOne({ where: { id: user.companyId } }).then(r => [r, user])
                }
                throw new RequestError('No Company Found', 401)
            })
            .then(([re, user]) => {
                if (re) {
                    mailer.employeeResetPassword(user.email, Password, re.alias)
                    return res.status(200).json({ success: true, "status": "Reset Link Sent." })
                }
                throw new RequestError('Mail Was Not Send', 401)
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
}