import { db } from '../../../models';
import { Tab } from '../../../modules/Tab';
import { validateMetaKeys } from '../../../functions';
import path from 'path'
import { Outlook } from '../../../modules/OutlookSearch';
import Audit from '../../../modules/Audit';
var parser = require('ua-parser-js');
var readTextFile = require('read-text-file');

function updateWhoElseDeletedAggregates(aggregateReportId, isDeleted) {
    console.log("Inside updateWhoElseDeletedAggregates", aggregateReportId);
    return db.tab_aggregateReports.findOne({
        where: {
            id: aggregateReportId
        },
    })
        .then(existingReport => {
            console.log("existingReport.deletedCountWhoElse", existingReport.deletedCountWhoElse, existingReport.id)
            return db.tab_aggregateReports.update({
                deletedCountWhoElse: isDeleted ? (existingReport.deletedCountWhoElse + 1) : (existingReport.deletedCountWhoElse > 0 ? existingReport.deletedCountWhoElse - 1 : 0)
            }, {
                where: { id: existingReport.id }
            }).then(r => {
                console.log("Data updated");
            })
        }).catch(err => {
            console.log("Error while updating who else count", err);
        })
}

function updateMailBoxSecuredStatus(reportedMailId, isDeleted) {
    return db.reportedMailReportMappings.findOne({
        where: {
            reportedMailId
        }
    }).then(r => {
        if (r)
            updateWhoElseDeletedAggregates(r.aggregateReportId, isDeleted);
    }).catch(err => {
        console.log("Error while updating who else deleted count", err);
    })
}

export default {
    async index(req, res, next) {
        console.log("Inside index", req.user.companyId);
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                var data = { companyCode: company.alias, type: req.query.type, limit: req.query.limit, page: req.query.page, startDate: req.query.startDate, endDate: req.query.endDate };
                console.log("data formed", data);
                // console.log("req.query.search", req.query.search)
                if (req.query.search) data['search'] = req.query.search;
                if (company) {
                    console.log("Company got", JSON.parse(JSON.stringify(company.id)));
                    return tab.listInbox(data);
                } else {
                    console.log("Inside else");
                    throw new RequestError('Server error!');
                }
            })
            .then(inbox => {
                if (inbox) return res.status(200).json(inbox)
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async fetchMail(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company) return tab.fetchMail({ companyCode: company.alias, id: req.query.id });
                else throw new RequestError('Invalid Request!', 401);
            })
            .then(inbox => {
                if (inbox) return res.status(200).json(inbox)
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async analyticsDomainAuthRisk(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company) return tab.fetchDmarcDomainAuthRisk({ companyCode: company.alias, id: req.query.id });
                else throw new RequestError('Server error!');
            })
            .then(inbox => {
                if (inbox) return res.status(200).json(inbox)
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async fetchUrl(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company) return tab.fetchLinkScanResult({ companyCode: company.alias, id: req.query.id });
                else throw new RequestError('Server error!');
            })
            .then(scan => {
                if (scan) return res.status(200).json(scan);
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },

    async scanAttachment(req, res, next) {
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                var tab = new Tab();
                if (company) return tab.scanAttachment({ companyCode: company.alias, name: req.body.fileName, type: req.body.type });
                else throw new RequestError('Server error!');
            })
            .then(scan => {
                if (scan) return res.status(200).json({ 'success': true });
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async fetchAttachment(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company) return tab.fetchAttachmentScanResult({ companyCode: company.alias, attachmentName: req.query.name });
                else throw new RequestError('Server error!');
            })
            .then(scan => {
                if (scan) return res.status(200).json(scan);
                else throw new RequestError('Server error!');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async deleteMail(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(async (company) => {
                if (company) {
                    var options = await validateMetaKeys(req.user)
                    return tab.deleteMail({ companyCode: company.alias, reportedMailId: req.body.id, options: options })
                } else throw new RequestError('Server error!');
            })
            .then(r => {
                return res.status(200).json(r);
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async deleteSingleMail(req, res, next) {
        var tab = new Tab();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company) return tab.deleteSingleMail({ companyCode: company.alias, reportedMailResultId: req.body.reportedMailResultId, reportedMailId: req.body.reportedMailId, options: validateMetaKeys(req.user) })
                else throw new RequestError('Server error!');
            })
            .then(r => {
                if (r) return res.status(200).json(r);
                else throw new RequestError('Error deleting email');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async blockDomain(req, res, next) {
        var tab = new Tab();
        tab.blockDomain({ reportedMailId: req.body.reportedMailId, options: validateMetaKeys(req.user) })
            .then(r => {
                if (r) {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} blocked the domain having mail id - ${req.body.reportedMailId}`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    return res.status(200).json(r)
                } else {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to block the domain having mail id - ${req.body.reportedMailId} (Some error occured)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    throw new RequestError('Error deleting email')
                }
            })
            .catch(err => {
                console.log(err);
                next(err);
                new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to block the domain having mail id - ${req.body.reportedMailId} (Some error occured)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            })
    },
    async blockSender(req, res, next) {
        var tab = new Tab();
        tab.blockSender({ reportedMailId: req.body.reportedMailId, options: validateMetaKeys(req.user) })
            .then(r => {
                if (r) {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} blocked the sender having mail id - ${req.body.reportedMailId}`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    return res.status(200).json(r)
                } else {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to block the sender having mail id - ${req.body.reportedMailId} (Some error occured)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    throw new RequestError('Error deleting email')
                }
            })
            .catch(err => {
                console.log(err);
                next(err);
                new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to block the sender having mail id - ${req.body.reportedMailId} (Some error occured)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            })
    },
    async changeStatus(req, res, next) {
        console.log("Inside changeStatus", req.body);
        var tab = new Tab();
        if (req.body.type === "approved" || req.body.type === "declined") {
            tab.changeStatus({ reportedMailId: req.body.reportedMailId, status: req.body.type })
                .then(r => {
                    if (r) {
                        new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} ${req.body.type} a mail`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                        return res.status(200).json(r);
                    } else {
                        new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to ${req.body.type}d a mail (Some error occured)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                        throw new RequestError('Error migrating email');
                    }
                })
                .catch(err => {
                    console.log(err);
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to ${req.body.type}d a mail (Some error occured while calling the API)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    next(err);
                })
        } else {
            next("Invalid arguments..");
        }
    },
    async attachmentRisk(req, res, next) {
        var tab = new Tab();
        tab.attachmentRisk({ reportedMailId: req.query.reportedMailId })
            .then(r => {
                if (r) return res.status(200).json(r);
                else throw new RequestError('Error deting email');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async mailRisk(req, res, next) {
        var tab = new Tab();
        tab.mailRisk({ reportedMailId: req.body.id })
            .then(r => {
                if (r) return res.status(200).json(r);
                else throw new RequestError('Error deting email');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async attachmentRiskLink(req, res, next) {
        console.log("Inside attachmentRiskLink", req.query);
        var tab = new Tab();
        tab.attachmentRisk({ reportedMailId: req.query.reportedMailId })
            .then(r => {
                if (r) return res.status(200).json(r);
                else throw new RequestError('Error deting email');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async downloadAttachment(req, res, next) {
        var filePath = path.join(appRoot, '..', '..', 'Content', 'Attachments');
        var fileName = req.query.attachmentName;
        var options = {
            root: filePath,
            dotfiles: 'deny',
            headers: {
                'Content-Disposition': 'attachment; filename=' + req.query.attachmentName,
                'x-timestamp': Date.now(),
                'x-sent': true
            }
        };
        res.sendFile(fileName, options, function (err) {
            if (err) {
                console.log(err);
                next(err);
            } else {
                console.log('Sent:', fileName);
            }
        });
    },
    async urlRisk(req, res, next) {
        console.log("Inside urlRisk", req.body);
        var tab = new Tab();
        tab.urlRisk({ reportedMailId: req.body.reportedMailId })
            .then(r => {
                if (r) return res.status(200).json(r);
                else throw new RequestError('Error deting email');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },

    async gctxRisk(req, res, next) {
        var tab = new Tab();
        tab.gctxRisk({ reportedMailId: req.body.reportedMailId })
            .then(r => {
                if (r) return res.status(200).json(r);
                else throw new RequestError('Error deting email');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },


    async recoverMail(req, res, next) {
        var tab = new Tab();
        tab.recoverMail({ companyCode: req.user.Company.alias, reportedMailId: req.body.reportedMailId, options: validateMetaKeys(req.user) })
            .then(r => {
                if (r) return res.status(200).json(r);
                else throw new RequestError('Error deleting email');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async recoverSingleMail(req, res, next) {
        var tab = new Tab();
        console.log("recoverSingleMail ->  req.user.Company.alias", req.user.Company.alias)
        tab.recoverSingleMail({ companyCode: req.user.Company.alias, reportedMailId: req.body.reportedMailId, reportedMailResultId: req.body.reportedMailResultId, options: validateMetaKeys(req.user) })
            .then(r => {
                if (r) return res.status(200).json(r);
                else throw new RequestError('Error deleting email');
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async deleteMail_v2(req, res, next) {
        console.log("Inside deleteMail_v2", req.body, JSON.parse(JSON.stringify(req.user)))
        var outlook = new Outlook();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(async (company) => {
                if (company) {
                    var options = await validateMetaKeys(req.user)
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} deleted a mail from exchange`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    return outlook.deleteMail({ companyCode: company.alias, reportedMailId: req.body.reportedMailId, reportedMailResultId: req.body.reportedMailResultId, options: options, outlook_access_token: req.user.outlook_access_token })
                } else {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to delete a mail from exchange but failed (User not exist)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    throw new RequestError('Server error!')
                };
            })
            .then(r => {
                return res.status(200).json(r);
            })
            .catch(err => {
                console.log(err);
                next(err);
                new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to delete a mail from exchange but failed (Error occured in query)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            })
    },
    async recoverMail_v2(req, res, next) {
        console.log("Inside recoverMail_v2", req.body)
        var outlook = new Outlook();
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(async (company) => {
                if (company) {
                    var options = await validateMetaKeys(req.user)
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} recovered a mail from exchange`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    return outlook.recoverMail({ companyCode: company.alias, reportedMailId: req.body.reportedMailId, reportedMailResultId: req.body.reportedMailResultId, options: options, outlook_access_token: req.user.outlook_access_token })
                } else {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to recover a mail from exchange but failed (User not exist)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    throw new RequestError('Server error!')
                };
            })
            .then(r => {
                return res.status(200).json(r);
            })
            .catch(err => {
                console.log(err);
                next(err);
                new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to recover a mail from exchange but failed (Error occured in query)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            })
    },
    async fetchRawHeader(req, res, next) {
        console.log("Inside fetchRawHeader", req.body);
        let fileName = await new Tab().headerFileName({ id: req.body.id })
        let headerFilePath = path.join(appRoot, '..', '..', 'Content', 'Unprocessed', fileName.file + '-header.txt');
        let rawHeader = readTextFile.readSync(headerFilePath);
        res.send({ Result: rawHeader });
    },
    async fetchDnsblScanReport(req, res, next) {
        console.log("Inside fetchDnsblScanReport", req.bod);
        let results = await new Tab().fetchDnsblScanReport({ ip: req.body.ipAddr })
        res.send(results);
    },
    async fetchDeceptiveScanReport(req, res, next) {
        console.log("Inside fetchDeceptiveScanReport", req.body);
        let result = await new Tab().depceptiveDomainCheck({ domain: req.body.domain })
        result.domain = req.body.domain;
        res.send(result);
    },
    async updateWhoElseDeletedAggregates(req, res, next) {
        console.log("Inside updateWhoElseDeletedAggregates", req.body);
        updateMailBoxSecuredStatus(req.body.reportedMailId, req.body.isDeleted)
        res.send("success");
        // let d = new Date(req.body.forDate);
        // let startDate = d.getFullYear() + '-' + (d.getMonth() + 1) + '-' + d.getDate() + ' 00:00:00';
        // let endDate = d.getFullYear() + '-' + (d.getMonth() + 1) + '-' + d.getDate() + ' 23:59:59';
        // console.log("Start and end date", startDate, endDate);
        // return db.tab_aggregateReports.findOne({
        //     where: {
        //         companyId: req.body.companyId,
        //         forDate: {
        //             $between: [startDate, endDate]
        //         },
        //         source: req.body.source
        //     },
        // })
        //     .then(existingReport => {
        //         console.log("existingReport.deletedCountWhoElse", existingReport.deletedCountWhoElse, existingReport.id)
        //         return db.tab_aggregateReports.update({
        //             deletedCountWhoElse: req.body.isDeleted ? (existingReport.deletedCountWhoElse + 1) : (existingReport.deletedCountWhoElse > 0 ? existingReport.deletedCountWhoElse - 1 : 0)
        //         }, {
        //             where: { id: existingReport.id }
        //         }).then(r => {
        //             console.log("Data updated");
        //             res.send("success");
        //         })
        //     }).catch(err => {
        //         console.log("Error while updating who else count", err);
        //         res.status(500).send("Something went wrong");
        //     })
    }
}