import express from 'express';
import inboxController from './inbox.controller';
import { sanitize } from '../../../middleware/sanitizer';
import path from 'path';
import multer from 'multer';
import { jwtStrategy } from '../../../middleware/strategy'
import { userPermission } from '../../../middleware/userPermission';
import { getUnpackedSettings } from 'http2';

export const inboxRouter = express.Router();
inboxRouter.route('/').get(sanitize(), jwtStrategy, userPermission('reported-mail-read'), inboxController.index);
inboxRouter.route('/mail').get(sanitize(), jwtStrategy, userPermission('reported-mail-read'), inboxController.fetchMail);
inboxRouter.route('/analyticsDomainAuthRisk').get(sanitize(), jwtStrategy, userPermission('reported-mail-ananytics-read'), inboxController.analyticsDomainAuthRisk);
inboxRouter.route('/attachment').get(sanitize(), jwtStrategy, userPermission('reported-mail-attachments-read'), inboxController.fetchAttachment);
inboxRouter.route('/link').get(sanitize(), jwtStrategy, userPermission('reported-mail-link-read'), inboxController.fetchUrl);
inboxRouter.route('/deleteMail').post(sanitize(), jwtStrategy, userPermission('mail-delete'), inboxController.deleteMail);
inboxRouter.route('/deleteSingleMail').post(sanitize(), jwtStrategy, userPermission('reported-mail-whoelse-delete'), inboxController.deleteSingleMail);
inboxRouter.route('/blockSender').post(sanitize(), jwtStrategy, userPermission('sender-block'), inboxController.blockSender);
inboxRouter.route('/blockDomain').post(sanitize(), jwtStrategy, userPermission('domain-block'), inboxController.blockDomain);
inboxRouter.route('/scan').post(sanitize(), jwtStrategy, userPermission('reported-mail-attachment-scan'), inboxController.scanAttachment);
inboxRouter.route('/mailRisk').post(sanitize(), jwtStrategy, userPermission(''), inboxController.mailRisk);
inboxRouter.route('/changeStatus').post(sanitize(), jwtStrategy, userPermission('change-status-approve-declined-write'), inboxController.changeStatus);
inboxRouter.route('/attachmentRisk').get(sanitize(), jwtStrategy, userPermission('reported-mail-attachment-result-read'), inboxController.attachmentRisk);
inboxRouter.route('/attachmentnow').get(sanitize(), jwtStrategy, userPermission('reported-mail-attachment-scan'), inboxController.downloadAttachment);
inboxRouter.route('/urlRisk').post(sanitize(), jwtStrategy, userPermission('reported-mail-link-result-read'), inboxController.urlRisk);
inboxRouter.route('/gctxRisk').post(sanitize(), jwtStrategy, userPermission('gctx-tab-read'), inboxController.gctxRisk);
inboxRouter.route('/recoverMail').post(sanitize(), jwtStrategy, userPermission('mail-recover'), inboxController.recoverMail);
inboxRouter.route('/recoverSingleMail').post(sanitize(), jwtStrategy, userPermission('reported-mail-whoelse-recover'), inboxController.recoverSingleMail);
inboxRouter.route('/deleteMail_v2').post(jwtStrategy, userPermission('mail-delete'), inboxController.deleteMail_v2);
inboxRouter.route('/recoverMail_v2').post(sanitize(), jwtStrategy, userPermission('mail-delete'), inboxController.recoverMail_v2);
inboxRouter.route('/fetchHeader').post(sanitize(), jwtStrategy, inboxController.fetchRawHeader);
inboxRouter.route('/dnsbl-scan').post(sanitize(), jwtStrategy, inboxController.fetchDnsblScanReport);
inboxRouter.route('/deceptive-scan').post(sanitize(), jwtStrategy, inboxController.fetchDeceptiveScanReport);
inboxRouter.route('/update-mail-box-secured').post(sanitize(), inboxController.updateWhoElseDeletedAggregates);