import fs from 'fs';
import path, { resolve } from 'path';
import { db } from '../../../models';
import bcrypt from 'bcrypt-nodejs';
import crypto from 'crypto';
import cmd from 'node-cmd';
import { rejects } from 'assert';
import { validateMetaKeys } from '../../../functions';
import Blowfish from 'egoroof-blowfish';
import { start } from 'repl';
import API from '../../../modules/API';
import { Api } from '../../../modules/Apis';
import { domain, exit } from 'process';
import { json } from 'express';
import data from '../../../config';
import { Outlook } from '../../../modules/OutlookSearch';
import authenticator from 'authenticator';
import QRCode from 'qrcode';
import Audit from '../../../modules/Audit';
var parser = require('ua-parser-js');

import redis from "redis";
const client = redis.createClient();

var bf = new Blowfish(toString(process.env.APP_SECRET), Blowfish.MODE.ECB, Blowfish.PADDING.NULL);
const fromHexString = hexString => new Uint8Array(hexString.match(/.{1,2}/g).map(byte => parseInt(byte, 16)));

function mkDirByPathSync(targetDir, { isRelativeToScript = false } = {}) {
    const sep = path.sep;
    const initDir = path.isAbsolute(targetDir) ? sep : '';
    const baseDir = isRelativeToScript ? __dirname : '.';

    return targetDir.split(sep).reduce((parentDir, childDir) => {
        const curDir = path.resolve(baseDir, parentDir, childDir);
        try {
            fs.mkdirSync(curDir);
        } catch (err) {
            if (err.code === 'EEXIST') { // curDir already exists!
                return curDir;
            }

            // To avoid `EISDIR` error on Mac and `EACCES`-->`ENOENT` and `EPERM` on Windows.
            if (err.code === 'ENOENT') { // Throw the original parentDir error on curDir `ENOENT` failure.
                throw new Error(`EACCES: permission denied, mkdir '${parentDir}'`);
            }

            const caughtErr = ['EACCES', 'EPERM', 'EISDIR'].indexOf(err.code) > -1;
            if (!caughtErr || caughtErr && curDir === path.resolve(targetDir)) {
                throw err; // Throw if it's just the last created dir.
            }
        }

        return curDir;
    }, initDir);
}

function insertUpdateMetaAuth(req, i) {
    var socKey = "socEmailAlert";
    if (Object.keys(req.body)[i] == "metaAuth1")
        socKey = "socEmailAlert0"
    else if (Object.keys(req.body)[i] == "metaAuth2")
        socKey = "socEmailAlert1"
    else if (Object.keys(req.body)[i] == "metaAuth3")
        socKey = "socEmailAlert2"

    return db.MetaAuth.findOne({ where: { companyId: req.user.companyId, value: Object.values(req.body)[i] } })
        .then((existing_data) => {
            return db.MetaAuth.findOne({ where: { companyId: req.user.companyId, key: socKey } }).then(r => [existing_data, r])
        })
        .then(([existing_data, existing_soc]) => {
            if (existing_soc) {
                if (existing_data == null || (existing_data != null && existing_data.key == socKey) || Object.values(req.body)[i].trim().length == 0)
                    return db.MetaAuth.update({ value: Object.values(req.body)[i] }, { where: { id: existing_soc.id } })
                else {
                    throw new RequestError('Duplicate Entry! Email already added!');
                }
            } else {
                if (Object.values(req.body)[i] != null && Object.values(req.body)[i].trim().length > 0) {
                    if (!existing_data) {
                        return db.MetaAuth.create({ companyId: req.user.companyId, key: socKey, value: Object.values(req.body)[i] });
                    } else {
                        throw new RequestError('Duplicate Entry! Email already added!');
                    }
                } else {
                    return true;
                }
            }
        }).catch((error) => { throw new RequestError(error.message) })
}


function generateGoogleAuthenticatorUrl(email) {
    var formattedKey = authenticator.generateKey();
    console.log("Key generated", formattedKey);
    var bf = new Blowfish(toString(process.env.APP_SECRET), Blowfish.MODE.ECB, Blowfish.PADDING.NULL);
    const toHexString = bytes => bytes.reduce((str, byte) => str + byte.toString(16).padStart(2, '0'), '');
    var encoded = bf.encode(formattedKey);
    var formattedKeyencrypted = toHexString(encoded);
    client.set(email + ":" + "g-auth", JSON.stringify({ key: formattedKeyencrypted }), redis.print);
    var formattedToken = authenticator.generateToken(formattedKey);
    console.log("Code generated", formattedToken);
    var url = authenticator.generateTotpUri(formattedKey, email, "TAB", 'SHA1', 6, 60);
    // console.log("url generated", url);
    return url;
}

function generateQr(url) {
    return new Promise(resolve => {
        QRCode.toDataURL(url, function(err, url) {
            // console.log("Qr code", url);
            resolve(url);
        })
    })
}

function cryptoCodes() {
    return new Promise(resolve => {
        crypto.randomBytes(16, function(ex, buf) {
            let token = buf.toString('hex');
            resolve(token);
        });
    })
}

async function generateRecoveryCodes() {
    let codes = [];
    console.log("codes.length", codes.length)
    while (codes.length <= 6) {
        console.log("Inside while")
        codes.push(await cryptoCodes());
        if (codes.length >= 6)
            return codes;
    }
}

function saveRecoveryCodes(email, codes) {
    return db.recoveryCodes.destroy({ where: { email } })
        .then(r => {
            return r;
        }).then(r => {
            return db.recoveryCodes.create({
                email,
                codes
            })
        }).catch(err => {
            console.log("Error while saving recovery codes", err);
            throw new RequestError("Something went wrong!")
        })
}

export default {
    async index(req, res, next) {
        db.Company.findOne({ where: { id: req.user.companyId }, include: [{ model: db.MetaAuth, where: { key: 'publicKey' } }] })
            .then(company => {
                return res.status(200).json({ company })
            })
            .catch(err => {
                next(err);
            })
    },
    async checkCompany(req, res, next) {
        db.Company.findOne({ where: { id: req.query.companyId } })
            .then(company => {
                if (company) return res.status(200).json({ success: 1 })
                else return res.status(200).json({ success: 0 })
            })
            .catch(err => {
                next(err);
            })
    },
    async generateKey(req, res, next) {
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                if (company.alias) {
                    var filePath = path.join(appRoot, '..', '..', 'Content', 'Clients', 'Keys', company.alias)
                    const { privateKey, publicKey } = crypto.generateKeyPairSync('rsa', {
                        modulusLength: 2048,
                        publicKeyEncoding: {
                            type: 'spki',
                            format: 'pem'
                        },
                        privateKeyEncoding: {
                            type: 'pkcs8',
                            format: 'pem'
                        }
                    });
                    if (fs.existsSync(filePath) && fs.existsSync(filePath + '/' + 'public.txt') && fs.existsSync(filePath + '/' + 'private.txt'))
                        return new RequestError("File already exists");
                    if (!fs.existsSync(filePath)) mkDirByPathSync(filePath);
                    if (!fs.existsSync(filePath + '/' + 'public.pem'))
                        fs.writeFile(filePath + '/' + 'public.pem', publicKey, function(err) {
                            if (err) throw err;
                            console.log('Public key created successfully.');
                        });
                    if (!fs.existsSync(filePath + '/' + 'private.pem'))
                        fs.writeFile(filePath + '/' + 'private.pem', privateKey, function(err) {
                            if (err) throw err;
                            console.log('Private key created successfully.');
                        });
                }
                return 1;
            })
            .then(r => {
                res.status(200).json({ success: 1, data: r });
            })
            .catch(err => {
                next(err)
            })
    },
    async downloadPem(req, res, next) {
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(company => {
                res.download(path.join(appRoot, '..', '..', 'Content', 'Clients', 'Keys', company.alias, 'public.pem'), function(err) {});

            })
            .catch(err => {
                next(err)
            })

    },
    async update(req, res, next) {
        const { firstname, lastname } = req.body;
        console.log("Inside update", req.body);
        // console.log("update -> firstname", req.user.id)
        if (req.user.type == "root") {
            db.User.findOne({ where: { id: req.user.id } })
                .then((user) => {
                    if (user) {
                        new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: { firstname: user.firstName, lastname: user.lastName }, newData: req.body, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} updated personal information`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                        return db.User.update({ firstName: firstname, lastName: lastname }, { where: { id: user.id } })
                    }
                })
                .then((success) => {
                    return res.status(200).json({ 'success': true });
                })
                .catch(err => {
                    next(err);
                })
        } else next(new RequestError("You don\'t have permission to change'"));
    },

    async changePassword(req, res, next) {
        var isMatch = await req.user.comparePassword(req.body.oldpassword)
        if (isMatch) {
            var hash = bcrypt.hashSync(req.body.password);
            db.User.update({ password: hash }, { where: { id: req.user.id } });
            new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} changed the password`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            return res.status(200).json({ success: true });
        } else {
            console.log("current password did not match")
            new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to change the password but failed - old password didn't matched`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            next(new RequestError('Current Password did not Match!'));
        }
    },
    async setExchange(req, res, next) {
        console.log("Inside setExchange", JSON.parse(JSON.stringify(req.user)));
        db.MetaAuth.findAll({ where: { companyId: req.user.companyId, $or: [{ key: "exchangeId" }, { key: "exchangePassword" }, { key: "exchangeBackupId" }, { key: "exchangeMailFolder" }] } })
            .then(r => {
                console.log("req.body.epass", req.body);
                console.log("req.user.companyId", req.user.companyId);
                console.log("data found", r);
                if (req.body.epass == "") {
                    if (r) {
                        new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} removed Exchange password`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                        return db.MetaAuth.destroy({ where: { companyId: req.user.companyId, $or: [{ key: "userName" }, { key: "targetMailbox" }, { key: "targetFolder" }] } })
                    } else return true;
                } else {
                    if (r) return db.MetaAuth.destroy({ where: { companyId: req.user.companyId, $or: [{ key: "userName" }, { key: "password" }, { key: "targetMailbox" }, { key: "targetFolder" }] } })
                    else return true;
                }
            })
            .then(r => {
                if (req.body.epass == "") {
                    var data = [];
                    data.push({ key: "userName", value: req.body.eId, companyId: req.user.companyId })
                    data.push({ key: "targetMailbox", value: req.body.eBackupId, companyId: req.user.companyId })
                    data.push({ key: "targetFolder", value: req.body.eMailFolder, companyId: req.user.companyId })
                    return db.MetaAuth.bulkCreate(data);
                } else {
                    var bf = new Blowfish(toString(process.env.APP_SECRET), Blowfish.MODE.ECB, Blowfish.PADDING.NULL);
                    const fromHexString = hexString => new Uint8Array(hexString.match(/.{1,2}/g).map(byte => parseInt(byte, 16)));
                    const toHexString = bytes => bytes.reduce((str, byte) => str + byte.toString(16).padStart(2, '0'), '');
                    var encoded = bf.encode(req.body.epass);
                    var password = toHexString(encoded);

                    var data = [];
                    data.push({ key: "userName", value: req.body.eId, companyId: req.user.companyId })
                    data.push({ key: "password", value: password, companyId: req.user.companyId })
                    data.push({ key: "targetMailbox", value: req.body.eBackupId, companyId: req.user.companyId })
                    data.push({ key: "targetFolder", value: req.body.eMailFolder, companyId: req.user.companyId })
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} has updated the Exchange credentials`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    return db.MetaAuth.bulkCreate(data);
                }
            })
            .then(r => {
                return res.status(200).json({ success: 1 });
            })
            .catch(err => {
                console.log(err);
                next(err);
                new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried updating the Exchange credentials (Some error occured)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            })
    },
    async getExchange(req, res, next) {
        console.log("Inside getExchange")
        db.MetaAuth.findAll({ where: { companyId: req.user.companyId, $or: [{ key: "userName" }, { key: "password" }, { key: "targetMailbox" }, { key: { like: '%socEmailAlert%' } }, { key: "targetFolder" }, { key: "hackButton" }, { key: "spf" }, { key: "dkim" }, { key: "content" }, { key: "dkimalign" }, { key: "spfalign" }, { key: "gctx" }, { key: "reportedby" }, { key: "country" }, { key: "dmarc" }, { key: "outlookAccessToken" }, { key: "outlookAdmin" }, {key: "attachments"}, {key: "score"}] } })
            .then(result => {
                if (result && result.length) {
                    return res.status(200).json({ success: 1, data: result, twoFA: req.user.twoFA })
                }
                return res.status(200).json({ success: 0, data: null })
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async fetchExchange(req, res, next) {
        db.Company.findOne({ where: { alias: req.query.companyCode }, include: [{ model: db.MetaAuth, required: true }] })
            .then(r => {
                if (r) {
                    var x = {};
                    for (var i = 0; i < r.MetaAuths.length; i++) {
                        if (r.MetaAuths[i].key == "password") {
                            var password = bf.decode(fromHexString(r.MetaAuths[i].value));
                            x[r.MetaAuths[i].key] = password;
                        } else x[r.MetaAuths[i].key] = r.MetaAuths[i].value;
                    }
                    return res.status(200).json({ success: 1, data: x })
                }
                return res.status(200).json({ success: 0 })
            })
            .catch(err => {
                console.log(err)
                next(err);
            })
    },
    async metaAuth_old(req, res, next) {
        console.log("Request for metaAuth", JSON.parse(JSON.stringify(req.user)));
        console.log("MetaAuths are", JSON.parse(JSON.stringify(req.user)).Company.MetaAuths)
        var companyId = req.user.companyId
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(r => {
                if (r) return db.MetaAuth.findOne({ where: { key: "socEmailAlert0", companyId: r.id } })
                else throw new Error("Coudnt find company with this company code")
            })
            .then((success) => {
                if (success) {
                    for (let i = 0; i < Object.keys(req.body).length; i++) {
                        db.MetaAuth.update({ value: Object.values(req.body)[i] }, { where: { companyId: success.companyId, key: "socEmailAlert" + i } })
                    }
                    return true
                } else {
                    for (let i = 0; i < Object.keys(req.body).length; i++) {
                        db.MetaAuth.create({ companyId: companyId, key: "socEmailAlert" + i, value: Object.values(req.body)[i] })
                    }
                    return true
                }
            })
            .then(r => {
                if (r) return res.status(200).json({ 'success': "SOC team email ID updated!" });
                else throw new Error("Coudnt Update SOC team Email ID!")
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async metaAuth(req, res, next) {
        console.log("Request for metaAuth", JSON.parse(JSON.stringify(req.user)));
        console.log("MetaAuths are", JSON.parse(JSON.stringify(req.user)).Company.MetaAuths);
        console.log("Request body is", JSON.parse(JSON.stringify(req.body)))
        db.Company.findOne({ where: { id: req.user.companyId } })
            .then(async(r) => {
                if (r) {
                    for (let i = 0; i < Object.keys(req.body).length; i++) {
                        await insertUpdateMetaAuth(req, i);
                    }
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: req.user.Company.MetaAuths.filter(r => r.key == "socEmailAlert1" || r.key == "socEmailAlert2" || r.key == "socEmailAlert3"), newData: req.body, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} updated SOC team Email(s)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    return true
                } else throw new Error("Coudnt Update SOC team Email ID!")
            })
            .then(r => {
                if (r) return res.status(200).json({ 'success': "SOC team email ID updated!" });
                else throw new Error("Coudnt Update SOC team Email ID!")
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async updateImap(req, res, next) {
        console.log("Inside updateImap", JSON.parse(JSON.stringify(req.body)), JSON.parse(JSON.stringify(req.user)));
        db.Company.findOne({ where: { alias: req.user.Company.alias } })
            .then(r => {
                if (r) return db.MetaAuth.findAll({ where: { companyId: r.id } }).then(result => [result, r])
                else throw new Error("Coudnt find any company ID for MetaAuth")
            })
            .then(([result, r]) => {
                var bf = new Blowfish(toString(process.env.APP_SECRET), Blowfish.MODE.ECB, Blowfish.PADDING.NULL);
                const fromHexString = hexString => new Uint8Array(hexString.match(/.{1,2}/g).map(byte => parseInt(byte, 16)));
                const toHexString = bytes => bytes.reduce((str, byte) => str + byte.toString(16).padStart(2, '0'), '');
                var encoded = bf.encode(req.body.imap_password);
                var password = toHexString(encoded);
                if (result && !(result.length == 0)) {
                    return Object.keys(req.body).reduce((p, value) => {
                        if (value == 'imap_password') return db.MetaAuth.update({ value: password }, { where: { companyId: r.id, key: value } })
                        else return db.MetaAuth.update({ value: req.body[value] }, { where: { companyId: r.id, key: value } })
                    }, Promise.resolve())
                } else {
                    return Object.keys(req.body).reduce((p, value) => {
                        if (value == 'imap_password') return db.MetaAuth.create({ companyId: r.id, key: value, value: password })
                        return db.MetaAuth.create({ companyId: r.id, key: value, value: req.body[value] })
                    }, Promise.resolve())
                }
            })
            .then(r => {
                if (r) {
                    new Api().enableImap({ companyCode: req.user.Company.alias })
                        .then(r => {
                            res.status(200).json({ success: true })
                        })
                        .catch(err => {
                            console.log(err);
                        })
                } else throw new Error("Coudnt not update IMAP setting in MetaAuth")
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },

    async findImapEnabled(req, res, next) {
        db.Company.findAll({ attributes: ['alias'], include: [{ model: db.MetaAuth, where: { key: 'imap', value: true }, required: true }] })
            .then(r => {
                if (r) {
                    var imapIsEnabled = [];
                    for (var i = 0; i < r.length; i++) {
                        imapIsEnabled.push(r[i].alias)
                    }
                    return res.status(200).json({ imapIsEnabled: imapIsEnabled });
                } else throw new RequestError("Coudnt find Any comany with imap enabled")
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },

    async findImapConfig(req, res, next) {
        db.Company.findOne({ where: { alias: req.query.companyCode }, include: [{ model: db.MetaAuth, required: true }] })
            .then(r => {
                if (r) { res.status(200).json({ config: r.MetaAuths }) } else throw new RequestError("There is no imap for this company ID")
            })
            .catch(err => {
                console.log(err)
                next(err);
            })
    },

    async disableImap(req, res, next) {
        db.Company.findOne({ where: { alias: req.user.Company.alias } })
            .then(r => {
                if (r.id) return db.MetaAuth.update({ value: 0 }, { where: { companyId: r.id, key: 'imap' } })
                else throw new Error("Coundt find company ID for the User")
            })
            .then(r => {
                if (r) {
                    new Api().disableImap({ companyCode: req.user.Company.alias })
                        .then(r => {
                            res.status(200).json({ success: true })
                        })
                        .catch(err => {
                            console.log(err);
                        })
                } else throw new RequestError("coudnt update MetaAuth")
            })
            .catch(err => {
                console.log(err)
                next(err);
            })
    },

    async checkConfig_old(req, res, next) {
        var domain = req.query.to.split("@")[1]
        var data = {}
        db.MetaAuth.findAll({ where: { value: domain } })
            .then(r => {
                if (r.length) return db.MetaAuth.findAll({ where: { companyId: r[0].companyId } })
                else { data['basic'] = true; return data; }
            })
            .then(r => {
                if (!r['basic']) {
                    return new Promise((resolve, reject) => {
                        for (var i = 0; i < r.length; i++) {
                            if (r[i].key == 'spf') data['spf'] = r[i].value;
                            if (r[i].key == 'dkim') data['dkim'] = r[i].value;
                            if (r[i].key == 'dmarc') data['dmarc'] = r[i].value;
                            if (r[i].key == 'content') data['content'] = r[i].value;
                            if (r[i].key == 'dkimalign') data['dkimalign'] = r[i].value;
                            if (r[i].key == 'spfalign') data['spfalign'] = r[i].value;
                            if (r[i].key == 'gctx') data['gctx'] = r[i].value;
                            if (r[i].key == 'reportedby') data['reportedby'] = r[i].value;
                            if (r[i].key == 'country') data['country'] = r[i].value;
                            if (r[i].key == 'hackbutton') data['hackbutton'] = r[i].value;
                        }
                        if (!r['basic']) resolve(data)
                        else reject("Coudnt find data from MetaAuths for COnfiguration of Report Button!")
                    })
                } else return data
            })
            .then(r => {
                if (r) res.status(200).json({ data: data })
                else throw new Error("Coudnt find Any data from MetaAuth for Report Button Config")
            })
            .catch(err => {
                console.log(err)
                next(err)
            })
    },

    async checkConfig(req, res, next) {
        var companyAlias = req.query.to; //.split("@")[1]
        console.log("CompanyAlias", companyAlias);
        var data = {}
        db.Company.findOne({ where: { alias: companyAlias } })
            .then(r => {
                if (r) return db.MetaAuth.findAll({ where: { companyId: r.id } })
                else return data['basic'] = true
            })
            .then(r => {
                console.log("R is", JSON.parse(JSON.stringify(r)));
                if (!r['basic']) {
                    return new Promise((resolve, reject) => {
                        for (var i = 0; i < r.length; i++) {
                            if (r[i].key == 'spf') data['spf'] = r[i].value;
                            if (r[i].key == 'dkim') data['dkim'] = r[i].value;
                            if (r[i].key == 'dmarc') data['dmarc'] = r[i].value;
                            if (r[i].key == 'content') data['content'] = r[i].value;
                            if (r[i].key == 'dkimalign') data['dkimalign'] = r[i].value;
                            if (r[i].key == 'spfalign') data['spfalign'] = r[i].value;
                            if (r[i].key == 'gctx') data['gctx'] = r[i].value;
                            if (r[i].key == 'reportedby') data['reportedby'] = r[i].value;
                            if (r[i].key == 'country') data['country'] = r[i].value;
                            if (r[i].key == 'hackButton') data['hackbutton'] = r[i].value;
                            if (r[i].key == 'attachments') data['attachments'] = r[i].value;
                            if (r[i].key == 'score') data['score'] = r[i].value;
                        }
                        if (!r['basic']) resolve(data)
                        else reject("Coudnt find data from MetaAuths for COnfiguration of Report Button!")
                    })
                } else return data
            })
            .then(r => {
                if (r) res.status(200).json({ data: data })
                else throw new Error("Coudnt find Any data from MetaAuth for Report Button Config")
            })
            .catch(err => {
                console.log(err)
                next(err)
            })
    },

    async reportButtonConfig(req, res, next) {
        console.log("Inside reportButtonConfig", req.body, JSON.parse(JSON.stringify(req.user)));
        db.Company.findOne({ where: { alias: req.user.Company.alias } }).then(r => {
                if (r) return db.MetaAuth.findAll({
                    where: {
                        companyId: req.user.companyId,
                        $or: [{ key: "dmarc" }, { key: "dkim" }, { key: "spf" }, { key: "content" }, { key: "hackButton" }, { key: "dkimalign" }, { key: "spfalign" }, { key: "gctx" }, { key: "reportedby" }, { key: "country" }, {key : "attachments"}, {key : "score"}]
                    }
                })
                else throw new Error("Coudnt find company!")
            }).then(r => {
                console.log("R is",JSON.parse(JSON.stringify(r)));
                if (r.length) {
                    return new Promise((resolve, reject) => {
                        for (const [key, value] of Object.entries(req.body)) {
                            db.MetaAuth.update({ value: `${value}` }, { where: { key: `${key}`, companyId: req.user.companyId } })
                        }
                        resolve(r)
                    })
                } else {
                    return new Promise((resolve, reject) => {
                        for (const [key, value] of Object.entries(req.body)) {
                            db.MetaAuth.create({ value: `${value}`, key: `${key}`, companyId: req.user.companyId })
                        }
                        resolve(r)
                    })
                }
            }).then(r => {
                if (r) {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: r, newData: req.body, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} has updated report configs`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    res.status(200).json({ success: true })
                } else {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to update report configs (Some error occured)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    throw new Error("Coudnt find anything in MetaAuths")
                }
            })
            .catch(err => {
                console.log(err)
                next(err)
                new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to update report configs (Cannot find existing configs)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            })
    },

    async fetchSocIds(req, res, next) {
        db.Company.findOne({ where: { alias: req.query.companyCode } })
            .then(r => {
                if (r) return db.MetaAuth.findAll({ attributes: ['value'], where: { companyId: r.id, key: { like: '%socEmailAlert%' } } })
                else throw new Error("Coudnt find Company with this company ID")
            })
            .then(r => {
                if (r) res.status(200).json(r)
                else throw new Error("Coudnt find SOc Team Email IDS")
            })
            .catch(err => {
                console.log(err)
            })
    },
    async outlookSignin(req, res, next) {
        req.user = JSON.parse(JSON.stringify(req.user));
        console.log("Inside outlookSignin", req.params, req.user.Company.id);
        // db.MetaAuths.create({
        //     companyId : req.user.Company.id,
        //     key : 'outlook-signin-status',
        //     value : 'InProgress',

        // })
    },
    async generateGoogleAuthCode(req, res, next) {
        req.user = JSON.parse(JSON.stringify(req.user));
        console.log("User is", req.user.email);
        let authUrl = await generateGoogleAuthenticatorUrl(req.user.email);
        let authUrlQr = await generateQr(authUrl);
        res.send({ qrCode: authUrlQr })
    },
    async verifyTwoFa(req, res, next) {
        console.log("Inside verifyTwoFa", req.body, req.user.email);
        if (req.body.otp) {
            client.get(req.user.email + ":" + "g-auth", async(err, reply) => {
                console.log("data from redis", reply);
                if (err)
                    throw new RequestError('Something went wrong!');
                else {
                    reply = JSON.parse(reply).key;
                    console.log("Reply", reply);
                    var recoveryCodes = await generateRecoveryCodes();
                    if (recoveryCodes.length)
                        await saveRecoveryCodes(req.user.email, JSON.stringify(recoveryCodes))
                    console.log("recoveryCodes", recoveryCodes);
                    var formattedKeyDecrypted = bf.decode(fromHexString(reply));
                    console.log("formattedKeyDecrypted", formattedKeyDecrypted);
                    var verify = formattedKeyDecrypted ? (authenticator.verifyToken(formattedKeyDecrypted, req.body.otp) ? 200 : 401) : 401;
                    console.log("verify", verify);
                    if (Number(verify) == 200)
                        db.User.update({ twoFA: 2 }, {
                            where: { id: req.user.id }
                        }).then(r => {
                            new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} has activated google otp 2FA`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                        }).catch(err => console.log("Error occured while updating 2FA google auth", err));
                    res.send({ status: verify, message: "2FA enabled successfully", recoveryCodes: (Number(verify) == 200 ? recoveryCodes : []) })
                }
            })
        } else if (req.body.emailOtp) {
            return db.User.update({ twoFA: 1 }, {
                    where: { id: req.user.id }
                })
                .then(r => {
                    console.log("Mail TwoFA Updated");
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} has activated mail 2FA`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    res.send({ status: 200, message: "2FA enabled successfully" })
                })
                .catch(err => {
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} tried to activate mail 2FA but failed (Some error occured)`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    console.log("Error occured while updating 2FA mail otp", err)
                    res.send({ status: 500 })
                });
        } else {
            return db.User.update({ twoFA: 0 }, {
                    where: { id: req.user.id }
                })
                .then(r => {
                    console.log("Mail TwoFA removed");
                    new Audit().saveAuditLog({ email: req.user.email, userId: req.user.id, companyId: req.user.Company.alias, EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `user ${req.user.email} deactivated 2FA`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
                    res.send({ status: 200, message: "2FA disabled successfully" })
                })
                .catch(err => {
                    console.log("Error occured while updating 2FA", err)
                    res.send({ status: 500, message: "Something went wrong!" })
                });
        }
    }
};