import express from 'express';
import settingController from './setting.controller';
import { sanitize } from '../../../middleware/sanitizer';
import { jwtStrategy } from '../../../middleware/strategy';
import { validateBody, schemas } from '../../../middleware/validator';
import { userPermission } from '../../../middleware/userPermission';
import { userImplant } from '../../../middleware/auth';

// import {validateBody, schemas} from '../../../helpers/routeHelper';

export const settingRouter = express.Router();
settingRouter.route('/').get(sanitize(), jwtStrategy, validateBody(), settingController.index);
settingRouter.route('/profile').patch(jwtStrategy, validateBody(schemas.profileUpdateSchemas), userPermission('setting-tab-update-information-wrte'), settingController.update);
settingRouter.route('/password').patch(jwtStrategy, userPermission('setting-tab-change-password-write'), settingController.changePassword);
settingRouter.route('/company').get(sanitize(), settingController.checkCompany);
settingRouter.route('/generate').post(sanitize(), jwtStrategy, settingController.generateKey);
settingRouter.route('/downloadpem').get(sanitize(), jwtStrategy, settingController.downloadPem);
settingRouter.route('/exchange').post(sanitize(), jwtStrategy, userPermission('setting-tab-exchange-write'), settingController.setExchange);
settingRouter.route('/exchange').get(sanitize(), jwtStrategy, userPermission('setting-tab-exchange-write'), settingController.getExchange);
settingRouter.route('/exchangeMetaAuth').get(sanitize(), settingController.fetchExchange);
settingRouter.route('/metaAuth').post(sanitize(), jwtStrategy, userImplant, settingController.metaAuth);
settingRouter.route('/updateImap').post(sanitize(), jwtStrategy, validateBody(schemas.updateImapSchemas), userPermission('setting-tab-exchange-write'), userImplant, settingController.updateImap);
settingRouter.route('/findImapEnabled').get(sanitize(), settingController.findImapEnabled);
settingRouter.route('/findImapConfig').get(sanitize(), settingController.findImapConfig);
settingRouter.route('/disableImap').post(sanitize(), jwtStrategy, userPermission('setting-tab-exchange-write'), userImplant, settingController.disableImap);
settingRouter.route('/checkConfig').get(sanitize(), settingController.checkConfig);
settingRouter.route('/reportButtonConfig').post(sanitize(), jwtStrategy, userPermission('setting-tab-exchange-write'), userImplant, settingController.reportButtonConfig);
settingRouter.route('/fetchSocIds').get(sanitize(), settingController.fetchSocIds);
settingRouter.route('/outlook-login').get(sanitize(), jwtStrategy, settingController.outlookSignin);
settingRouter.route('/generate-google-auth-code').post(sanitize(), jwtStrategy, settingController.generateGoogleAuthCode);
settingRouter.route('/verify-two-fa').post(sanitize(), jwtStrategy, settingController.verifyTwoFa);