import passport from 'passport';
import JWT from 'jsonwebtoken';
import config from '../config';
import Audit from '../modules/Audit';
var parser = require('ua-parser-js');

var JWTSign = function(user, date) {
    return JWT.sign({
        iss: config.app.name,
        sub: user.id,
        iam: user.type,
        iat: date.getTime(),
        exp: new Date().setMinutes(date.getMinutes() + 30)
    }, config.app.secret);
}

var TokenExtractor = function(req) {
    try {
        if (req.query.token && req.query) {
            let jwt = JWT.decode(req.query.token, { complete: true });
            if (jwt && jwt.payload) return true
            else return null;
        } else return null;
    } catch (err) {
        console.log('Error from token extractor for ThreatCop: ', err);
        return null;
    }
}

export var jwtStrategy = (req, res, next) => {
    passport.authenticate('jwt', { session: false }, (err, user, info) => {
        // console.log("err",err);
        // console.log("user",user);
        // console.log("info",info);
        let contype = req.headers['content-type'];
        var json = !(!contype || contype.indexOf('application/json') !== 0);
        if (!json && info && typeof info === 'string' && info.includes("isNewUser")) {
            if (req.url != '/change-password/' && req.url != '/order/success') {
                return res.redirect('/change-password/');
            }
        }
        if (err && err == 'expired') { return json ? res.status(419).json({ errors: ['Session is expired'] }) : res.redirect('/logout'); }
        if (err && err == 'invalid') { return json ? res.status(500).json({ errors: ['Invalid token recieved'] }) : res.redirect('/logout'); }
        if (err && err == 'user') { return json ? res.status(500).json({ errors: ['Invalid user recieved'] }) : res.redirect('/logout'); }
        if (err && Object.keys(err).length) { return res.status(500).json({ errors: [err] }); }
        if (err) { return res.status(403).json({ errors: ['Invalid user recieved'] }); }
        if (!user) { return json ? res.status(403).json({ errors: ['Session is Expired'] }) : res.redirect('/logout'); }

        //Update Token
        var date = new Date();
        var token = JWTSign(user, date);
        res.cookie('XSRF-token-tab', token, {
            expire: new Date().setMinutes(date.getMinutes() + 30),
            httpOnly: true,
            secure: config.app.secure
        });

        req.user = user;
        // console.log("user is", JSON.parse(JSON.stringify(user)));
        next();
    })(req, res, next);
};

export var jwtLogoutStrategy = (req, res, next) => {
    passport.authenticate('jwt', { session: false }, (err, user, info) => {
        if (user) {
            req.user = user;
        }
        next();
    })(req, res, next);
};

export var localStrategy = (req, res, next) => {
    passport.authenticate('local', { session: false }, (err, user, info) => {
        if (err && err.startsWith("isNewUser")) { return res.status(401).json({ errors: ['You have not created a password for your accout. Please contact your Admin to reset the password'] }); }
        if (err && err == 'invalid') { return res.status(401).json({ errors: ['Email Id not verified'] }); }
        if (err && err == 'attempt') { return res.status(401).json({ errors: ['Too many invalid attempts. Please reset your password.'] }); }
        if (err && err.startsWith('attempt:')) { return res.status(401).json({ errors: ['Invalid Credentials (' + err.split(':')[1] + ' Attempt(s) Left)'] }); }
        if (err) { return res.status(401).json({ errors: [err] }); }
        if (!user) { return res.status(401).json({ errors: ['Invalid Credentials'] }); }
        user.type = "employee";
        req.user = user;
        next();
    })(req, res, next);
};

export var rootLoginStrategy = (req, res, next) => {
    passport.authenticate('root-login', {
        session: false
    }, async(err, user, info) => {
        // console.log("req.body.email", req.body.email, user)
        if (err && err.startsWith("isNewUser")) {
            new Audit().saveAuditLog({ email: req.body.email, userId: "", companyId: "", EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `anonymous user with ${req.body.email} - New User`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            var email = err.split("/");
            return res.status(200).json({ isnewuser: 1, email: email[1] });
        }
        if (err && err == 'invalid') {
            new Audit().saveAuditLog({ email: req.body.email, userId: "", companyId: "", EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `anonymous user with ${req.body.email} - email id not verified`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            return res.status(401).json({ errors: ['Email Id not verified'] });
        }
        if (err && err == 'attempt') {
            new Audit().saveAuditLog({ email: req.body.email, userId: "", companyId: "", EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `anonymous user with ${req.body.email} - password attempt limit crossed`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            return res.status(401).json({ errors: ['Too many invalid attempts. Please reset your password.'] });
        }
        if (err && err.startsWith('attempt:')) {
            new Audit().saveAuditLog({ email: req.body.email, userId: "", companyId: "", EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `anonymous user with ${req.body.email} - password attempt ${(5 - err.split(':')[1])}`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            return res.status(401).json({ errors: ['Invalid Credentials (' + err.split(':')[1] + ' Attempt(s) Left)'] });
        }
        if (err) {
            new Audit().saveAuditLog({ email: req.body.email, userId: "", companyId: "", EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `anonymous user with ${req.body.email} - some error occured`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            return res.status(401).json({ errors: [err] });
        }
        if (!user) {
            new Audit().saveAuditLog({ email: req.body.email, userId: "", companyId: "", EmployeeId: "", oldData: {}, newData: {}, metaData: parser(req.headers['user-agent']), activity: `anonymous user with ${req.body.email} - Invalid Credentials`, IP: req.headers['x-real-ip'] || req.connection.remoteAddress.split(`:`).pop() })
            return res.status(401).json({ errors: ['Invalid Credentials'] });
        }
        user.type = "root";
        req.user = user;
        next();
    })(req, res, next);
};

export var magicLinkStrategy = (req, res, next) => {
    passport.authenticate('magic-login', { session: false }, (err, user, info) => {
        if (err && err == 'notMatch') { return next(new RequestError('Invalid Token', 400)); }
        if (err) { return next(new RequestError('Server Error', 500, err)); }
        if (!user) { return next(new RequestError('Invalid User Recieved', 400, err)); }

        var date = new Date();
        var token = JWTSign(user, date);
        res.cookie('XSRF-token-tab', token, {
            expire: new Date().setMinutes(date.getMinutes() + 30),
            httpOnly: true,
            secure: config.app.secure
        });
        user.type = "root";
        req.user = user;
        next();
    })(req, res, next);
};

export var otpLoginStrategy = (req, res, next) => {
    passport.authenticate('otp-login', { session: false }, (err, user, info) => {

        // if (err && err == 'invalid') {return res.status(401).json({ errors: ['Email Id not verified']}); }
        if (err && err == 'attempt') { return res.status(401).json({ errors: ['Too many invalid attempts. Please reset your password.'] }); }
        if (err && err.startsWith('attempt:')) { return res.status(401).json({ errors: ['Invalid Credentials (' + err.split(':')[1] + ' Attempt(s) Left)'] }); }
        if (err && err == 'expired') { return res.status(401).json({ errors: ['OTP expired.'] }); }
        if (err) { return res.status(401).json({ errors: [err] }); }
        if (!user) { return res.status(401).json({ errors: ['Invalid Credentials'] }); }
        user.type = "root";
        var date = new Date();
        var token = JWTSign(req.user, date);
        res.cookie('XSRF-token-tab', token, {
            expire: new Date().setMinutes(date.getMinutes() + 30),
            httpOnly: true,
            secure: config.app.secure
        });
        req.user = user;
        next();
    })(req, res, next);
};

export var employeeMagicStrategy = (req, res, next) => {
    passport.authenticate('Employee-magic-login', { session: false }, (err, user, info) => {
        if (err && err == 'notMatch') { return next(new RequestError('Invalid Token', 400)); }
        if (err) { return res.redirect('/login?alias=' + req.params.alias); /*next(new RequestError('Server Error', 500, err));*/ }
        if (!user) { return next(new RequestError('Invalid User Recieved', 400, err)); }

        var date = new Date();
        var token = JWTSign(user, date);
        res.cookie('XSRF-token-tab', token, {
            expire: new Date().setMinutes(date.getMinutes() + 30),
            httpOnly: true,
            secure: config.app.secure
        });
        req.user = user;
        next();
    })(req, res, next);
};

export var employeeResetPasswordMagicStrategy = (req, res, next) => {
    passport.authenticate('Employee-reset-password-magic-login', { session: false }, (err, user, info) => {
        if (err && err == 'notMatch') { return next(new RequestError('Invalid Token', 400)); }
        if (err) { return next(new RequestError('Server Error', 500, err)); }
        if (!user) { return next(new RequestError('Invalid User Recieved', 400, err)); }

        var date = new Date();
        var token = JWTSign(user, date);
        res.cookie('XSRF-token-tab', token, {
            expire: new Date().setMinutes(date.getMinutes() + 30),
            httpOnly: true,
            secure: config.app.secure
        });
        req.user = user;
        next();
    })(req, res, next);
};

export var jwtStrategyThreacop = (req, res, next) => {
    passport.authenticate('jwt-threatcop', { session: false }, (err, meta, info) => {
        if (err && err == 'invalid-session') { return res.status(401).json({ errors: ['Invalid key received.'] }); }
        if (err && err == 'invalid-user') { return res.status(401).json({ errors: ['Invalid user received.'] }); }
        if (err && err == 'error-key') { return res.status(401).json({ errors: ['Invalid key received.'] }); }
        if (err) { return res.status(401).json({ errors: [err] }); }

        var date = new Date();
        meta.type = 'root'
        var token = JWTSign(meta, date);
        if (req.query.token) {
            var verifyToken = TokenExtractor(req);
            if (verifyToken) {
                res.cookie('XSRF-token-tab', token, {
                    expire: new Date().setMinutes(date.getMinutes() + 30),
                    httpOnly: true,
                    secure: config.app.secure
                });
            }
        }
        req.user = meta;
        next();
    })(req, res, next);
};