import BaseJoi from 'joi';
import DateExtension from 'joi-date-extensions';
const Joi = BaseJoi.extend(DateExtension);

export var validateBody = (schema) => {
    // console.log("Inside validateBody");
    return (req, res, next) => {
        const result = req.method != 'GET' ? Joi.validate(req.body, schema) : Joi.validate(req.query, schema);
        if (result.error) {
            return res.status(400).json(result.error);
        }

        if (!req.value) {
            req.value = {};
        }
        req.value['body'] = result.value;
        next();
    }
}

export var schemas = {
    registerSchema: Joi.object().keys({
        firstName: Joi.string().required(),
        lastName: Joi.string().required(),
        email: Joi.string().email().required(),
        password: Joi.string().regex(/^(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[@#$%]).{8,20}$/).min(8).required().error((err) => {
            return 'Password has to be between 8 to 15 characters which contain at least one numeric digit, one uppercase and one lowercase letter'
        }),
    }),

    advanceScanSchemas: Joi.object().keys({
        senderID: Joi.string().email().required(),
        startDate: Joi.string().required(),
        endDate: Joi.string().required(),
        senderIP: Joi.string().optional(),
        mailSubject: Joi.string().optional(),
    }),

    updateImapSchemas: Joi.object().keys({
        imap: Joi.boolean().required(),
        imap_userID: Joi.string().regex(/^([a-zA-Z0-9_\-\.]+)@([a-zA-Z0-9_\-\.]+)\.([a-zA-Z]{2,5})$/).required().error((err) => {
            return 'Please Enter a valid Email'
        }),
        imap_password: Joi.string().required(),
        imap_host: Joi.string().required(),
        imap_port: Joi.number().integer().required(),
        imap_tls: Joi.boolean().required(),
    }),

    profileUpdateSchemas: Joi.object().keys({
        firstname: Joi.string().regex(/^[a-zA-Z0-9]+$/).required().error((err) => {
            return 'Firstname cannot contain Special character or spaces!'
        }),
        lastname: Joi.string().regex(/^[a-zA-Z0-9]+$/).required().error((err) => {
            return 'Lastname cannot contain Special character or spaces!'
        }),
    }),

    otp: Joi.object().keys({
        otp: Joi.string().error((err) => {
            return 'Please Enter OTP!'
        }),
        email: Joi.string().email().required(),
        password: Joi.string(),
        recovery_code: Joi.boolean().optional(),
    }),

    check2FA: Joi.object().keys({
        email: Joi.string().email().required(),
    }),

    loginSchema: Joi.object().keys({
        email: Joi.string().email().required(),
        password: Joi.string().required(),
        alias: Joi.string().required(),
    }),
    userCheckSchema: Joi.object().keys({
        email: Joi.string().email().required(),
    }),
    rootLoginSchema: Joi.object().keys({
        email: Joi.string().email().required(),
        password: Joi.string().required(),
        otp: Joi.string(),
        recovery_code: Joi.boolean().optional(),
    }),
    rootCheckSchema: Joi.object().keys({
        // email: Joi.string().email().optional().allow(null).empty(''),
        // alias: Joi.string().optional().allow(null).empty(''),
    }),
    resetPassword: Joi.object().keys({
        email: Joi.string().email().required(),
        verificationCode: Joi.string().required(),
        password: Joi.string().regex(/^(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[@#$%]).{8,20}$/).min(8).required().error((err) => {
            return 'Password has to be between 8 to 15 characters which contain at least one numeric digit, one uppercase and one lowercase letter'
        }),
    }),
    userProfile: Joi.object().keys({
        firstname: Joi.string().required(),
        lastname: Joi.string().required(),
    }),
    sendResetPassword: Joi.object().keys({
        email: Joi.string().regex(/^([a-zA-Z0-9_\-\.]+)@([a-zA-Z0-9_\-\.]+)\.([a-zA-Z]{2,5})$/).required().error((err) => {
            return 'Please Enter a valid Email'
        }),
    }),
    updatePolicy: Joi.object().keys({
        policyName: Joi.string().regex(/^[a-zA-Z0-9]{4,15}$/).required().error((err) => {
            return 'Please Enter a valid PolicyName.'
        }),
        description: Joi.string().required(),
        action: Joi.array().optional().allow(null).empty(''),
        policyId: Joi.number().integer().required()
    }),

    createPolicy: Joi.object().keys({
        policyName: Joi.string().regex(/^[a-zA-Z0-9]{4,15}$/).required().error((err) => {
            return 'Please Enter a valid PolicyName.'
        }),
        description: Joi.string().required(),
        action: Joi.array().optional().allow(null).empty(''),
    }),


    reportMail: Joi.object().keys({
        companyId: Joi.string().required(),
        reportType: Joi.string().required(),
        filename: Joi.string().required(),
        buffer: Joi.string().required()
    }),
    domainCreate: Joi.object().keys({
        domain: Joi.string().regex(/^((([a-zA-Z]{1,2})|([0-9]{1,2})|([a-zA-Z0-9]{1,2})|([a-zA-Z0-9][a-zA-Z0-9-]{1,61}[a-zA-Z0-9]))\.)+[a-zA-Z]{2,6}$/).required().error((err) => {
            return 'Enter a valid Domain Name'
        }),
    }),
    showTxt: Joi.object().keys({
        key: Joi.string().required(),
        domainName: Joi.string().required()
    }),
    domainVerify: Joi.object().keys({
        domainId: Joi.number().required(),
    }),

    employeePolicyUpdateSchema: Joi.object().keys({
        list: Joi.array().required(),
    }),
    addUserSchema: Joi.object().keys({
        firstName: Joi.string().required().regex(/^[a-zA-Z0-9]{4,20}$/).required().error((err) => {
            return 'Please Enter a valid First Name.'
        }),
        lastName: Joi.string().required().regex(/^[a-zA-Z0-9]{4,20}$/).required().error((err) => {
            return 'Please Enter a valid Last Name.'
        }),
        email: Joi.string().email().required().regex(/^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$/).required().error((err) => {
            return 'Please Enter a valid Email ID.'
        }),
        phone: Joi.string().optional().allow(null).empty(''),
        domainCode: Joi.string().optional().allow(null).empty(''),
    }),
    editUserSchema: Joi.object().keys({
        firstName: Joi.string().required(),
        lastName: Joi.string().optional().allow(null).empty(''),
        email: Joi.string().email().required(),
        phone: Joi.string().optional().allow(null).empty(''),
    }),
    otpSchema: Joi.object().keys({
        email: Joi.string().email().required(),
    }),
    otpVerfiySchema: Joi.object().keys({
        otp: Joi.string().required(),
        email: Joi.string().required(),
    }),
    orderSchema: Joi.object().keys({
        priceId: Joi.number().required(),
        packageId: Joi.number().required(),
        countryCode: Joi.string().optional().allow(null).empty(''),
        type: Joi.string().required()
    }),
    paySchema: Joi.object().keys({
        razorpay_order_id: Joi.string().required(),
        razorpay_payment_id: Joi.string().required(),
        razorpay_signature: Joi.string().required(),
        type: Joi.string().required(),
        name: Joi.string().optional().allow(null).empty(''),
        address: Joi.string().optional().allow(null).empty(''),
        gst: Joi.string().optional().allow(null).empty(''),
        priceId: Joi.number().optional().allow(null).empty(''),
        shipping: Joi.string().optional().allow(null).empty('')
    }),
    changePassword: Joi.object().keys({
        newPass: Joi.string().regex(/^(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[@#$%]).{8,20}$/).min(8).required().error((err) => {
            return 'Password has to be between 8 to 15 characters which contain at least one numeric digit, one uppercase and one lowercase letter'
        }),
        confirmPass: Joi.string().valid(Joi.ref('newPass')).required().error((err) => {
            return "Password does not match."
        }),
    }),
    otpLinkSchema: Joi.object().keys({
        email: Joi.string().email().required(),

    }),
    employeeResetPassword: Joi.object().keys({
        id: Joi.number().required(),
    }),



}