import { generateQueryString } from '../functions';
import request from 'request'
export default class API {

    constructor(baseUrl = null) {
        this.baseUrl = baseUrl;
        this.get = this.get.bind(this);
    }

    setBaseUrl(baseUrl) {
        this.baseUrl = baseUrl;
    }

    get(query = null, queryParams = {}) {
        if (query) {
            var url;
            if (query.url) url = query.url;
            if (!query.url && query.baseUrl) this.baseUrl = query.baseUrl;
            if (!query.url && query.path) {
                if (!this.baseUrl) throw new Error('Base URL not set.');
                url = this.baseUrl + query.path;
            }
        } else throw new Error('URL Not available')
        return new Promise((resolve, reject) => {
            request.get(url + '?' + generateQueryString(queryParams), (error, response, body) => {
                // console.log("response",response.statusCode);
                // console.log("body", body);
                console.log("url is", url + '?' + generateQueryString(queryParams))
                if (error) { return reject(error); }
                if (response && response.statusCode >= 400) {
                    try {
                        if (isJson(body) && JSON.parse(body) && JSON.parse(body).error) {
                            return reject(new RequestError(JSON.parse(body).error, response.statusCode));
                        } else if (body && body.error) {
                            return reject(new RequestError(body.error, response.statusCode));
                        } else {
                            return reject(new RequestError(body, response.statusCode));
                        }
                    } catch (err) {
                        return reject(err);
                    }
                } else if (response && body) {
                    resolve(isJson(body) ? JSON.parse(body) : body);
                } else {
                    return reject(new RequestError("Unexpected Response.."));
                }
            });
        });
    }
    post(query = null, data) {
        console.log("Post url to hit", query.url);
        if (query) {
            var url;
            if (query.url) url = query.url;
            if (!query.url && query.baseUrl) this.baseUrl = query.baseUrl;
            if (!query.url && query.path) {
                if (!this.baseUrl) throw new Error('Base URL not set.');
                url = this.baseUrl + query.path;
            }
        } else throw new Error('URL Not available')
        return new Promise((resolve, reject) => {
            request.post(url, { json: data }, (error, response, body) => {
                // console.log("url is", url)
                // console.log("response of post API", JSON.parse(JSON.stringify(response)));
                if (error) { return reject(error); }
                if (response && response.statusCode >= 400) {
                    try {
                        if (isJson(body) && JSON.parse(body) && JSON.parse(body).error) {
                            return reject(new RequestError(JSON.parse(body).error, response.statusCode));
                        } else if (body && body.error) {
                            return reject(new RequestError(body.error, response.statusCode));
                        } else {
                            return reject(new RequestError(body, response.statusCode));
                        }
                    } catch (err) {
                        return reject(err);
                    }
                } else if (response && body) {
                    console.log("isJson(body)", body)
                    resolve(isJson(body) ? JSON.parse(body) : body);
                } else {
                    return reject(new RequestError("Unexpected Response.."));
                }
            });
        });
    }
    patch(query = null, data) {
        if (query) {
            var url;
            if (query.url) url = query.url;
            if (!query.url && query.baseUrl) this.baseUrl = query.baseUrl;
            if (!query.url && query.path) {
                if (!this.baseUrl) throw new Error('Base URL not set.');
                url = this.baseUrl + query.path;
            }
        } else throw new Error('URL Not available')
        return new Promise((resolve, reject) => {
            request.patch(url, { json: data }, (error, response, body) => {
                if (error) { return reject(error); }
                if (response && response.statusCode >= 400) {
                    try {
                        if (isJson(body) && JSON.parse(body) && JSON.parse(body).error) {
                            return reject(new RequestError(JSON.parse(body).error, response.statusCode));
                        } else if (body && body.error) {
                            return reject(new RequestError(body.error, response.statusCode));
                        } else {
                            return reject(new RequestError(body, response.statusCode));
                        }
                    } catch (err) {
                        return reject(err);
                    }
                } else if (response && body) {
                    resolve(isJson(body) ? JSON.parse(body) : body);
                } else {
                    return reject(new RequestError("Unexpected Response.."));
                }
            });
        });
    }

}