console.log("Inside header-analysis.js");

function buildGraph(elem, value) {
    var options = {
        series: [value],
        chart: {
            type: 'radialBar',
            offsetY: -20,
            sparkline: {
                enabled: true
            }
        },
        plotOptions: {
            radialBar: {
                startAngle: -90,
                endAngle: 90,
                track: {
                    background: "#e7e7e7",
                    strokeWidth: '97%',
                    margin: 5, // margin is in pixels
                    dropShadow: {
                        enabled: true,
                        top: 2,
                        left: 0,
                        color: '#999',
                        opacity: 1,
                        blur: 2
                    }
                },
                dataLabels: {
                    name: {
                        show: false
                    },
                    value: {
                        offsetY: -2,
                        fontSize: '22px'
                    }
                }
            }
        },
        grid: {
            padding: {
                top: -10
            }
        },
        fill: {
            type: 'gradient',
            gradient: {
                shade: 'light',
                shadeIntensity: 0.4,
                inverseColors: false,
                opacityFrom: 1,
                opacityTo: 1,
                stops: [0, 50, 53, 91]
            },
        },
        labels: ['Average Results'],
    };

    var chart = new ApexCharts(document.querySelector(elem), options);

    chart.render();
}

function showAnimation() {
    $(".progress").each(function() {

        var $bar = $(this).find(".bar");
        var $valSpf = $(this).find("#spfScorePercent");
        var $valDkim = $(this).find("#dkimScorePercent");
        var $valDmarc = $(this).find("#dmarcScorePercent");
        var percSpf = parseInt($valSpf.text(), 10);
        var percDkim = parseInt($valDkim.text(), 10);
        var percDmarc = parseInt($valDmarc.text(), 10);

        // console.log("$valSpf", $valSpf.text())

        $({ p: 0 }).animate({ p: percSpf }, {
            duration: 1000,
            easing: "swing",
            step: function(p) {
                $bar.css({
                    transform: "rotate(" + (45 + (p * 1.8)) + "deg)", // 100%=180° so: ° = % * 1.8
                    // 45 is to add the needed rotation to have the green borders at the bottom
                });
                $valSpf.text(p | 0);
            }
        });
        $({ p: 0 }).animate({ p: percDkim }, {
            duration: 1000,
            easing: "swing",
            step: function(p) {
                $bar.css({
                    transform: "rotate(" + (45 + (p * 1.8)) + "deg)", // 100%=180° so: ° = % * 1.8
                    // 45 is to add the needed rotation to have the green borders at the bottom
                });
                $valDkim.text(p | 0);
            }
        });
        $({ p: 0 }).animate({ p: percDmarc }, {
            duration: 1000,
            easing: "swing",
            step: function(p) {
                $bar.css({
                    transform: "rotate(" + (45 + (p * 1.8)) + "deg)", // 100%=180° so: ° = % * 1.8
                    // 45 is to add the needed rotation to have the green borders at the bottom
                });
                $valDmarc.text(p | 0);
            }
        });
    });
}

var getHeaderFailedCounts = function() {
    api.aggregateReports.headerAnalysis({ startDate, endDate }, function(success) {
        console.log("Response got getHeaderFailedCounts", success);
        // buildGraph("#spfScore", ((success.spfFailCount / success.totalMails) * 100).toFixed(0));
        $("#spfScore").html((((success.spfFailCount / success.totalMails) * 100).toFixed(0) == "NaN" ? 0 : ((success.spfFailCount / success.totalMails) * 100).toFixed(0)) + "%");
        $("#spfScorePercent").html((((success.spfFailCount / success.totalMails) * 100).toFixed(0)) == "NaN" ? 0 : ((success.spfFailCount / success.totalMails) * 100).toFixed(0));
        // buildGraph("#dkimScore", ((success.dkimFailCount / success.totalMails) * 100).toFixed(0));
        $("#dkimScore").html((((success.dkimFailCount / success.totalMails) * 100).toFixed(0) == "NaN" ? 0 : ((success.dkimFailCount / success.totalMails) * 100).toFixed(0)) + "%");
        $("#dkimScorePercent").html((((success.dkimFailCount / success.totalMails) * 100).toFixed(0)) == "NaN" ? 0 : ((success.dkimFailCount / success.totalMails) * 100).toFixed(0));
        // buildGraph("#dmarcScore", ((success.dmarcFailCount / success.totalMails) * 100).toFixed(0));
        $("#dmarcScore").html((((success.dmarcFailCount / success.totalMails) * 100).toFixed(0) == "NaN" ? 0 : ((success.dmarcFailCount / success.totalMails) * 100).toFixed(0)) + "%");
        $("#dmarcScorePercent").html((((success.dmarcFailCount / success.totalMails) * 100).toFixed(0)) == "NaN" ? 0 : ((success.dmarcFailCount / success.totalMails) * 100).toFixed(0));
        showAnimation();
    }, function(err) {
        showError("Error Showing Counts");
    })
}
var getHeaderFailedMails = function() {
    api.aggregateReports.failedHeadersMails({ startDate, endDate }, function(success) {
        console.log("Response got getHeaderFailedMails", success);
        if (success) {
            $('#failedHeaderMails').html('')
            success.forEach(element => {
                // $('#failedHeaderMails').append('<tr><td><strong>' + element.source + '</strong></td><td>' + element.totalMails + '</td><td>' + ((element.dmarcFailCount / element.totalMails) * 100).toFixed(0) + '%</td>  <td>' + ((element.spfFailCount / element.totalMails) * 100).toFixed(0) + '%<td>' + ((element.dkimFailCount / element.totalMails) * 100).toFixed(0) + '%</td></tr>')
                $('#failedHeaderMails').append('<tr><td><strong>' + element.source + '</strong></td><td>' + element.totalMails + '</td><td><div class="progress mt-0" style="width: 80%;"> <div class="progress-bar progress-bar-striped bg-danger" role="progressbar" style="width: ' + ((element.dmarcFailCount / element.totalMails) * 100).toFixed(0) + '%;" aria-valuenow="' + ((element.dmarcFailCount / element.totalMails) * 100).toFixed(0) + '" aria-valuemin="0" aria-valuemax="100">' + ((element.dmarcFailCount / element.totalMails) * 100).toFixed(0) + '%</div></div></div></td>  <td>' + ((element.spfFailCount / element.totalMails) * 100).toFixed(0) + '%<td>' + ((element.dkimFailCount / element.totalMails) * 100).toFixed(0) + '%</td></tr>')
            });
        }
    }, function(err) {
        showError("Error Showing Mails");
    })
}

getHeaderFailedCounts();
getHeaderFailedMails();