function makeGraphAttachments(safeAttachments, unsafeAttachments) {
    var options = {
        series: [safeAttachments, unsafeAttachments],
        chart: {
            width: 450,
            type: 'pie',
        },
        labels: ['Safe Attachments', 'Unsafe Attachments'],
        colors: ['#4fb68a', '#E34234'],
        responsive: [{
            breakpoint: 480,
            options: {
                chart: {
                    width: 200
                },
                legend: {
                    position: 'top'
                }
            }
        }]
    };

    var chart = new ApexCharts(document.querySelector("#chart"), options);
    chart.render();
}

function makeGraphUrls(safeUrls, unsafeUrls) {
    console.log("safe and unsafe", safeUrls, unsafeUrls);
    var options = {
        series: [safeUrls, unsafeUrls],
        chart: {
            width: 405,
            type: 'pie',
        },
        labels: ['Safe Urls', 'Unsafe Urls'],
        colors: ['#4fb68a', '#E34234'],
        responsive: [{
            breakpoint: 480,
            options: {
                chart: {
                    width: 200
                },
                legend: {
                    position: 'top'
                }
            }
        }]
    };

    var chart = new ApexCharts(document.querySelector("#chart3"), options);
    chart.render();
}

function scannedAttachmentCounts() {
    console.log("Insdie scannedAttachmentCounts");
    api.aggregateReports.scannedAttachments({ startDate, endDate }, function(success) {
        console.log("Response got scannedAttachmentCounts", success);
        let safeAttachments = Number(success.totalAttachmentScanned) - Number(success.attachmentsVirusFound);
        let unsafeAttachments = Number(success.totalAttachmentScanned) - safeAttachments;
        let safeAttachmentsPercent = (safeAttachments / Number(success.totalAttachmentScanned) * 100).toFixed(0);
        let unsafeAttachmentsPercent = (unsafeAttachments / Number(success.totalAttachmentScanned) * 100).toFixed(0);
        makeGraphAttachments(Number(safeAttachmentsPercent), Number(unsafeAttachmentsPercent));
    }, function(err) {
        showError("Error Showing Counts");
    })
}

function scannedUrlsCounts() {
    console.log("Insdie scannedUrlsCounts");
    api.aggregateReports.scannedUrls({ startDate, endDate }, function(success) {
        let safeUrls = Number(success.totalUrlScanned) - Number(success.urlVirusFound);
        let unsafeUrls = Number(success.totalUrlScanned) - safeUrls;
        let safeUrlsPercent = (safeUrls / Number(success.totalUrlScanned) * 100).toFixed(0);
        let unsafeUrlsPercent = (unsafeUrls / Number(success.totalUrlScanned) * 100).toFixed(0);
        makeGraphUrls(Number(safeUrlsPercent), Number(unsafeUrlsPercent));
    }, function(err) {
        showError("Error Showing Counts");
    })
}

function makeGraphGctx(...args) {
    console.log("...agrs", args);
    var options = {
        chart: {
            height: 270,
            type: 'radialBar',
        },
        plotOptions: {
            radialBar: {
                dataLabels: {
                    name: {
                        fontSize: '18px',
                        fontFamily: 'Muli',
                    },
                    value: {
                        fontSize: '16px',
                        fontFamily: 'Muli',
                    },
                    total: {
                        show: true,
                        label: 'Total',
                        formatter: function(w) {
                            return args[4]
                        }
                    }
                }
            }
        },
        colors: ['#556fe6', '#4fa5f1', '#f1b44c', '#dadce1'],
        series: [args[0], args[1], args[2], args[3]],
        labels: ['Attachments', 'Domains', 'Urls', 'IPs'],
    }
    var chart = new ApexCharts(
        document.querySelector("#apex-radial-chart"),
        options
    );
    chart.render();
}

function gctxScannedCounts() {
    console.log("Insdie gctxScannedCounts");
    api.aggregateReports.gctxAnalysis({ startDate, endDate }, function(success) {
        console.log("gctxScannedCounts response", success);
        let totalData = Number(success.totalAttachmentScannedGctx) + Number(success.totalDomainScanned) + Number(success.totalIpScannedGctx) + Number(success.totalUrlScannedGctx);
        let totalVirus = Number(success.domainVirusFound) + Number(success.ipVirusFoundGctx) + Number(success.attachmentVirusFoundGctx) + Number(success.urlVirusFoundGctx);
        let totalSafeData = totalData - totalVirus;
        $("#totalGctxScanned").html(totalData);
        $('#safe_data').html(totalSafeData);
        $('#unsafe_data').html(totalVirus);
        $('#attachmentsGctx').html(success.totalAttachmentScannedGctx);
        $('#urlsGctx').html(success.totalUrlScannedGctx);
        $('#domainGctx').html(success.totalDomainScanned);
        $('#ipGctx').html(success.totalIpScannedGctx);
        let attachmentPercent = (Number(success.attachmentVirusFoundGctx) / Number(success.totalAttachmentScannedGctx) * 100).toFixed(0);
        let urlPercent = (Number(success.urlVirusFoundGctx) / Number(success.totalUrlScannedGctx) * 100).toFixed(0)
        let domainPercent = (Number(success.domainVirusFound) / Number(success.totalDomainScanned) * 100).toFixed(0)
        let ipPercent = (Number(success.ipVirusFoundGctx) / Number(success.totalIpScannedGctx) * 100).toFixed(0)
        makeGraphGctx(attachmentPercent, domainPercent, urlPercent, ipPercent, totalVirus)
    }, function(err) {
        showError("Error Showing Counts");
    })
}

scannedAttachmentCounts();
scannedUrlsCounts();
gctxScannedCounts();