import { db } from '../../models';

export default {
    async index(req, res) {
        let limit = 10;
        let offset = 0;
        let page = 1;
        let search = '%%';
        let sort = ['createdAt', 'DESC'];
        var condition = {};
        if(req.query.limit && req.query.limit != undefined){
            limit = parseInt(req.query.limit);
        }
        if (req.query.search && req.query.search != undefined) {
            search = '%' + req.query.search + '%';
            condition['indicator'] = {like: search}
        }
        if (req.query.page && req.query.page != undefined) {
            page = req.query.page;
            if (page < 1)
                page = 1;
        }
        if (req.query.sort && req.query.sort.length) {
            if (req.query.sort == 'name') {
                sort = ['name'];
            }
        }
        if(req.query.type && req.query.type != undefined) {
            condition['type'] = {in: (req.query.type).split(',')};
        }
        db.indicator.count({
          where: condition,
       })
        .then((count) => {
            let pages = Math.ceil(count / limit);
            offset = limit * (page - 1);
            return db.indicator.findAll({
                where: condition,
                limit: limit,
                offset: offset,
                order: [sort],
            }).then(r => [r, count, pages])
        })
        .then(([ gctx, count, pages ]) => {
            return res.status(200).json({'success': true, 'gctx': gctx, 'count': count, 'pages': pages });
        })
        .catch((error) => {
            console.log(error);
            next(error);
        });
    },

    async md5HashCheck(req,res,next){
        var md5hash= req.query.hash;
        return db.indicator.findOne({
            where : {
                type: "FileHash-MD5",
                indicator: md5hash
            }
        })
        .then(r =>{
            if(r) return res.status(200).json({ md5HashCheck: "True" });
            else return res.status(200).json({ md5HashCheck: "False" });
        })
        .catch(err=>{
            console.log(err);
            next(err);
        })
    },

    async sha256HashCheck(req,res,next){
        var sha256hash= req.query.hash;
        return db.indicator.findOne({
            where : {
                type: "FileHash-SHA256",
                indicator: sha256hash
            }
        })
        .then(r =>{
            if(r) return res.status(200).json({ sha256HashCheck: "True" });
            else return res.status(200).json({ sha256HashCheck: "False" });
        })
        .catch(err=>{
            console.log(err);
            next(err);
        })
    },

    async sha1HashCheck(req,res,next){
        var sha1hash= req.query.hash;
        return db.indicator.findOne({
            where : {
                type: "FileHash-SHA1",
                indicator: sha1hash
            }
        })
        .then(r =>{
            if(r) return res.status(200).json({ sha1hash: "True" });
            else return res.status(200).json({ sha1hash: "False" });
        })
        .catch(err=>{
            console.log(err);
            next(err);
        })
    },

    async urlCheck(req,res,next){
        var url= 'http://'+req.query.url+'/';
        return db.indicator.findOne({
            where : {
                type: "URL",
                indicator: { like: '%' + url + '%' },
            }
        })
        .then(r =>{
            if(r) return res.status(200).json({ url: "True" });
            else return res.status(200).json({ url: "False" });
        })
        .catch(err=>{
            console.log(err);
            next(err);
        })
    },

    async checkDomain(req,res,next){
        console.log("Inside checkDomain",req.body.domain, req.body.hostname)
        var data={};
        return db.indicator.findOne({
            where:{
                type: "hostname",
                indicator: { like :'%'+ req.body.domain+'%' }
            }
        })
        .then(r=>{
            if(r) return data['domain'] = "success";
            else return data['domain']="failed";
        })
        .then(r=>{
            if(r){
                return db.indicator.findOne({
                    where:{
                        type: "domain",
                        indicator: { like: '%'+ req.body.hostname +'%' }
                    }
                })
            }
            else throw new Error("coudnt find domain in gctx")
        }).then(r=>{
            if(r) return data['hostname'] = "success";
            else return data['hostname']="failed";
        })
        .then(r=>{
            if(r) res.status(200).json({data: data})
            else throw new Error("coudnt find hostname in gctx");
        })
        .catch(err=>{
            console.log(err);
            res.status(500).json({err: err})
        })
    },


    async checkIp(req , res ,next){
        var IP = req.body.ip;
        return db.indicator.findOne({
            where : {
                type: "IPv4",
                indicator: IP
            }
        })
        .then(r =>{
            if(r) return res.status(200).json({ IP: "True" });
            else return res.status(200).json({ IP: "False" });
        })
        .catch(err=>{
            console.log(err);
            next(err);
        })
    },
}
