/*
 * Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
 * See LICENSE in the project root for license information.
 */

/* global document, Office */
var header = '',
    body = '',
    mailAttachments = [],
    accessToken = '',
    key = null,
    from = null,
    to = null,
    mailEml64 = '',
    currentItem = null,
    bodyText = '',
    subject = '';

/**
 * Error Display Function
 */

function showErrorMessage(message) {
    $('#analysis').hide();
    $('#error-message').html(message);
    $('#error-message').fadeIn();
}

function hideErrorMessage() {
    $('#error-message').fadeOut();
}

(function loadOffice() {
    $("#loadingText").html("Initializing....");
    console.log("Initializing office....");
    // First check if the script already exists on the dom
    // by searching for an id
    let id = 'office'
    if (document.getElementById(id) === null) {
        let script = document.createElement('script')
        script.setAttribute('src', 'https://appsforoffice.microsoft.com/lib/1/hosted/office.js')
        script.setAttribute('id', id)
        document.body.appendChild(script)

        // now wait for it to load...
        script.onload = () => {
            // script has loaded, you can now use it safely
            console.log("Office loaded successfully")
            initializeOffice()
        }

        // now wait for it to load...
        script.onerror = () => {
            // script has loaded, you can now use it safely
            console.log("Office not loaded")
            showErrorMessage("Office not loaded")
        }
    } else {
        console.log("Inside else of office load....");
    }
})()

function initializeOffice() {
    Office.initialize = function () { };
    $("#loadingText").html("Office Initialized....");

    Office.onReady(function (info) {
        showPageLoader();
        from = Office.context.mailbox.item.from.emailAddress;
        to = Office.context.mailbox.userProfile.emailAddress;
        getDataFromMail_v2();
        document.getElementById("hackRecord").onclick = showHackRecord;
        document.getElementById("refresh").onclick = Refresh;
        document.getElementById("back").onclick = backButton;
        document.getElementById("ham").onclick = markHam;
        $('#analysis').hide();
        $('#back').hide();
    });
}

function getAttachments(attachments) {
    console.log("attachments.length", attachments.length)
    attachments.forEach(function (attachment) {
        // console.log("attachments", attachment);
        Office.context.mailbox.item.getAttachmentContentAsync(attachment.id, function (result) {
            // console.log("reulst", result)
            mailAttachments.push({ format: result.value.format, Value: result.value.content, Key: attachment.name, attachmentType: attachment.attachmentType });
            // console.log("Attachments now", mailAttachments);
        });
    })
}

function getMailHeaders(callback) {
    try {
        Office.context.mailbox.item.getAllInternetHeadersAsync(
            function (asyncResult) {
                if (asyncResult.status === Office.AsyncResultStatus.Succeeded) {
                    // console.log("asyncResult.value",asyncResult.value)
                    callback(asyncResult.value);
                    // callback("");
                } else {
                    $("#loadingText").html("Headers fething failed....!");
                    callback("");
                }
            });
    } catch (err) {
        console.log("Error while fetching headers", err);
        $("#loadingText").html("Headers fething failed....!");
        callback("");
    }
}

function getMailBody(type, done) {
    Office.context.mailbox.item.body.getAsync(
        type,
        function callback(result) {
            done(result.value)
        });
}

function getDataFromMail_v2() {
    console.log("Inside getDataFromMail_v2", Office.context.mailbox.item, Office.context.mailbox.item.subject);
    getMailBody("text", function (strBody) {
        bodyText = strBody;
        $("#loadingText").html("Fetching mail body....");
        getMailBody(Office.CoercionType.Html, function (htmlBody) { //HTML Body TYPE
            body = htmlBody;
            $("#loadingText").html("Fetching mail headers....");
            getMailHeaders(function (rawHeader) {
                header = rawHeader;
                checkConfig();
            });
        });
    })
}

function Refresh() {
    showPageLoader()
    from = Office.context.mailbox.item.from.emailAddress;
    to = Office.context.mailbox.userProfile.emailAddress;
    // subject = Office.context.mailbox.item.subject;
    getDataFromMail_v2();
    document.getElementById("hackRecord").onclick = showHackRecord;
    document.getElementById("back").onclick = backButton;
    document.getElementById("ham").onclick = markHam;
    $('#hackRecordTable').hide();
    $('#hacksafe').hide();
    $('#analysis').show();
    $('#back').hide();
}

function markHam() {
    $.ajax({
        headers: {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        },
        url: "https://test1.threatalertbutton.com/api/email/notSpam",
        type: "POST",
        data: JSON.stringify({ from: from, to: to, header: header, body: body, token: getUrlParameter('name'), subject: Office.context.mailbox.item.subject }),
    })
        .done(function (item) {
            showPageLoader()
            Refresh()
        })
        .fail(function (jqXHR, textStatus) {
            hidePageLoader()
            // console.log('error sending', textStatus, jqXHR);
        })
}

function backButton() {
    $('#hackRecordTable').hide();
    $('#hacksafe').hide();
    Refresh();
    $('#analysis').show();
}

function showHackRecord() {
    $('#analysis').hide();
    $('#back').show();
    var x = $('#hackRecordTable').find("td").text()
    if (x) $('#hackRecordTable').show();
    else $('#hacksafe').show();
}

function showPageLoader() {
    $('.page-loader').removeClass('d-none')
    $('.page-loader').addClass('d-block')
}

function hidePageLoader() {
    $('.page-loader').removeClass('d-block')
    $('.page-loader').addClass('d-none')
}

var getUrlParameter = function getUrlParameter(sParam) {
    var sPageURL = window.location.search.substring(1),
        sURLVariables = sPageURL.split('&'),
        sParameterName,
        i;

    for (i = 0; i < sURLVariables.length; i++) {
        sParameterName = sURLVariables[i].split('=');

        if (sParameterName[0] === sParam) {
            return sParameterName[1] === undefined ? true : decodeURIComponent(sParameterName[1]);
        }
    }
};

$(function () {
    $('[data-toggle="tooltip"]').tooltip()
})

function checkConfig() {
    $("#loadingText").html("Checking configuration....");
    // console.log("Checkconfig", getUrlParameter('tablite').split("&et")[0])
    var data = { to: getUrlParameter('tablite').split("&et")[0] }
    // console.log("data is", data);
    $.ajax({
        headers: {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        },
        url: "https://test1.threatalertbutton.com/api/email/checkConfig",
        type: "GET",
        data: data,
    })
        .done(function (success) {
            fetchDmarc(success);
        })
        .fail(function (jqXHR, textStatus) {
            console.log("Coundnt find configuration for Report Button")
        })
}

function sendRequest() {
    var data = {
        header: header,
        body: body,
        token: getUrlParameter('name'),
        from,
        from,
        to: to,
        attachments: mailAttachments,
        subject: Office.context.mailbox.item.subject
    }
    let url;
    getUrlParameter('tablite').split("&et")[0] == 0 ? url = "https://test1.threatalertbutton.com/api/email/eml" : url = "https://app.threatcop.ai/api/report"
    $.ajax({
        headers: {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        },
        url: url,
        type: "POST",
        data: JSON.stringify(data),
    })
        .done(function (item) {
            var item = Office.context.mailbox.item;
            moveToJunk(item.itemId);
            Refresh()
            hidePageLoader()
        })
        .fail(function (jqXHR, textStatus) {
            hidePageLoader()
            document.getElementById("warning").show()
            document.getElementById("app-body").hide();
            console.log('error sending', textStatus, jqXHR);
        })
}

function moveToJunk(itemId) {
    // console.log("moveToJunk itemId", itemId)
    easyEws.moveItem(itemId, "deleteditems", function () {
        console.log("Item moved to deleted items");
    }, function (error) {
        console.log(error);
    });
}

function fetchDmarc(data) {
    $("#loadingText").html("Analyzing data....");
    $.ajax({
        headers: {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        },
        url: "https://test1.threatalertbutton.com/api/email/parseheader",
        type: "POST",
        data: JSON.stringify({ header: header, body: body, from: from, to: to, config: data, token: getUrlParameter('name'), bodyText, subject: Office.context.mailbox.item.subject }),
    })
        .done(function (item) {
            // console.log("item is", item);
            if (item.data.dkimAlignment == 'Aligned') var dkimalignclass = "text-success";
            else if (item.data.dkimAlignment == 'Not Aligned') var dkimalignclass = "text-danger";
            if (item.data.spfAlignment == 'Aligned') var spfalignclass = "text-success";
            else if (item.data.spfAlignment == 'Not Aligned') var spfalignclass = "text-danger";
            if (item.data.spam == 'Ham') var spamclass = "text-success";
            else if (item.data.spam == 'Spam') var spamclass = "text-danger";
            if (item.data.analytics[0].spf == 'Pass') var spfclass = "badge-pass";
            else if (item.data.analytics[0].spf == 'Fail') var spfclass = "badge-fail";
            if (item.data.analytics[0].dkim == 'Pass') var dkimclass = "badge-pass";
            else if (item.data.analytics[0].dkim == 'Fail') var dkimclass = "badge-fail";
            if (item.data.analytics[0].dmarc == 'Pass') var dmarcclass = "badge-pass";
            else if (item.data.analytics[0].dmarc == 'Fail') var dmarcclass = "badge-fail";
            if (item.data.gctxhostname == 'success' && item.data.gctxdomain == 'success') { var gctxClass = "text-danger"; var gctxData = "Found"; } else if (item.data.gctxhostname == 'failed' && item.data.gctxdomain == 'failed') var gctxClass = "text-success";
            var gctxData = "Not Found";
            if (item.data.spambutton == 'spam') {
                $('#run').hide();
                $("#ham").show();
            }
            if (item.data.spambutton == 'ham') {
                $('#run').show();
                $('#ham').hide();
            }
            if (item.data.spambutton == 'Both') {
                $('#run').show();
                $('#ham').show();
            }
            if (Math.sign(item.data.reportedCount) == -1) item.data.reportedCount = 0;
            if (!item.settings) {
                $('#anaytics').html('<tr><td>Server Authenticity</td><td><span class="badge ' + spfclass + '">' + item.data.analytics[0].spf + '</span></td></tr><tr><td>Email Authenticity</td><td><span class="badge ' + dmarcclass + '">' + item.data.analytics[0].dmarc + '</span></td></tr><tr><td>DKIM</td><td><span class="badge ' + dkimclass + '">' + item.data.analytics[0].spf + '</span></td></tr><tr><td>Content</td><td><span class=' + spamclass + '>' + item.data.spam + '</span></td></tr><tr><td>DKIM Alignment</td> <td><span class=' + dkimalignclass + '>' + item.data.dkimAlignment + '</span></td></tr><tr><td>SPF Alignment</td><td><span class=' + spfalignclass + '>' + item.data.spfAlignment + '</span></td></tr><tr><td>GCTX</td><td><span class=' + gctxClass + '>' + gctxData + '</span></td></tr><tr><td>Reported By</td><td>' + item.data.reportedCount + ' People</td></tr><tr><td>Country</td><td>' + item.data.country + '</td></tr>')
                // var percentage = Math.floor((item.data.score / 70) * 100);
            } else {
                var settings = item.settings;
                //report button config
                var spf = '',
                    dkim = '',
                    dmarc = '',
                    content = '',
                    dkimalign = '',
                    spfalign = '',
                    gctx = '',
                    reportedby = '',
                    country = '',
                    graphtotal = 0;
                if (settings.spf == 'true') {
                    graphtotal = graphtotal + 1;
                    spf = '<tr><td>Server Authenticity</td><td><span class="badge ' + spfclass + '">' + item.data.analytics[0].spf + '</span></td></tr>'
                }
                if (settings.dmarc == 'true') {
                    graphtotal = graphtotal + 1;
                    dmarc = '<tr><td>Email Authenticity</td><td><span class="badge ' + dmarcclass + '">' + item.data.analytics[0].dmarc + '</span></td></tr>'
                }
                if (settings.dkim == 'true') {
                    graphtotal = graphtotal + 1;
                    dkim = '<tr><td>DKIM</td><td><span class="badge ' + dkimclass + '">' + item.data.analytics[0].spf + '</span></td></tr>'
                }
                if (settings.content == 'true') {
                    graphtotal = graphtotal + 1;
                    content = '<tr><td>Content</td><td><span class=' + spamclass + '>' + item.data.spam + '</span></td></tr>'
                }
                if (settings.dkimalign == 'true') {
                    graphtotal = graphtotal + 1;
                    dkimalign = '<tr><td>DKIM Alignment</td> <td><span class=' + dkimalignclass + '>' + item.data.dkimAlignment + '</span></td></tr>'
                }
                if (settings.spfalign == 'true') {
                    graphtotal = graphtotal + 1;
                    spfalign = '<tr><td>SPF Alignment</td><td><span class=' + spfalignclass + '>' + item.data.spfAlignment + '</span></td></tr>'
                }
                if (settings.gctx == 'true') {
                    graphtotal = graphtotal + 1;
                    gctx = '<tr><td>GCTX</td><td><span class=' + gctxClass + '>' + gctxData + '</span></td></tr>'
                }
                if (settings.reportedby == 'true') { reportedby = '<tr><td>Reported By</td><td>' + item.data.reportedCount + ' People</td></tr>' }
                if (settings.country == 'true') { country = '<tr><td>Country</td><td>' + item.data.country + '</td></tr>' }
                var config = spf.concat(dmarc, dkim, content, dkimalign, spfalign, gctx, reportedby, country)
                //report button config end

                $('#anaytics').html(config)
                // var percentage = Math.floor((item.data.score / 70) * 100);
            }
            if (settings.score == "true") {
                console.log("mail score view")
                var percentage = item.data.score;
                // $(".chart-wrap").css({display : "block"})
                // $("#graph").css({display : "block"})
                $("#refresh").css({ display: "block" })
                $('#graph').attr('value', percentage)
                if (percentage > 67) $('#graph').attr('data-fgColor', "#1cbf65")
                else if (percentage > 33 && percentage < 67) $('#graph').attr('data-fgColor', "#ffa800")
                else $('#graph').attr('data-fgColor', "#e83e3e")
                $(".knob").knob();
            } else {
                // console.log("Inside else")
                $(".chart-wrap").css({ display: "none" })
                $("#graph").css({ display: "none" })
            }

            console.log("item.data.spam", item.data.spam)
            if (settings.spf == 'false' && settings.dmarc == 'false' && settings.dkim == 'false' && settings.content == 'false' && settings.dkimalign == 'false' && settings.spfalign == 'false' && settings.gctx == 'false' && settings.reportedby == 'false' && settings.country == 'false' && settings.hackbutton == 'false' && settings.score == 'false') {
                $("#display-message").html(item.data.spambutton == 'ham' ? "Is this mail looking suspicious? Report it to the Admin." : "Is this mail looking safe? Mark it not spam.")
                $("#display-message").css({ display: "block" })
                $("#refresh").css({ display: "none" })
            }
            if (item.data.hackRecords.length > 0) {
                if (settings.hackbutton == 'true') $('#hackRecord').show()
                for (var i = 0; i < item.data.hackRecords.length; i++) {
                    $('#records').append('<tr><td>' + item.data.hackRecords[i].Name + '</td><td>' + item.data.hackRecords[i].BreachDate + '</td><td><i class="fa fa-question-circle" aria-hidden="true" data-toggle="tooltip" id="tool-' + i + '"></i> </td></tr>')
                    $('#tool-' + i).attr('title', item.data.hackRecords[i].Description);
                }
            } else {
                if (settings.hackbutton == 'true') { $('#hackRecord').show() } else $('#hackRecord').hide()
            }

            function clock() {
                var $s = $(".second"),
                    $m = $(".minute"),
                    $h = $(".hour");
                var d = new Date(),
                    s = d.getSeconds(),
                    m = d.getMinutes(),
                    h = d.getHours();
                $s.val(s).trigger("change");
                $m.val(m).trigger("change");
                $h.val(h).trigger("change");
                setTimeout(clock, 1000);
            }
            clock();
            hidePageLoader()
            $('#analysis').show();
            document.getElementById("run").onclick = run;
            if (settings.attachments == "true") {
                getAttachments(Office.context.mailbox.item.attachments);
            }
        })
        .fail(function (jqXHR, textStatus) { })
}

export async function run() {
    showPageLoader()
    Office.onReady()
        .then(function () {
            sendRequest();
        })
        .catch(function (err) {
            console.log(err);
        })
}