import request from 'request';
import Tab from '../../../modules/Tab';
import Api from '../../../modules/Apis';
import { db } from '../../../models';
import jwt from 'jsonwebtoken';
import config from '../../../config';
import fs from 'fs';

function makeRequest(reqData) {
    let startDate = null;
    let endDate = null;
    let dateFilter = "'";
    if (reqData.startDate != undefined) {
        startDate = new Date(reqData.startDate);
        endDate = new Date(reqData.endDate);
        startDate = startDate.getFullYear() + "-" + (startDate.getMonth() + 1).pad(2) + '-' + startDate.getDate().pad(2);
        endDate = endDate.getFullYear() + "-" + (endDate.getMonth() + 1).pad(2) + '-' + endDate.getDate().pad(2);
        console.log("Date", startDate, endDate);
    } else {
        var date = new Date();
        endDate = date.getFullYear() + "-" + (date.getMonth() + 1).pad(2) + '-' + date.getDate().pad(2);
        date.setDate(date.getDate() - 10);
        startDate = date.getFullYear() + "-" + (date.getMonth() + 1).pad(2) + '-' + date.getDate().pad(2);
    }

    startDate += 'T00:00:00';
    endDate += 'T23:59:59';
    dateFilter += " and StartDate eq datetime'" + startDate + "' and EndDate eq datetime'" + endDate + "'";
    const options = {
        url: "https://reports.office365.com/ecp/reportingwebservice/reporting.svc/MessageTrace?$format=Json&$filter=" + encodeURIComponent("SenderAddress eq '" + reqData.sender + dateFilter),
        headers: {
            'Content-Type': 'application/json'
        },
        auth: {
            user: reqData.username,
            pass: reqData.password
        }
    };

    console.log("Request formed", options);
    return new Promise(resolve => {
        request.get(options, (error, response, body) => {
            console.log("error", error);
            if (error)
                resolve([])
            else {
                body = JSON.parse(body);
                // console.log("response",body.d.results)
                resolve(body.d.results);
            }
        });
    })
}

function formatMailResults(mails, subject, senderIP) {
    if (mails && mails.length) {
        return mails.map(mail => {
            let newData = {};
            if (subject && mail["Subject"].toLowerCase().indexOf(subject.toLowerCase()) == -1)
                return newData;
            if (senderIP && mail["FromIP"].indexOf(senderIP) == -1)
                return newData;
            newData.organization = mail["Organization"]
            newData.senderAddress = mail["SenderAddress"]
            newData.recipientAddress = mail["RecipientAddress"]
            newData.subject = mail["Subject"]
            newData.status = mail["Status"]
            newData.fromIp = mail["FromIP"]
            newData.size = mail["Size"]
            newData.message_id = mail["MessageId"]
            newData.recieved = new Date(Number(mail["Received"].replace('/Date(', '').replace(')/', '')))
            return newData;
        }).filter(data => data.senderAddress)
    } else {
        return mails;
    }
}

function getMailIdGraph(url, accessToken) {
    const options = {
        url: url,
        headers: {
            'Content-Type': 'application/json',
            'Authorization': accessToken
        }
    };

    console.log("Request formed", options);
    return new Promise(resolve => {
        request.get(options, (error, response, body) => {
            console.log("error", error);
            if (error)
                resolve([])
            else {
                body = JSON.parse(body);
                console.log("response", body)
                if (body.value && body.value.length)
                    resolve(body.value[0]);
                else if (body.error)
                    resolve(body.error.message)
                else
                    resolve(null);
            }
        });
    })
}

async function getAndMoveMails(mails, toFolder, accessToken) {
    for (let i = 0; i < mails.length; i++) {
        let mail = await getMailIdGraph('https://graph.microsoft.com/v1.0/users/' + mails[i].recipientAddress + '/messages?$filter=internetMessageId' + encodeURIComponent(" eq '" + mails[i].message_id + "'"), accessToken);
        console.log("Mail found", mail)
        if (mail) {
            new Api().moveMail({ url: "https://graph.microsoft.com/v1.0/users/" + mails[i].recipientAddress + "/messages/" + mail.id + "/move", token: accessToken, destination: toFolder })
            if (toFolder.indexOf("delete") != -1)
                new Tab().updateDeleteStatus({ id: mails[i].id, isDeleted: true, deleteStatus: 1, reportedMailId: mails[i].reportedMailId })
            else
                new Tab().updateDeleteStatus({ id: mails[i].id, isDeleted: false, deleteStatus: null, reportedMailId: mails[i].reportedMailId })
        } else {
            console.log("No mailbox found or on premises malbox or some error occured")
            if (toFolder.indexOf("delete") != -1)
                new Tab().updateDeleteStatus({ id: mails[i].id, isDeleted: false, deleteStatus: 3, reportedMailId: mails[i].reportedMailId })
            else
                new Tab().updateDeleteStatus({ id: mails[i].id, isDeleted: false, deleteStatus: null, reportedMailId: mails[i].reportedMailId })
        }
    }
}

function extractPayloadFromToken(token) {
    return jwt.decode(token, (err, decoded) => {
        console.log("Error", err);
        console.log("decoded", decoded);
        return decoded;
    });
}

export default {
    async getMessageTrace(req, res, next) {
        console.log("Inside getMessageTrace", req.body);
        let mailsJson = await makeRequest(req.body);
        if (mailsJson)
            mailsJson = await formatMailResults(mailsJson, req.body.subject, req.body.senderIP);

        // if (req.body.subject)
        //     mailsJson = mailsJson.filter((mail) => {
        //         if (mail) {
        //             console.log("mail subject", mail.subject.toLowerCase(), req.body.subject.toLowerCase())
        //             return mail.subject.toLowerCase() == req.body.subject.toLowerCase() || mail.subject.toLowerCase() == "re: " + req.body.subject.toLowerCase()
        //         }
        //     })
        console.log("Final mail", mailsJson)
        res.send({ mails: mailsJson });
    },
    async callbackToken(req, res, next) {
        let tab_user = JSON.parse(JSON.stringify(req.tab_user));
        let decodedToken = await extractPayloadFromToken(req.tokens.accessToken)
        let expiry = (Date.now() / 1000) + 3600;
        if (decodedToken)
            expiry = decodedToken.exp;
        console.log("Inside callbackToken", req.user, req.tokens, tab_user, expiry);
        db.User.update({ outlook_access_token: req.tokens.accessToken, outlook_refresh_token: req.tokens.refreshToken, outlook_token_expiry: expiry }, { where: { id: req.tab_user.id } }).then(r => console.log("Access tokens updated"))
        let html = '<h2>Credentials Verified Successfully</h2><a href="JavaScript:window.close()"><button>Close window</button></a>'
        res.send(html);
    },
    async callbackTokenV2(req, res, next) {
        if (req.tab_user) {
            let tab_user = JSON.parse(JSON.stringify(req.tab_user));
            let decodedToken = await extractPayloadFromToken(req.tokens.accessToken)
            let expiry = (Date.now() / 1000) + 3600;
            if (decodedToken)
                expiry = decodedToken.exp;
            console.log("Inside callbackToken", req.user, req.tokens, tab_user, expiry);
            db.User.update({ outlook_access_token: req.tokens.accessToken, outlook_refresh_token: req.tokens.refreshToken, outlook_token_expiry: expiry }, { where: { id: req.tab_user.id } }).then(r => console.log("Access tokens updated"))
            let html = '<h2>Credentials Verified Successfully</h2><a href="JavaScript:window.close()"><button>Close window</button></a>'
            res.send(html);
        } else {
            console.log("req.tokens", req.user)
            let decodedToken = await extractPayloadFromToken(req.tokens.accessToken)
            let expiry = (Date.now() / 1000) + 3600;
            if (decodedToken)
                expiry = decodedToken.exp;
            console.log("Inside callbackToken", req.tokens.accessToken);
            let html = '<h2>Succesfully Authenticated</h2>\
            <a href="https://ohimlagjgmleoefdhhnaaienoehnflpc.chromiumapp.org/#authToken=' + req.tokens.accessToken + "#user=" + req.user.emails[0].value + '"><button>Redirect To App</button></a>\
            <a href="JavaScript:window.close()"><button>Close window</button></a>'
            res.send(html);

        }
    },
    async deleteMail(req, res, next) {
        console.log("Inside deleteMail", req.body);
        let condition = { reportedMailId: req.body.reportedMailId }
        if (req.body.reportedMailResultId) {
            condition["id"] = req.body.reportedMailResultId
        } else {
            condition["isDeleted"] = true;
            new Tab().updateBaseMailStatus({ condition })
            delete condition["isDeleted"];
        }

        console.log("Condition formed", condition)
        let mails = await new Tab().getWhoElseMails({ condition }) //Get all who else mails from the database
        console.log("mails", mails);
        if (mails.length) {
            await getAndMoveMails(mails, "deletedItems", req.body.outlook_access_token); //Get mail Id of the mail for every receipients and move it to deletedFolder
        }
        res.send("Success");
    },
    async recoverMail(req, res, next) {
        console.log("Inside recoverMail", req.body);
        let condition = { reportedMailId: req.body.reportedMailId }
        if (req.body.reportedMailResultId) {
            condition["id"] = req.body.reportedMailResultId
        } else {
            condition["isDeleted"] = false;
            new Tab().updateBaseMailStatus({ condition })
            delete condition["isDeleted"];
        }

        console.log("Condition formed", condition)
        let mails = await new Tab().getWhoElseMails({ condition }) //Get all who else mails from the database
        console.log("mails", mails);
        if (mails.length)
            await getAndMoveMails(mails, "inbox", req.body.outlook_access_token); //Get mail Id of the mail for every receipients and move it to junnk
        res.send("Success");
    },

    async getMailHeader(req, res, next) {
        console.log("Inside getMailHeader", req.body.to);
        let mail = null;
        let rawHeader = null;
        if (req.body.type == "gmail") {
            mail = req.body;
        } else {
            mail = await getMailIdGraph("https://graph.microsoft.com/v1.0/me/messages?$filter=conversationId eq '" + req.body.itemId + "'", req.body.token);
        }
        // console.log("MailId is", mail.id)
        if ((mail && mail.id) || req.body.type == "gmail") {
            if (req.body.type == "gmail") {
                req.body.rawHeader = JSON.parse(req.body.rawHeader)
                rawHeader = Buffer.from(req.body.rawHeader.raw, 'base64').toString('utf8');
            } else
                // rawHeader = await new Api().getRawHeader({ url: "https://graph.microsoft.com/v1.0/me/messages/" + mail.id + "/$value", token: req.body.token })
                rawHeader = fs.readFileSync(__dirname+'/../../../../omegamailsample.eml',{encoding:'utf8', flag:'r'});;
            console.log("Raw header", rawHeader)
            let parsedEmail = await new Api().parseEmail({ body: rawHeader, to: req.body.to })
            console.log("Parsed Email", parsedEmail)
            let parseHeader = await new Api().parseheader({ header: rawHeader, body: parsedEmail.result.body, from: parsedEmail.result.from, to: parsedEmail.result.to, config: { "success": { "basic": true } }, token: req.body.userAuth })
            parseHeader.data.data = { "success": { "basic": true } };
            // console.log("parseHeader", parseHeader)
            res.send({ rawHeader, parseHeader, body: parsedEmail.result.body, attachments: parsedEmail.result.attachments, from: parsedEmail.result.from, to: parsedEmail.result.to });
        } else if (mail && !mail.id && req.body.type != "gmail") {
            res.status(401).send("Session Expired! Kindly Login Again.")
        } else {
            res.status(500).send("Something Went Wrong! Cannot Parse Email!")
        }
    },
    async movetojunk(req, res, next) {
        console.log("Inside movetojunk", req.body);
        let mail = await getMailIdGraph("https://graph.microsoft.com/v1.0/me/messages?$filter=conversationId eq '" + req.body.itemId + "'", req.body.token);
        if (mail) {
            await new Api().moveMail({ url: "https://graph.microsoft.com/v1.0/me/messages/" + mail.id + "/move", token: req.body.token, destination: "junkEmail" })
        } else {
            res.status(500).send("Something Went Wrong! Cannot Parse Email!")
        }
    }

}