import passport from 'passport';
import config from '../config';
import JWT from 'jsonwebtoken';

var JWTSign = function(user, date) {
    return JWT.sign({
        iss: config.app.name,
        sub: user.id,
        iam: user.type,
        iat: date.getTime(),
        exp: new Date().setMinutes(date.getMinutes() + 30)
    }, config.app.secret);
}

export var jwtStrategy = (req, res, next) => {
    passport.authenticate('jwt', { session: false }, (err, user, info) => {
        console.log("err", err);
        // console.log("user",user);
        // console.log("info",info);
        let contype = req.headers['content-type'];
        var json = !(!contype || contype.indexOf('application/json') !== 0);
        if (!json && info && typeof info === 'string' && info.includes("isNewUser")) {
            if (req.url != '/change-password/' && req.url != '/order/success') {
                return res.redirect('/change-password/');
            }
        }
        if (err && err == 'expired') { return json ? res.status(419).json({ errors: ['Session is expired'] }) : res.redirect('/logout'); }
        if (err && err == 'invalid') { return json ? res.status(500).json({ errors: ['Invalid token recieved'] }) : res.redirect('/logout'); }
        if (err && err == 'user') { return json ? res.status(500).json({ errors: ['Invalid user recieved'] }) : res.redirect('/logout'); }
        if (err && Object.keys(err).length) { return res.status(500).json({ errors: [err] }); }
        if (err) { return res.status(403).json({ errors: ['Invalid user recieved'] }); }
        // if (!user) { return json ? res.status(403).json({ errors: ['Session is Expired'] }) : res.redirect('/logout'); }

        //Update Token
        var date = new Date();
        var token = JWTSign(user, date);
        res.cookie('XSRF-token-tab', token, {
            expire: new Date().setMinutes(date.getMinutes() + 30),
            httpOnly: true,
            secure: config.app.secure
        });

        req.tab_user = user;
        next();
    })(req, res, next);
};

export var outlookLogin = (req, res, next) => {
    passport.authenticate('msgraph', {
        scope: [
            'openid',
            'profile',
            'offline_access',
            'https://outlook.office.com/.default'
        ]
    }, (err, user, info) => {
        console.log("err", err);
        console.log("user", user);
        console.log("info", info);
        // req.user = user;
        next();
    })(req, res, next);
};

export var outlookLogin_v2 = (req, res, next) => {
    console.log("Inside outlookLogin_v2")
    passport.authenticate('microsoft', (err, user, info) => {
        console.log("err", err);
        console.log("user", user);
        console.log("info", info);
        req.user = user;
        req.tokens = info;
        next();
    })(req, res, next);
};