import cron from 'node-cron';
import { db } from './models';
import config from './config'
import request from 'request';

function getUsersWhoseAccessTokenWillExpire() {
    console.log("Inside getUsersWhoseAccessTokenWillExpire")
    return db.sequelize.query("select id,outlook_access_token,outlook_refresh_token,(UNIX_TIMESTAMP(NOW()) - outlook_token_expiry) as expiry from Users where outlook_access_token is not null and (UNIX_TIMESTAMP(NOW()) - outlook_token_expiry) > -300", { raw: true })
        .then(r => {
            if (r)
                r = JSON.parse(JSON.stringify(r))[0]
            console.log("Tokens which are about to expire", r)
            return r;
        }).catch(err => {
            console.log("Some error occured", err)
            return [];
        })
}

async function updateAccessToken(companyId, newTokens) {
    console.log("Inside updateAccessToken", companyId, newTokens)
    let expiry = (Date.now() / 1000) + newTokens.expires_in;
    return db.User.update({
            outlook_access_token: newTokens.access_token,
            outlook_refresh_token: newTokens.refresh_token,
            outlook_token_expiry: expiry,
        }, { where: { id: companyId } })
        .then(r => console.log("New token updated"))
        .catch(err => console.log("Error while updating new tokens to database", err))
}

function regenerateAccessToken(token) {
    console.log("Inside regenerateAccessToken", token, config.app.client_id, config.app.client_secret)
    var headers = {
        'content-type': 'application/x-www-form-urlencoded'
    };

    var dataString = 'grant_type=refresh_token&client_id=' + config.app.client_id + '&client_secret=' + config.app.client_secret + '&refresh_token=' + token.outlook_refresh_token;

    var options = {
        url: 'https://login.microsoftonline.com/ee193c2d-2f30-4141-bc12-604041932049/oauth2/v2.0/token',
        method: 'POST',
        headers: headers,
        body: dataString
    };

    function callback(error, response, body) {
        if (!error && response.statusCode == 200) {
            console.log("Response", JSON.parse(body));
            updateAccessToken(token.id, JSON.parse(body))
        }
    }

    request(options, callback);

}

var scheduler = {
    init: () => {
        console.log("Scheduler initialized");
        cron.schedule('* * * * *', async() => {
            console.log('running a task every minute');
            let tokensToExpire = await getUsersWhoseAccessTokenWillExpire();
            tokensToExpire.forEach(token => regenerateAccessToken(token))
        });
    },
}

export default scheduler;