import fs from 'fs';
import path from 'path';
import { db } from '../../../models';
import { queue } from '../../../kue';
import config from '../../../config';
import VirusTotal from './../../../modules/VirusTotal';
import md5file from 'md5-file/promise';
import { randomString } from '../../../functions';

db.Content.findOne({
        include: [{ model: db.VirusTotal }],
        required: true,
        logging: console.log
    })
    .then(content => {});

var detected = function(content) {
    if (content) {
        var keys = Object.entries(content.scans);
        var detectedTrue = 0,
            virusCountTotal = 0
        for (var i = 0; i < keys.length; i++) {
            virusCountTotal = virusCountTotal + 1;
            if (keys[i][1].detected == true) {
                detectedTrue = detectedTrue + 1
            }
        }

        return [detectedTrue, virusCountTotal]
    }
}

var scanResult = (content, apiKey, type) => {
    return new Promise((resolve, reject) => {
        if (type == 'url') {
            new VirusTotal().getUrlResult({ resource: content.VirusTotals[0].scanId, apikey: apiKey })
                .then(([body, statusCode]) => {
                    if (statusCode == 200) return resolve(body);
                    if (statusCode == 204) {
                        queue.create('url-scan-result', { resource: content.VirusTotals[0], apikey: apiKey })
                        throw new RequestError("Come back later for result", 504)
                    }
                    throw new RequestError(`Virus total retured with unexpected status code ${statusCode}`, statusCode)
                })
                .catch(err => {
                    console.log(err)
                    reject(err)
                })
        } else {
            new VirusTotal().reportAttachment({ resource: content.VirusTotals[0].scanId, apikey: apiKey })
                .then(([body, statusCode]) => {
                    if (statusCode == 204 || (statusCode == 200 && body && body['response_code'] == -2)) {
                        queue.create('attachment-scan-result', { resource: content.VirusTotals[0], apikey: apiKey }).save();
                        throw new RequestError("Come back later for result", 504)
                    } else if (statusCode == 200 && body) return resolve(body);
                    throw new RequestError(`Virus total retured with unexpected status code ${statusCode}`, statusCode)
                })
                .catch(err => {
                    console.log(err)
                    reject(err)
                })
        }
    })
}

export default {
    async index(req, res, next) {
        console.log("Inside index", req.query);
        var { name, type, apiKey } = req.query;
        apiKey = apiKey || config.virusTotal.key;
        try {
            if (apiKey && apiKey.length) {
                new Promise((resolve, reject) => {
                        if (type == 'url') {
                            resolve([null, name])
                        } else {
                            var filePath = path.join(appRoot, '..', '..', 'Content', 'Attachments', name)
                            if (!fs.existsSync(filePath)) reject(new RequestError('File not availble in Attachments folder'));
                            return md5file(filePath).then(r => resolve([r, null])).catch(err => reject(err))
                        }
                    })
                    .then(([hash, name]) => {
                        let filter = (hash) ? { hash } : { name }
                        filter.type = type;
                        return db.Content.findOne({
                            include: [{ model: db.VirusTotal }],
                            where: filter,
                            required: true
                        })
                    })
                    .then(content => {
                        if (content && content.VirusTotals && content.VirusTotals.length && content.VirusTotals[0].status == 'completed' && isJson(content.VirusTotals[0].result)) {
                            return [false, content.VirusTotals[0].result, content.VirusTotals[0].id];
                        } else if (content && content.VirusTotals && content.VirusTotals.length) {
                            return scanResult(content, apiKey, content.type).then(r => [true, r, content.VirusTotals[0].id]);
                        } else if (content) {
                            throw new RequestError('Come back later for this result', 504)
                        } else throw new RequestError('Please press scan button to start the file scanning & then view result!', 400)
                    })
                    .then(([shouldUpdate, content, virusTotalId]) => {
                        if (shouldUpdate) {
                            if (content && content['response_code'] == -2) return new RequestError('Scan result not yet available', 504);
                            else if (content) {
                                var ratio = detected(content)
                                return db.VirusTotal.update({
                                    status: 'completed',
                                    result: JSON.stringify(content),
                                    virusCount: ratio[0],
                                    totalScan: ratio[1]
                                }, {
                                    where: {
                                        id: virusTotalId,
                                    }
                                }).then(r => content)
                            } else throw new RequestError('Invalid Request')
                        } else return JSON.parse(content);
                    })
                    .then(content => { //api or from database use shouldupdate for checking new  or from database  
                        return res.status(200).json({ 'success': true, 'content': content }); //new paramareetr new result or old result 
                    })
                    .catch(err => {
                        console.log(err);
                        next(err);
                    })
            } else {
                throw new RequestError('VirusTotal api key not set');
            }
        } catch (err) {
            console.log(err)
            next(err)
        }
    },
    async scan(req, res, next) {
        const { name, type, options } = req.body;
        console.log("Request body for scan", req.body);
        let apiKey = null;
        apiKey = options['apiKey'] || config.virusTotal.key;
        new Promise((resolve, reject) => {
                if (type == 'url') {
                    var hash = randomString(50)
                    resolve([hash, name])
                } else {
                    var filePath = path.join(appRoot, '..', '..', 'Content', 'Attachments', name)
                    if (!fs.existsSync(filePath)) reject(new RequestError('File not availble in Attachments folder'));
                    return md5file(filePath).then(r => resolve([r, null])).catch(err => reject(err))
                }
            })
            .then(([hash, url]) => {
                console.log("hash is", hash);
                console.log("name is", url);
                let filter = url ? { name: url } : { hash: hash };
                filter.type = type;
                console.log("Filter is", filter);
                return db.Content.findOne({
                    where: filter,
                    include: [{
                        model: db.VirusTotal
                    }]
                }).then(r => [r, hash])
            })
            .then(([content, hash]) => {
                console.log("content is", JSON.parse(JSON.stringify(content)));
                if (!content) {
                    return db.Content.create({
                        name: name,
                        hash: hash,
                        type: type,
                        scanDate: new Date()
                    }).then(r => [r, hash])
                }
                return [content, hash];
            })
            .then(([content, hash]) => {
                if (content.VirusTotals && content.VirusTotals.length) {
                    return [content.VirusTotals, hash];
                } else {
                    if (type == 'file')
                        queue.create('virus-scan-file', { 'name': content.name, 'type': type, 'contentId': content.id, 'apiKey': apiKey }).save();
                    else
                        queue.create('virus-scan-url', { 'name': content.name, 'type': type, 'contentId': content.id, 'apiKey': apiKey }).save();
                    return [false, hash]
                }
            })
            .then(([scanResult, hash]) => {
                return res.status(200).json({ 'success': true, 'hash': hash, scanResult: scanResult, 'msg': 'File Received for Scanning' });
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async attachmentRisk(req, res, next) {
        console.log("Inside attachmentRisk", req.query);
        var { name } = req.query;
        db.Content.findOne({
                where: { name: name }
            })
            .then(content => {
                if (content) {
                    console.log("Content id found", content.id);
                    return db.VirusTotal.findAll({
                        attributes: [
                            [db.sequelize.fn('sum', db.sequelize.col('virusCount')), 'vCount'],
                            [db.sequelize.fn('sum', db.sequelize.col('totalScan')), 'tScan']
                        ],
                        where: { contentId: content.id },
                        raw: true,
                        logging: true,
                        group: ['virusCount']
                    }).then(c => [content, c])
                } else return [false, false]
                    // else throw new RequestError('Could not find Attachment',404)
            })
            .then(([virusScan, v]) => {
                if (virusScan) return res.status(200).json({ 'success': true, 'totalCount': v, 'virusCount': virusScan });
                // else res.status(204).json({ 'success': true });
                else res.status(200).json({ 'success': true }); //Yashu commented the above line and added this line
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },

    async urlRisk(req, res, next) {
        console.log("Inside urlRisk", req.query);
        var { name } = req.query;
        db.Content.findOne({
                where: { name: name },
                include: [{ model: db.VirusTotal }]
            })
            .then(virusTotal => {
                return res.status(200).json({ 'success': true, 'virusTotal': virusTotal });
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    }
}