import { generateQueryString } from '../functions';
import request from 'request'
export default class API {
	constructor(baseUrl = null) {
		this.baseUrl = baseUrl;
		this.get = this.get.bind(this);
		// this.post = this.post.bind(this);
	}

	setBaseUrl(baseUrl) {
		this.baseUrl = baseUrl;
	}

	formData(url, form) {
		return new Promise((resolve, reject) => {
			request({ method: 'POST', url: url, headers: { 'Content-Type': 'multipart/form-data' }, formData: form }, function (error, response, body) {
				if (error) { return reject(error); }
				if (response && response.statusCode >= 400) {
					try {
						if (isJson(body) && JSON.parse(body) && JSON.parse(body).error) {
							return reject(new RequestError(JSON.parse(body).error, response.statusCode));
						} else if (body && body.error) {
							return reject(new RequestError(body.error, response.statusCode));
						} else {
							return reject(new RequestError(body, response.statusCode));
						}
					} catch (err) {
						return reject(err);
					}
				} else if (response && response.statusCode == 204) {
					resolve([null, response.statusCode])
				} else if (response && body) {
					resolve([isJson(body) ? JSON.parse(body) : body, response.statusCode]);;
				} else {
					return reject(new RequestError("Unexpected Response.."));
				}
				// resolve([body, res.statusCode]);
			})
		});
	}
	get(url, data) {
		return new Promise((resolve, reject) => {
			// console.log("GET url to hit", url + '?' + generateQueryString(data));
			request.get(url + '?' + generateQueryString(data), (error, response, body) => {
				if (error) { return reject(error); }
				else if (response && response.statusCode >= 400) {
					try {
						if (isJson(body) && JSON.parse(body) && JSON.parse(body).error) {
							return reject(new RequestError(JSON.parse(body).error, response.statusCode));
						} else if (body && body.error) {
							return reject(new RequestError(body.error, response.statusCode));
						} else {
							return reject(new RequestError(body, response.statusCode));
						}
					} catch (err) {
						return reject(err);
					}
				} else if (response && response.statusCode == 204) {
					resolve([null, response.statusCode])
				} else if (response && body) {
					resolve([isJson(body) ? JSON.parse(body) : body, response.statusCode]);
				} else {
					return reject(new RequestError("Unexpected Response.."));
				}
			});
		});
	}

	// post(url, data ) {
	// 	return new Promise((resolve, reject) => {
	// 		request.post(url, { json : data }, (error, response, body) => {
	// 			if (error) { return reject(error); }
	// 			if(response && response.statusCode >= 400){ 
	// 				try{
	// 					if(isJson(body) && JSON.parse(body) && JSON.parse(body).error){
	// 						return reject(new RequestError(JSON.parse(body).error, response.statusCode));
	// 					} else if(body && body.error){
	// 						return reject(new RequestError(body.error, response.statusCode));
	// 					} else {
	// 						return reject(new RequestError(body, response.statusCode));
	// 					}
	// 				}catch(err){
	// 					return reject(err);
	// 				}
	// 			} else if(response && response.statusCode == 204){
	// 				resolve([null, response.statusCode])
	// 			} else if(response && body){
	// 				resolve([isJson(body) ? JSON.parse(body) : body, response.statusCode]);;
	// 			} else {
	// 				return reject(new RequestError("Unexpected Response.."));
	// 			}
	// 		});
	// 	});
	// }

	post(query = null, data) {
		if (query) {
			var url;
			if (query.url) url = query.url;
			if (!query.url && query.baseUrl) this.baseUrl = query.baseUrl;
			if (!query.url && query.path) {
				if (!this.baseUrl) throw new Error('Base URL not set.');
				url = this.baseUrl + query.path;
			}
		} else throw new Error('URL Not available')
		return new Promise((resolve, reject) => {
			request.post(url, { json: data }, (error, response, body) => {
				if (error) { return reject(error); }
				if (response && response.statusCode >= 400) {
					try {
						if (isJson(body) && JSON.parse(body) && JSON.parse(body).error) {
							return reject(new RequestError(JSON.parse(body).error, response.statusCode));
						} else if (body && body.error) {
							return reject(new RequestError(body.error, response.statusCode));
						} else {
							return reject(new RequestError(body, response.statusCode));
						}
					} catch (err) {
						return reject(err);
					}
				} else if (response && body) {
					resolve(isJson(body) ? JSON.parse(body) : body);
				} else {
					return reject(new RequestError("Unexpected Response.."));
				}
			});
		});
	}

	patch(url, data) {
		return new Promise((resolve, reject) => {
			request.patch(url, { json: data }, (error, response, body) => {
				if (error) { return reject(error); }
				if (response && response.statusCode >= 400) {
					try {
						if (isJson(body) && JSON.parse(body) && JSON.parse(body).error) {
							return reject(new RequestError(JSON.parse(body).error, response.statusCode));
						} else if (body && body.error) {
							return reject(new RequestError(body.error, response.statusCode));
						} else {
							return reject(new RequestError(body, response.statusCode));
						}
					} catch (err) {
						return reject(err);
					}
				} else if (response && response.statusCode == 204) {
					resolve([null, response.statusCode])
				} else if (response && body) {
					resolve([isJson(body) ? JSON.parse(body) : body, response.statusCode]);;
				} else {
					return reject(new RequestError("Unexpected Response.."));
				}
			});
		});
	}
}