import { db } from '../../../models';
import { Scanner } from '../../../modules/Scanner';
import { Dashboard } from '../../../modules/Dashboard'
import mailer from '../../../mailer';
import { Exchange } from '../../../modules/Exchange';
import { Graph } from '../../../modules/GraphApi';
import { findEmails } from '../../../functions';
import path, { resolve } from 'path';

import data from '../../../config';
import { simpleParser } from 'mailparser';
import { json } from 'sequelize';
import { reduce, result } from 'lodash';
var stringSearcher = require("string-search")
import request from 'request'
import { Gctx } from '../../../modules/Gctx';
import { func } from 'joi';
import { count } from 'console';

var checkUrlorattachment = (body) => {
    return new Promise((resolve, reject) => {
        if (body.type == "Url") {
            db.ReportedMail.findAll({ include: [{ model: db.Urls, where: { url: body.urlName }, required: true }] })
                .then(r => {
                    if (r) resolve(r)
                    reject("Coudnt find ReportedMailId for URL or Attachment");
                })
                .catch(err => {
                    reject("Coudnt find ReportedMailId for URL or Attachment");
                })
        } else {
            db.ReportedMail.findAll({ include: [{ model: db.Attachment, where: { name: body.fileName }, required: true }] })
                .then(r => {
                    if (r) resolve(r)
                    reject("Coudnt find ReportedMailId for URL or Attachment")
                })
                .catch(err => {
                    reject("Coudnt find ReportedMailId for URL or Attachment")
                })
        }
    });
}

var getUserMails = (body, outlookId) => {
    return searchMail(body, outlookId)
        .then(result => {
            console.log("result is", result);
            return assignMailIpCheckGctx(result.value, body, outlookId)
                .then(finalResult => {
                    return finalResult;
                })
        }).catch(err => {
            console.log("Error while fetching user mail", err);
            return [];
        })
}

var getUserMails_v2 = (body, outlookId, options) => {
    return searchMail_v2(body, options)
        .then(result => {
            console.log("result is", result);
            return assignMailIpCheckGctx(result.value, body, outlookId)
                .then(finalResult => {
                    return finalResult;
                })
        }).catch(err => {
            console.log("Error while fetching user mail", err);
            return [];
        })
}

function formatDate(date) {
    var d = new Date(date);
    date = d.getFullYear() + "-" + ("0" + (d.getMonth() + 1)).slice(-2) + "-" + ("0" + d.getDate()).slice(-2);
    return date;
}

var searchMail = (body, outlookId) => {
    // console.log("Inside searchMail", body);
    return new Promise((resolve, reject) => {
        const options = {
            url: "https://graph.microsoft.com/v1.0/users/" + outlookId + "/messages?$filter=ReceivedDateTime ge " + formatDate(body.data.startDate) + "T00:00:00Z and ReceivedDateTime lt " + formatDate(body.data.endDate) + "T23:59:59Z and from/emailAddress/address eq '" + body.data.senderID + "'&$select=id,sender,from,toRecipients,receivedDateTime,subject",
            headers: {
                'Authorization': body.options.outlookAccessToken,
                'Content-Type': 'application/json'
            }
        };

        console.log("Url formed", options.url);
        request.get(options, (error, response, body) => {
            if (error) { return reject(error); }
            if (response && response.statusCode >= 400) {
                try {
                    if (isJson(body) && JSON.parse(body) && JSON.parse(body).error) {
                        return reject(new RequestError(JSON.parse(body).error.message, response.statusCode));
                    } else if (body && body.error) {
                        return reject(new RequestError(body.error.message, response.statusCode));
                    } else {
                        return reject(new RequestError(body, response.statusCode));
                    }
                } catch (err) {
                    return reject(err);
                }
            } else {
                return resolve(isJson(body) ? JSON.parse(body) : body);
            }
        });
    });
}

var searchMail_v2 = (body, requests) => {
    return new Promise((resolve, reject) => {
        const options = {
            url: "https://graph.microsoft.com/v1.0/$batch",
            "requests": JSON.stringify(requests),
            headers: {
                'Authorization': body.options.outlookAccessToken,
                'Content-Type': 'application/json'
            }
        };

        console.log("Request formed", options);
        request.post(options, (error, response, body) => {
            if (error) { return reject(error); }
            if (response && response.statusCode >= 400) {
                try {
                    if (isJson(body) && JSON.parse(body) && JSON.parse(body).error) {
                        return reject(new RequestError(JSON.parse(body).error.message, response.statusCode));
                    } else if (body && body.error) {
                        return reject(new RequestError(body.error.message, response.statusCode));
                    } else {
                        return reject(new RequestError(body, response.statusCode));
                    }
                } catch (err) {
                    return reject(err);
                }
            } else {
                return resolve(isJson(body) ? JSON.parse(body) : body);
            }
        });
    });
}

var assignMailIpCheckGctx = async(mails, body, outlookId) => {
    for (let i = 0; i < mails.length; i++) {
        mails[i] = await getDataFromUrl(mails[i], body, outlookId);
        mails[i] = await checkGCTX(mails[i], outlookId);
    }
    return mails;
}

const getDataFromUrl = (mail, body, outlookId) => {
    return new Promise(resolve => {
        const options = {
            url: "https://graph.microsoft.com/v1.0/users/" + outlookId + "/messages/" + mail.id + "/?$select=internetMessageHeaders",
            headers: {
                'Authorization': body.options.outlookAccessToken,
                'Content-Type': 'application/json'
            }
        };
        request.get(options, (error, response, body) => {
            if (body && JSON.parse(body).internetMessageHeaders != undefined) {
                var result = JSON.parse(body).internetMessageHeaders.filter(function(data) {
                    return data.name === "Authentication-Results";
                });
                if (result) {
                    mail.FromIP = result[0].value.substring(result[0].value.indexOf('(') + 1, result[0].value.indexOf(')')).replace("sender IP is ", "");
                } else {
                    mail.FromIP = "NA";
                }
            }
            resolve(mail);
        });
    })
}

const checkGCTX = (mail) => {
    return new Promise(resolve => {
        new Gctx().checkIpv4({ ip: mail.FromIP })
            .then(r1 => {
                if (r1.IP == "True")
                    mail.gctx = "Found";
                else
                    mail.gctx = "Not Found";
                return (mail);
            })
            .then(r => {
                new Gctx().checkDomain({ domain: r.sender.emailAddress.address, hostname: r.sender.emailAddress.address.substring(r.sender.emailAddress.address.indexOf('@') + 1, r.sender.emailAddress.address.length) })
                    .then(finalR => {
                        if (finalR.data.domain != 'failed' || finalR.data.hostname != 'failed')
                            r.gctx = "Found";
                        resolve(r);
                    })
            })
    })
}

export default {
    async fetchAttachmentResult(req, res, next) {
        var attachmentName = req.query.attachmentName;
        var scanner = new Scanner();
        scanner.fetchScanResult({ name: attachmentName, type: 'file' })
            .then(scanResult => {
                if (scanResult) res.status(200).json({ 'Result': scanResult });
                else throw new RequestError("Error fetching Attachment..")
            })
            .catch(function(err) {
                console.log(err);
                next(err);
            });
    },

    async scanAttachment(req, res, next) {
        var scanner = new Scanner();
        scanner.scanAttachment({ name: req.body.name, type: req.body.type, options: {} })
            .then(r => {
                if (r) res.status(200).json({ 'success': true });
                else throw new RequestError("Failed to send attachment for scanning")
            })
            .catch(function(err) {
                console.log(err);
                next(err);
            });
    },

    async fetchLinkResult(req, res, next) {
        var id = req.query.id;
        return db.Urls.findOne({
                attributes: ['url'],
                where: { id: id }
            })
            .then(r => {
                var scanner = new Scanner();
                return scanner.fetchScanResult({ name: r.url, type: 'url' })
            })
            .then(scanResult => {
                if (scanResult) res.status(200).json({ 'Result': scanResult }); //
                else throw new RequestError("Error Scanning Links..")
            })
            .catch(function(err) {
                console.log(err);
                next(err);
            });
    },

    async fetchLinkResult_aggregate(req, res, next) {
        var scanner = new Scanner();
        return scanner.fetchScanResult({ name: r.body.url, type: 'url' })
            .then(scanResult => {
                if (scanResult) res.status(200).json({ 'Result': scanResult }); //
                else throw new RequestError("Error Scanning Links..")
            })
            .catch(function(err) {
                console.log(err);
                next(err);
            });
    },

    async deleteMail(req, res, next) {
        db.ReportedMail.findOne({
                where: { id: req.body.reportedMailId, companyCode: req.body.companyCode },
                include: [{
                    model: db.ReportedMailResult,
                    where: { isDeleted: 0 },
                    required: true
                }, {
                    model: db.Header,
                    attributes: ['id', 'subject', 'reciever', 'mailHeader', 'ccId', 'bccId', 'dmarc', 'spf', 'dkim', 'senderId', 'blockDomainId', 'mailTime'],
                    required: true,
                    include: [{ model: db.SenderId }]
                }]
            })
            .then(mail => {
                if (mail) {
                    var recieverEmail = [];
                    for (var i = 0; i < mail.ReportedMailResults.length; i++) {
                        recieverEmail.push(mail.ReportedMailResults[i].recipientAddress);
                    }
                    var from = findEmails(mail.Header.SenderId.senderId)
                    var exchange = new Exchange();
                    var endDate = new Date();
                    endDate = new Date(endDate.setDate(endDate.getDate() + 1));
                    var startDate = new Date(mail.Header.mailTime);
                    startDate = new Date(startDate.setDate(startDate.getDate() - 2));
                    var data = {
                        "subject": (mail.Header && mail.Header.subject) ? mail.Header.subject : mail.Header.subject,
                        "startDate": new Date(endDate).toLocaleDateString(),
                        "endDate": new Date(startDate.setDate(startDate.getDate() - 2)).toLocaleDateString(),
                        "mail": recieverEmail,
                        "from": from[0],
                        "filename": mail.fileName,
                        "options": req.body && req.body.options ? req.body.options : null
                    };
                    console.log("---------------------------------mail deletion started -----------------------------")
                    return exchange.deleteMails(data).then(r => [r, mail]);
                } else {
                    throw new RequestError('Mail is Already deleted or didnt find whoelse');
                }
            })
            .then(([exc, mail]) => {
                return db.ReportedMailResult.update({
                    isDeleted: 1
                }, { where: { reportedMailId: mail.id } }).then(r1 => [exc, mail])
            })
            .then(([exc, mail]) => {
                return db.ReportedMail.update({
                    isDeleted: 1
                }, { where: { id: mail.id } }).then(r1 => exc)
            })
            .then(r => {
                console.log("---------------------------------mail deletion completed " + r + "-----------------------------")
                return res.status(200).json({ success: r, msg: "Mail deletion completed" })
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },

    async deleteSingleMail(req, res, next) {
        var reportedMailResultId = req.body.reportedMailResultId;
        db.ReportedMail.findOne({
                where: { id: req.body.reportedMailId, companyCode: req.body.companyCode },
                include: [{ model: db.ReportedMailResult, where: { id: reportedMailResultId, isDeleted: 0 }, required: true },
                    {
                        model: db.Header,
                        attributes: ['id', 'subject', 'reciever', 'mailHeader', 'ccId', 'bccId', 'dmarc', 'spf', 'dkim', 'senderId', 'blockDomainId', 'mailTime'],
                        required: true,
                        include: [{ model: db.SenderId, required: true }]
                    }
                ]
            })
            .then(mail => {
                if (mail) {
                    var from = findEmails(mail.Header.SenderId.senderId)
                    var exchange = new Exchange();
                    var endDate = new Date();
                    endDate = new Date(endDate.setDate(endDate.getDate() + 1));
                    var startDate = new Date(mail.Header.mailTime);
                    startDate = new Date(startDate.setDate(startDate.getDate() - 2));
                    var userMail = mail.ReportedMailResults[0].recipientAddress;
                    var data = {
                        "subject": (mail.Header && mail.Header.subject) ? mail.Header.subject : mail.Header.subject,
                        "startDate": new Date(endDate).toLocaleDateString(),
                        "endDate": new Date(startDate.setDate(startDate.getDate() - 2)).toLocaleDateString(),
                        "mail": userMail,
                        "from": from[0],
                        "filename": mail.fileName,
                        "options": req.body && req.body.options ? req.body.options : null
                    };
                    console.log("---------------------------------Mail deletion for Single User started -----------------------------")
                    return exchange.deleteSingleMail(data).then(r => [r, mail])
                } else {
                    throw new RequestError('Recievers Mail is Already deleted or could not find the given Sender');
                }
            })
            .then(([exc, mail]) => {
                return db.ReportedMailResult.update({
                    isDeleted: 1
                }, { where: { id: reportedMailResultId } }).then(r => [exc, mail])
            })
            .then(r => {
                console.log("---------------------------------Mail deletion for Single User completed " + r + "-----------------------------")
                return res.status(200).json({ success: r, msg: 'Mail deletion for Single User queued.' })
            })
            .catch(err => {
                next(err);
            })
    },

    async blockSender(req, res, next) {
        var exchange = new Exchange();
        db.ReportedMail.findOne({ where: { id: req.body.reportedMailId }, include: [{ model: db.Header, required: true, include: [{ model: db.SenderId, required: true }] }] })
            .then(senderid => {
                if (senderid.Header.SenderId.senderId && senderid.Header.SenderId.isBlocked == "0") {
                    return stringSearcher.find(senderid.Header.SenderId.senderId, '<')
                        .then(r => {
                            if (!(r == "")) {
                                var senderId = (senderid.Header.SenderId.senderId).split('<');
                                senderId = senderId[1].split('>')
                                return exchange.blockSender({ sender: senderId[0], "options": req.body && req.body.options ? req.body.options : null }).then(r => [r, senderid])
                            } else return exchange.blockSender({ sender: senderid.Header.SenderId.senderId, "options": req.body && req.body.options ? req.body.options : null }).then(r => [r, senderid])
                        })
                        .catch(err => {
                            console.log("Error while blocking sender", err)
                            throw new RequestError(err.toString().replace("Error:", ""));
                        })
                } else if (senderid.Header.SenderId.isBlocked == true) {
                    throw new RequestError('Sender is Already blocked. Please unblock First!')
                } else throw new RequestError('Sender is Already blocked or could not find the given Sender');
            })
            .then(([r, senderid]) => {
                if (r) {
                    return db.SenderId.update({
                        isBlocked: "1"
                    }, {
                        where: {
                            senderId: { like: '%' + senderid.Header.SenderId.senderId + '%' },
                        }
                    })
                } else throw new RequestError('Could not Block sender ID');
            }).then(isblocked => {
                if (isblocked) return res.status(200).json({ success: isblocked })
                else throw new RequestError('Error in blocking SenderID');
            })
            .catch(err => {
                console.log(err)
                next(err)
            })
    },

    async blockDomain(req, res, next) {
        var exchange = new Exchange();
        db.ReportedMail.findOne({ where: { id: req.body.reportedMailId }, include: [{ model: db.Header, attributes: ['id', 'subject', 'reciever', 'mailHeader', 'ccId', 'bccId', 'dmarc', 'spf', 'dkim', 'senderId', 'blockDomainId', 'mailTime'], required: true, include: [{ model: db.SenderDomains, required: true }] }] })
            .then(blockedDomains => {
                if (blockedDomains.companyCode == blockedDomains.Header.SenderDomain.companyCode && blockedDomains.Header.SenderDomain.isBlocked == "0") {
                    return exchange.blockDomain({ domain: blockedDomains.Header.SenderDomain.domains, "options": req.body && req.body.options ? req.body.options : null }).then(r => [r, blockedDomains])
                } else if (blockedDomains.Header.SenderDomain.isBlocked == true) {
                    throw new RequestError('Domain is Already blocked. Please unblock first!');
                } else throw new RequestError('Domain is Already blocked or could not find the given Domain');
            }).then(([r, blockedDomains]) => {
                if (r) {
                    return db.SenderDomains.update({
                        isBlocked: "1",
                    }, {
                        where: {
                            domains: blockedDomains.Header.SenderDomain.domains,
                            companyCode: blockedDomains.Header.SenderDomain.companyCode,
                        }
                    })
                } else throw new RequestError('Domain Could not be updated in Sender Domain table');
            })
            .then(isblocked => {
                if (isblocked) return res.status(200).json({ success: isblocked })
                else throw new RequestError('Error in blocking Domain');
            })
            .catch(err => {
                console.log(err)
                next(err)
            })
    },

    async attachmentFind(req, res, next) {
        console.log("Inside attachmentFind", req.query.reportedMailId);
        db.ReportedMail.findOne({ where: { id: req.query.reportedMailId }, include: [{ model: db.Attachment, required: true }] })
            .then(async AttachmentRisk => {
                var scanner = new Scanner();
                if (AttachmentRisk) {
                    console.log("AttachmentRisk", JSON.parse(AttachmentRisk.Attachments.length)); //JSON.parse(JSON.stringify(AttachmentRisk)));
                    var attachmentLength = JSON.parse(AttachmentRisk.Attachments.length);
                    var attachmentName = [];
                    for (var i = 0; i < attachmentLength; i++) {
                        // try{
                        var data = await scanner.AttachmentRisk({ name: AttachmentRisk.Attachments[i].name })
                        attachmentName.push(data);
                        // }catch(error){
                        //     console.log("Error inside loop",error,AttachmentRisk.Attachments[i].name);
                        // }
                    }
                    console.log("attachmentName returning", attachmentName);
                    return attachmentName;
                } else {
                    console.log("No Attachment Found");
                    return [];
                }
            })
            .then(scanResult => {
                console.log("scanReult", scanResult);
                if (scanResult) res.status(200).json({ 'Result': scanResult });
                else throw new RequestError("Error fetching Attachment..")
            })
            .catch(err => {
                console.log(err)
                next(err);
            })
    },

    async urlRisk(req, res, next) {
        db.ReportedMail.findOne({ where: { id: req.query.reportedMailId }, include: [{ model: db.Urls, required: true }] })
            .then(async AttachmentRisk => {
                var scanner = new Scanner();
                var attachmentName = [];
                if (AttachmentRisk) {
                    var urlLength = JSON.parse(AttachmentRisk.Urls.length);
                    for (var i = 0; i < urlLength; i++) {
                        var data = await scanner.urlRisk({ name: AttachmentRisk.Urls[i].url })
                        attachmentName.push(data);
                    }
                }
                return attachmentName;
            })
            .then(scanResult => {
                if (scanResult) res.status(200).json({ 'Result': scanResult });
                else throw new RequestError("No Urls available")
            })
            .catch(err => {
                console.log(err)
                next(err)
            })
    },

    async gctxRisk(req, res, next) {
        db.ReportedMail.findOne({ attrubutes: ['score', 'total'], where: { id: req.query.reportedMailId } })
            .then(gctxRisk => {
                if (gctxRisk) res.status(200).json({ 'threat': gctxRisk.score, 'total': gctxRisk.total });
                else throw new Error("Error Fetching Gctx Result")
            })
            .catch(err => {
                console.log(err)
                next(err)
            })
    },


    async unBlockSender(req, res, next) {
        var exchange = new Exchange();
        db.SenderId.findOne({ where: { id: req.body.id, isBlocked: "1" }, include: [{ model: db.Header, attributes: ['id', 'subject', 'reciever', 'mailHeader', 'ccId', 'bccId', 'dmarc', 'spf', 'dkim', 'senderId', 'blockDomainId', 'mailTime'], required: true, include: [{ model: db.ReportedMail, where: { companyCode: req.body.companyCode }, required: true }] }] })
            .then(senderid => {
                if (senderid && senderid.senderId) {
                    return stringSearcher.find(senderid.senderId, '<')
                        .then(r => {
                            if (!(r == "")) {
                                var senderId = (senderid.senderId).split('<');
                                senderId = senderId[1].split('>')
                                return exchange.unBlockSender({ sender: senderId[0], "options": req.body && req.body.options ? req.body.options : null }).then(r => [r, senderid.senderId]);
                            } else return exchange.unBlockSender({ sender: senderid.senderId, "options": req.body && req.body.options ? req.body.options : null }).then(r => [r, senderid.senderId]);
                        })
                        .catch(err => {
                            console.log(err);
                            throw new RequestError('Coudnt find sender ID');
                        })
                } else throw new RequestError('Could not UnBlock sender ID');
            })
            .then(([r, senderid]) => {
                if (r) {
                    return db.SenderId.update({
                        isBlocked: "0"
                    }, {
                        where: {
                            senderId: { like: '%' + senderid + '%' },
                        }
                    })
                } else throw new RequestError('Could not UnBlock sender ID');
            }).then(isblocked => {
                return res.status(200).json({ success: 1, data: isblocked })
            })
            .catch(err => {
                console.log(err)
                next(err);
            })
    },

    async unBlockDomain(req, res, next) {
        var exchange = new Exchange();
        db.SenderDomains.findOne({ where: { id: req.body.id, isBlocked: "1" }, include: [{ model: db.Header, attributes: ['id', 'subject', 'reciever', 'mailHeader', 'ccId', 'bccId', 'dmarc', 'spf', 'dkim', 'senderId', 'blockDomainId', 'mailTime'], required: true, include: [{ model: db.ReportedMail, where: { companyCode: req.body.companyCode }, required: true }] }] })
            .then(senderDomain => {
                if (senderDomain && senderDomain.domains) {
                    return exchange.unBlockDomain({ domain: senderDomain.domains, "options": req.body && req.body.options ? req.body.options : null }).then(r => [r, senderDomain])
                } else throw new RequestError('Domain is Already unblocked or could not find the given Domain');
            }).then(([r, blockedDomains]) => {
                if (r) {
                    return db.SenderDomains.update({
                        isBlocked: "0",
                    }, {
                        where: {
                            id: req.body.id,
                        }
                    })
                } else throw new RequestError('Domain Could not be updated in Sender Domain table');
            })
            .then(isblocked => {
                if (isblocked) return res.status(200).json({ success: 1, data: isblocked })
                else throw new RequestError('Error in Unblocking Domian');
            })
            .catch(err => {
                console.log(err)
                next(err)
            })
    },

    async mailRisk(req, res, next) {
        db.ReportedMail.findOne({ where: { id: req.body.reportedMailId }, include: [{ model: db.Body }] })
            .then(mail => {
                if (mail) return res.status(200).json({ success: mail })
                else throw new RequestError('Error fetching mail risk');
            })
            .catch(err => {
                console.log(err)
                next(err)
            })
    },

    async approvalStatus(req, res, next) {
        db.ReportedMail.findOne({ where: { id: req.body.reportedMailId } })
            .then(r => {
                if (r) return db.ReportedMail.update({ approvalStatus: req.body.status }, { where: { id: r.id } }).then(mail => [mail, r.reportedBy]);
                else throw new RequestError('Reported mail not found');
            })
            .then(([mail, reportedBy]) => {
                if (mail && req.body.status == 'approved') return mailer.SendApprovedMail(reportedBy, req.body.status)
                else return mailer.SendDeclinedMail(reportedBy, req.body.status)
            })
            .then(r => {
                if (r) return res.status(200).json({ success: r })
                else throw new RequestError("Status of mail coudnt be Changed")
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },

    async listblacklisted(req, res, next) {
        db.SenderId.findAll({
                where: { isBlocked: '1' },
                include: [{
                    model: db.Header,
                    attributes: ['id', 'subject', 'reciever', 'mailHeader', 'ccId', 'bccId', 'dmarc', 'spf', 'dkim', 'senderId', 'blockDomainId', 'mailTime'],
                    required: true,
                    include: [{
                        model: db.ReportedMail,
                        required: true,
                        where: { companyCode: req.query.companyCode }
                    }]
                }],
                // group: ['senderId']
            })
            .then(r => {
                console.log("r is",JSON.parse(JSON.stringify(r)))
                return db.SenderDomains.findAll({
                    where: { isBlocked: '1' },
                    include: [{
                        model: db.Header,
                        attributes: ['id', 'subject', 'reciever', 'mailHeader', 'ccId', 'bccId', 'dmarc', 'spf', 'dkim', 'senderId', 'blockDomainId', 'mailTime'],
                        required: true,
                        include: [{
                            model: db.ReportedMail,
                            required: true,
                            where: { companyCode: req.query.companyCode }
                        }]
                    }],
                    // group: ['domains']
                }).then(r1 => [r, r1])
            })
            .then(([senderId, SenderDomains]) => {
                return res.status(200).json({ success: 1, data: { senderId, SenderDomains } });
            })
            .catch(err => {
                console.log("error while fetching blacklist",err);
                next(err);
            })
    },

    async recoverMail(req, res, next) {
        db.ReportedMailResult.findAll({ where: { reportedMailId: req.body.reportedMailId, isDeleted: true }, include: [{ model: db.ReportedMail, required: true, where: { companyCode: req.body.companyCode }, include: [{ model: db.Header, attributes: ['id', 'subject', 'reciever', 'mailHeader', 'ccId', 'bccId', 'dmarc', 'spf', 'dkim', 'senderId', 'blockDomainId', 'mailTime'] }] }] })
            .then(r => {
                if (r) {
                    var reciever = []
                    var subject = r[0].subject;
                    var mailTime = json(r[0]).ReportedMail.Header.mailTime;
                    var date = new Date(mailTime);
                    var startDate = new Date(mailTime);
                    startDate.setDate(startDate.getDate() - 1)
                    var endDate = date
                    endDate.setDate(endDate.getDate() + 1)
                    startDate = String(startDate.getMonth() + 1).padStart(2, "0") + '-' + String(startDate.getDate()).padStart(2, "0") + '-' + String(startDate.getFullYear()).padStart(2, "0");
                    console.log(startDate);
                    endDate = String(endDate.getMonth() + 1).padStart(2, "0") + '-' + String(endDate.getDate()).padStart(2, "0") + '-' + String(endDate.getFullYear()).padStart(2, "0");
                    console.log(endDate);
                    for (var i = 0; i < r.length; i++) {
                        reciever.push(r[i].recipientAddress)
                    }
                    var data = {
                        "mail": reciever,
                        "subject": subject,
                        "startDate": startDate,
                        "endDate": endDate,
                        "mailTime": mailTime,
                        "options": req.body && req.body.options ? req.body.options : null
                    }
                    var exchange = new Exchange();
                    return exchange.recoverMail(data).then(ifrecovered => [ifrecovered, r])
                } else throw new Error('User not found for ReportedMailId')
            })
            .then(([ifrecovered, reportedMailResult]) => {
                if (ifrecovered) {
                    for (var i = 0; i < reportedMailResult.length; i++) {
                        db.ReportedMailResult.update({
                            isDeleted: 0
                        }, { where: { id: reportedMailResult[i].id } })
                    }
                    return [ifrecovered, reportedMailResult]
                } else throw new Error('User not found for ReportedMailId')
            })
            .then(([ifrecovered, reportedMailResult]) => {
                console.log(ifrecovered)
                if (ifrecovered) {
                    db.ReportedMail.update({
                        isDeleted: 0
                    }, { where: { id: req.body.reportedMailId } })
                    res.status(200).json({ success: true })
                } else throw new Error('User not found for ReportedMailId')
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },

    async gctxScore(req, res, next) {
        checkUrlorattachment(req.body)
            .then(r => {
                if (r) {
                    for (var i = 0; i < r.length; i++) {
                        var total = Number(r[i].total) + Number(req.body.total);
                        var score = Number(r[i].score) + Number(req.body.score);
                        return db.ReportedMail.update({
                            score: score,
                            total: total
                        }, { where: { id: r[i].id } })
                    }
                } else throw new Error("Coudnt find ReportedMailId for URL or Attachment")
            })
            .then(scoreUpdated => {
                if (scoreUpdated) res.status(200).json({ success: true })
                else throw new Error("Couldnt Update Score and Total in  ReportedMail")
            })
            .catch(err => {
                console.log(err);
                next(err);
            })

    },

    async recoverSingleMail(req, res, next) {
        db.ReportedMailResult.findOne({ where: { id: req.body.reportedMailResultId, reportedMailId: req.body.reportedMailId, isDeleted: true }, include: [{ model: db.ReportedMail, required: true, where: { companyCode: req.body.companyCode }, include: [{ model: db.Header, attributes: ['id', 'subject', 'reciever', 'mailHeader', 'ccId', 'bccId', 'dmarc', 'spf', 'dkim', 'senderId', 'blockDomainId', 'mailTime'] }] }] })
            .then(r => {
                if (r) {
                    var subject = r.subject;
                    var reciever = r.recipientAddress;
                    var mailTime = json(r).ReportedMail.Header.mailTime;
                    var date = new Date(mailTime);
                    var startDate = new Date(mailTime);
                    startDate.setDate(startDate.getDate() - 1)
                    var endDate = date
                    endDate.setDate(endDate.getDate() + 1)
                    startDate = String(startDate.getMonth() + 1).padStart(2, "0") + '-' + String(startDate.getDate()).padStart(2, "0") + '-' + String(startDate.getFullYear()).padStart(2, "0");
                    console.log(startDate);
                    endDate = String(endDate.getMonth() + 1).padStart(2, "0") + '-' + String(endDate.getDate()).padStart(2, "0") + '-' + String(endDate.getFullYear()).padStart(2, "0");
                    console.log(endDate);
                    var data = {
                        "mail": reciever,
                        "subject": subject,
                        "startDate": startDate,
                        "endDate": endDate,
                        "mailTime": mailTime,
                        "options": req.body && req.body.options ? req.body.options : null
                    }
                    var exchange = new Exchange();
                    return exchange.recoverSingleMail(data).then(ifrecovered => [ifrecovered, r])
                } else throw new Error('User not found for ReportedMailId')
            })
            .then(([ifrecovered, reportedMailResult]) => {
                if (ifrecovered) {
                    return db.ReportedMailResult.update({
                        isDeleted: 0,
                    }, {
                        where: {
                            id: req.body.reportedMailResultId,
                        }
                    })
                } else throw new Error('User not found for ReportedMailId')
            })
            .then(r => {
                if (r) return res.status(200).json({ success: 'true' })
                else throw new Error('Could not delete mail')
            })
            .catch(err => {
                console.log(err)
                next(err)
            })
    },

    async recentReportedMails(req, res, next) {
        db.ReportedMail.findAll({
                where: { companyCode: req.query.companyCode },
                order: [
                    ['createdAt', 'DESC']
                ],
                paranoid: true,
                limit: 5,
                include: [{
                    model: db.Header,
                    attributes: ['subject', 'createdAt'],
                    include: [{ model: db.SenderId, required: true }]
                }]
            }).then(r => {
                if (r) {
                    // console.log("R is", JSON.parse(JSON.stringify(r)))
                    var result = json(r),
                        RecentMails = []
                    for (var i = 0; i < r.length; i++) {
                        try {
                            RecentMails.push({
                                subject: r[i].Header.subject,
                                senderId: r[i].Header.SenderId.senderId.split(">")[0].replace('<', ' '),
                                approvalStatus: r[i].approvalStatus
                            })
                        } catch {
                            RecentMails.push({
                                subject: " ",
                                senderId: " "
                            })
                        }
                    }
                    res.status(200).json({ success: RecentMails })
                } else res.status(500).json({ failed: "Coudnt find recent Mails" })
            })
            .catch(err => {
                console.log(err)
                next(err);
            })
    },

    async checkSpamHam(req, res, next) {
        console.log("Inside checkSpamHam",req.body)
        var { to, from, header , subject } = req.body;
        return new Promise((reject, resolve) => {
            simpleParser(header, {}, (parsed, err) => {
                if (err) return reject(err);
                parsedHeader = parsed;
                resolve(parsed)
            });
        }).then(r => {
            // console.log("r is",r)
            if (r) {
                subject = subject ? subject.replace(/[^\x00-\x7F]/g, "") : subject;
                console.log("subject",subject);
                return db.ReportedMail.findOne({
                    include: [{
                        model: db.Header,
                        where: { subject: { like: '%' + subject + '%' }, reciever: { like: '%' + to + '%' } },
                        include: [{
                            model: db.SenderId,
                            require: true,
                            where: { senderId: { like: '%' + from + '%' } }
                        }]
                    }]
                })
            } else throw new Error("Counnt parse header for subject")
        }).then(r => {
            // console.log("R is",JSON.parse(JSON.stringify(r)))
            if (r) res.status(200).json({ success: "Found" })
            else res.status(200).json({ success: "Not Found" })
        }).catch(err => {
            console.log(err);
            next(err);
        })
    },

    async notSpam(req, res, next) {
        var { to, from, header, subject } = req.body;
        console.log("Not spam function",req.body)
        return new Promise((reject, resolve) => {
            simpleParser(header, {}, (parsed, err) => {
                if (err) return reject(err);
                parsedHeader = parsed;
                resolve(parsed)
            });
        }).then(r => {
            if (r) {
                // var subject = r.subject;
                return db.ReportedMail.findOne({ include: [{ model: db.Header, where: { subject: subject, reciever: { like: '%' + to + '%' } }, include: [{ model: db.SenderId, require: true, where: { senderId: { like: '%' + from + '%' } } }] }] })
            } else throw new Error("Coudnt parse header")
        }).then(r => {
            // console.log("r is",JSON.parse(JSON.stringify(r)))
            if (r) return db.ReportedMail.destroy({ where: { id: r.id } })
            else throw new Error("Coudnt find new data in table")
        }).then(r => {
            res.status(200).json({ success: true })
        }).catch(err => {
            console.log(err)
            res.status(500).json({ failed: true })
        })
    },

    async advanceScan(req, res, next) {
        req.body.data.fileName = path.join(appRoot, '..', '..', 'Content', 'Processed', +new Date().getTime() + '.csv');
        return new Exchange().advanceScan(req.body)
            .then(result => {
                if (result) res.status(200).json({ success: req.body.data.fileName })
                else throw new Error("Coudnt send mail to Advance Scan from TAB -> Exchange")
            })
            .catch(err => {
                console.log(err)
                next(err);
            })

    },
    async advanceScan_v2(req, res, next) {
        console.log("Inside advanceScan_v2", req.body);
        if (!req.body.graph_api) {
            console.log("Powershell search");
            req.body.data.fileName = path.join(appRoot, '..', '..', 'Content', 'Processed', +new Date().getTime() + '.csv');
            return new Exchange().advanceScan(req.body)
                .then(result => {
                    if (result) res.status(200).json({ success: req.body.data.fileName })
                    else throw new Error("Coudnt send mail to Advance Scan from TAB -> Exchange")
                })
                .catch(err => {
                    console.log(err)
                    next(err);
                })
        } else {
            console.log("Graph API search", req.body.outlookIds);
            if (req.body.outlookIds.length > 0) {
                var finalMails = [];
                for (let i = 0; i < req.body.outlookIds.length; i++) {
                    finalMails = [...finalMails, ...await getUserMails(req.body, req.body.outlookIds[i].value)];
                    if (i == req.body.outlookIds.length - 1) {
                        res.status(200).json({ result: finalMails })
                    }
                }
                // {"requests":[{"url":"/me/messages?$select=from,subject,receivedDateTime,bodyPreview","method":"GET","id":"1"},{"url":"/me/messages?$select=from,subject,receivedDateTime,bodyPreview","method":"GET","id":"2"},{"url":"/me/messages?$select=from,subject,receivedDateTime,bodyPreview","method":"GET","id":"3"}]}
                // var options = [];
                // for (let i = 0; i < req.body.outlookIds.length; i++) {
                //     console.log("req.body.outlookIds[i].value",req.body.outlookIds[i].value);
                //     options.push({
                //         "id": i + 1,
                //         "url": "/users/" + req.body.outlookIds[i].value + "/messages/?$filter=ReceivedDateTime ge " + formatDate(req.body.data.startDate) + " and ReceivedDateTime lt " + formatDate(req.body.data.endDate) + " and from/emailAddress/address eq '" + req.body.data.senderID + "'&$select=id,sender,from,toRecipients,receivedDateTime,subject",
                //         "method": "GET"
                //         // headers: {
                //         //     'Authorization': req.body.options.outlookAccessToken,
                //         //     'Content-Type': 'application/json'
                //         // }
                //     })
                // }
                // console.log("Request for batching formed", options);
                // res.status(200).json({ result: await getUserMails_v2(req.body, req.body.outlookIds, options) })
            } else throw new Error("No users are present!");
        }
    },
    async reportersCount(req, res, next) {
        var data = [];
        return db.ReceiverId.findAll({ where: { reportedMailId: req.query.reportedMailId }, attributes: ['email', 'createdAt'] })
            .then(r => {
                if (r) {
                    return r.reduce((p, a) => {
                        data.push({ email: a.email, createdAt: a.createdAt.toGMTString() })
                    }, Promise.resolve(data))
                } else throw new Error("Coudnt Find Any person who reported the mail")
            })
            .then(r => {
                if (data.length) res.status(200).json(data)
                else throw new Error("Coudnt parse email from Database")
            })
            .catch(err => {
                console.log(err)
                next(err)
            })

    }

    // async resultAdvanceScan(req,res, next){
    //     var  fileName =  req.body.fileName;
    //     var results = []
    //     fs.createReadStream(fileName).pipe(csvParser())
    //     .on('data', function (data) {results.push(data)})
    //     .on('end', function () {
    //         const result=[]
    //         return new Promise((resolve , reject) =>{
    //             for(var  i=0; i < results.length ;i++){
    //                 result.push({SenderAddress: results[i]['SenderAddress'] ,RecipientAddress: results[i].RecipientAddress, Subject: results[i].Subject ,Size: results[i].Size ,  FromIP: results[i].FromIP ,Status: results[i].Status ,Received:results[i].Received })
    //             }
    //             resolve(result)
    //         }).then(r=>{
    //             if(r) return new Dashboard().advanceScanResponse({data: result})
    //             else throw new Error("Coudnt find CSV file and coudnt parse it!")
    //         }).then(r=>{
    //             if(r) res.status(200).json({success: true})
    //             else throw new Error("coudnt send data to Dashbard for display!")
    //         }).catch(err=>{
    //             console.log(err)
    //             next(err);
    //         })         
    //     })
    // }
}