import kue from 'kue';
import fs from 'fs'
import path, { parse } from 'path';
import config from './config'
import { simpleParser } from 'mailparser';
import { db } from './models';
import { Scanner } from './modules/Scanner';
import getUrls from 'get-urls';
import urlParse from 'url-parse';
import { Exchange } from './modules/Exchange';
import { Dashboard } from './modules/Dashboard';
import mailer from './mailer';
var readTextFile = require('read-text-file');
import { findEmails } from './functions';
import csvParser from 'csv-parser'
import spamcheck from 'spam-detection'
var stringSearcher = require("string-search")
import { Gctx } from './modules/Gctx';
import md5 from 'md5';
import sha256File from 'sha256-file';
import sha1 from 'sha1';
import { Outlook } from './modules/OutlookSearch';
import { DNSBL } from './modules/Dnsbl';
import { DeceptiveCheck } from './modules/deceptiveCheck';

const createCsvWriter = require('csv-writer').createObjectCsvWriter;

export function convertExponentialToDecimal(number) {
    var exponentialNumber;
    exponentialNumber = number
    var str = exponentialNumber.toString();
    if (str.indexOf('e') !== -1) {
        var exponent = parseInt(str.split('-')[1], 10);
        var result = exponentialNumber.toFixed(exponent);
        return result;
    } else {
        return exponentialNumber;
    }
}

function makeid() {
    var result = '',
        characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789',
        charactersLength = 62;
    for (var i = 0; i < 7; i++) { result += characters.charAt(Math.floor(Math.random() * charactersLength)) }
    return result;
}

function createAggregateReport(aggreagateData) {
    // console.log("Inside createAggregateReport", aggreagateData);
    var dashboard = new Dashboard();
    dashboard.saveAggregateReports(aggreagateData);
}

export var queue = kue.createQueue({
    prefix: 'q1',
    redis: {
        host: config.redis.host,
        port: config.redis.port,
        auth: config.redis.password
    }
});

var score, updatedScore, totalScore;

import dns from 'dns';
import spfParser from 'spf-parse';
import { SPFValidator } from './modules/spf';
import cidrTools from 'cidr-tools';
import ipRangeCheck from "ip-range-check";
import dkimVerify from 'python-dkim-verify';
import dmarc from 'dmarc-solution';

function fetchSPFRecordForDomainFromTXT(records) {
    // console.log("records",records)
    var spfString = null;
    try {
        if (records == undefined) {
            return "v=spf1 ";
        } else {
            for (var i = 0; i < records.length; i++) {
                if (records[i][0].startsWith('v=spf1')) {
                    var spfRecord = spfParser(records[i][0]);
                    if (spfRecord.valid) {
                        for (var j = 0; j < spfRecord.mechanisms.length; j++) {
                            if (spfRecord.mechanisms[j].prefix == 'v') {
                                spfString = spfRecord.mechanisms[j].prefix + "=" + spfRecord.mechanisms[j].value;
                                continue;
                            }
                            else if (spfRecord.mechanisms[j].type == 'all') {

                                break;
                            }
                            else {
                                if (spfRecord.mechanisms[j].value) spfString = spfString + " " + spfRecord.mechanisms[j].type + ":" + spfRecord.mechanisms[j].value;
                                else spfString = spfString + " " + spfRecord.mechanisms[j].type;

                            }
                        }
                        if (spfString && spfString.includes("all"))
                            break;
                    }
                }
            }
        }
        return spfString;
    } catch (err) {
        console.log(err)
        return "v=spf1 ";
    }
}

function getSPfRecord(domain) {
    // console.log("getSPfRecord",domain)
    return new Promise((resolve) => {
        dns.resolveTxt(domain, async (err, addresses) => {
            console.log('TXT records:', addresses, err)
            resolve(await fetchSPFRecordForDomainFromTXT(addresses))
        })
    });
}

function flattenSpfRecord(record, domain, version) {
    return new Promise((resolve, reject) => {
        var sp = new SPFValidator({ 'domain': record, 'expandIncludes': true })
        sp.getRecords(record)
            .then(r => {
                // console.log("r",r)
                var ips = [];
                if (r[record].ip4 && r[record].ip4.length) ips = cidrTools.merge(r[record].ip4)
                if (r[record].ip6 && r[record].ip6.length) ips = ips.concat(cidrTools.merge(r[record].ip6))
                if (r[record].ext && r[record].ext.length) ips = ips.concat(r[record].ext);
                // console.log("ips",ips);
                return ips;
            })
            .then(r => {
                resolve(r)
            })
            .catch(e => {
                console.log("err", e);
                reject(e)
            })
    })
}

async function checkSpf(senderIp, senderDomain) {
    console.log("senderDomain", senderDomain);
    let spfRecord = await getSPfRecord(senderDomain)
    // console.log("spfRecord is",spfRecord)
    let ips = await flattenSpfRecord(spfRecord)
    var check = ipRangeCheck(senderIp, ips) ? true : ipRangeCheck(senderIp, ips)
    // console.log("check is",check)
    return check ? "pass" : "fail";
}

async function checkDkim(header) {
    try {
        const result = await dkimVerify(header);
        console.log("result is", result);
        return result ? "pass" : "fail";
    } catch (err) {
        console.error("Error while verifying dkim", err);
        return "fail";
    }
}

function fetchDmarcClass(senderDomain) {
    let aspf = "r";
    let adkim = "r";
    // Fetcher from DNS
    return dmarc.fetch(senderDomain)
        .then(record => {
            // console.log(record);        
            if (record.tags.aspf)
                aspf = record.tags.aspf.value;
            if (record.tags.adkim)
                adkim = record.tags.adkim.value;

            return { aspf, adkim }
        })
        .catch(err => {
            console.log(err);
            return { aspf, adkim }
        });
}

async function checkDeceptive(data) {
    console.log("Insid checkDeceptive", data);
    let result = await new DeceptiveCheck().scanDomain({ domain: [{ "url": "http://" + data.url }, { "url": "https://" + data.url }] });
    console.log("Result is", result);
    if (!data.id)
        return result && result.matches ? true : false;
    return db.Urls.update({
        deceptive: result && result.matches ? true : false
    }, {
        where: {
            id: data.id
        }
    }).catch(err => {
        console.log("Error while checking deceptive url", err);
    })
}

function checkExistingDnsblRecord(ipAddress) {
    return db.dnsblIpScanneds.findOne({
        where: { ip: ipAddress }
    }).then(r => {
        if (r)
            return true;
        else
            return false;
    }).catch(err => {
        console.log("Error while checking existing dnsbl record", err);
        return false;
    })
}

export default {
    init: () => {
        queue.process('report', (job, done) => {
            console.log("Inside report", job.data);
            let headerFilePath = path.join(appRoot, '..', '..', 'Content', 'Unprocessed', job.data.fileName + '-header.txt');
            let bodyFilePath = path.join(appRoot, '..', '..', 'Content', 'Unprocessed', job.data.fileName + '-body.txt');
            console.log("headerFilePath", headerFilePath)
            var contents = "";
            var rawHeader = "";
            try {
                contents = readTextFile.readSync(bodyFilePath);
                rawHeader = readTextFile.readSync(headerFilePath);
            } catch (err) {
                console.log("Error while rading header files", err);
            }
            var aggreagateData = {};
            aggreagateData["companyCode"] = job.data.companyCode;
            aggreagateData["totalMails"] = 1;
            aggreagateData["totalAttachmentScannedGctx"] = 0;
            aggreagateData["attachmentVirusFoundGctx"] = 0;
            aggreagateData["totalAttachmentScanned"] = 0;
            aggreagateData["attachmentsVirusFound"] = 0;
            aggreagateData["totalDomainScanned"] = 0;
            aggreagateData["domainVirusFound"] = 0;
            aggreagateData["totalUrlScanned"] = 0;
            aggreagateData["urlVirusFound"] = 0;
            aggreagateData["spfPassCount"] = 0;
            aggreagateData["dkimPassCount"] = 0;
            aggreagateData["dmarcPassCount"] = 0;
            aggreagateData["forDate"] = new Date();
            var SenderIP = null;
            var spfDomain = null;
            var mailId = null;

            new Promise((resolve, reject) => {
                fs.readFile(headerFilePath, function (err, data) {
                    if (err) reject(err);
                    else {
                        resolve(data)
                    };
                })
            })
                .then(headerFileContent => {
                    return new Promise((reject, resolve) => {
                        simpleParser(headerFileContent, {}, (parsed, err) => {
                            if (err) return reject(err);
                            console.log(parsed)
                            resolve(parsed)
                        });
                    })
                })
                .then((parsedEmail) => {
                    if (parsedEmail && parsedEmail.from && parsedEmail.from.text) {
                        aggreagateData["parsedEmail"] = parsedEmail;
                        let senderId;
                        if (parsedEmail.from.text.split("&#xD")[0].endsWith("&gt, ")) senderId = parsedEmail.from.text.split(",")[1].split("&gt")[0].split(" ")[1];
                        else senderId = parsedEmail.from.text.split("&#xD")[0];
                        spfDomain = senderId.split('@')[1].replace(/[^.,a-zA-Z0-9]/g, '');
                        return db.SenderId.create({
                            senderId: senderId,
                            isBlocked: "0",
                        }).then(r => ([r, parsedEmail]))
                    } else {
                        return db.SenderId.create({
                            senderId: job.data.from ? job.data.from : job.data.sender,
                            isBlocked: "0",
                        }).then(r => ([r, parsedEmail]))
                    }
                })
                .then(([sender, parsedEmail]) => {
                    // console.log("parsedEmail",parsedEmail)
                    var index = null;
                    for (var i = 0; i < parsedEmail.headerLines.length; i++) {
                        if (parsedEmail.headerLines[i]['key'] == "received-spf") {
                            index = i;
                        }
                    }
                    var r = /\b\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\b/;
                    SenderIP = index ? parsedEmail.headerLines[index]['line'].match(r) : "";
                    console.log("senderDomain", spfDomain);
                    console.log("SenderIP", SenderIP);
                    return [sender, parsedEmail]
                }).then(([sender, parsedEmail]) => {
                    console.log("---------------------------------------Email Parsing Starts ---------------------------------------------")
                    // console.log("parsedEmail", json(parsedEmail));
                    // console.log("sender", sender);
                    var subject = parsedEmail.subject;
                    if (parsedEmail.to) {
                        if (parsedEmail.to) parsedEmail.to = json(parsedEmail.to);
                        if (parsedEmail.to && !parsedEmail.to.value) parsedEmail.to.value = json(parsedEmail.to[0].value);
                        var recieverId = [],
                            ccId = [],
                            bccId = [];
                        for (var i = 0;
                            (parsedEmail.to && parsedEmail.to.value && i < parsedEmail.to.value.length); i++) {
                            recieverId.push(parsedEmail.to.value[i].name + ' <' + parsedEmail.to.value[i].address + '>'); //TODO: check name
                        }
                        for (var j = 0;
                            (parsedEmail.cc && parsedEmail.cc.value && j < parsedEmail.cc.value.length); j++) {
                            ccId.push(parsedEmail.cc.value[j].name + ' <' + parsedEmail.cc.value[j].address + '>'); //TODO: check name
                        }
                        for (var k = 0;
                            (parsedEmail.bcc && parsedEmail.bcc.value && k < parsedEmail.bcc.value.length); k++) {
                            bccId.push(parsedEmail.bcc.value[k].name + ' <' + parsedEmail.bcc.value[k].address + '>'); //TODO: check name
                        }
                    }
                    console.log("---------------------------------------Header Parsing Starts ---------------------------------------------")
                    return db.Header.create({
                        subject: subject ? subject : job.data.subject,
                        reciever: JSON.stringify(recieverId), //TODO: Move to another table
                        ccId: JSON.stringify(ccId),
                        bccId: JSON.stringify(bccId),
                        mailTime: (parsedEmail && parsedEmail.date) ? parsedEmail.date : new Date(),
                        senderId: (sender && sender.id) ? sender.id : null,
                    }).then(r => [r, parsedEmail, sender, recieverId])
                        .catch(err => console.log("Error while inserting headers", err));
                }).then(([header, parseEmail, sender, recieverId]) => {
                    console.log("parseEmail", parseEmail, JSON.parse(JSON.stringify(header)), job.data.sender.split('@')[1].split('>')[0].split("&")[0])
                    var domain = job.data.sender.split('@')[1].split('>')[0].split("&")[0];
                    // domain = domain[1].split('>')
                    // aggreagateData["domain"] = domain[0].split("&")[0];
                    return db.SenderDomains.findOne({
                        attributes: ['id', 'domains', 'companyCode'],
                        where: { domains: domain[0], companyCode: job.data.companyCode }
                    }).then(r => [r, header, parseEmail, sender, domain[0]])
                }).then(([senderDomain, header, parseEmail, sender, domain]) => {
                    if (senderDomain) {
                        return [senderDomain, header, parseEmail, sender]
                    } else {
                        return db.SenderDomains.create({
                            domains: domain,
                            isBlocked: "0",
                            companyCode: job.data.companyCode,
                        }).then(r => [r, header, parseEmail, sender])
                    }
                })
                .then(([senderDomain, header, parsedEmail, sender]) => {
                    return db.ReportedMail.findOne({
                        where: {
                            fileName: job.data.fileName,
                        }
                    }).then(r => [r, senderDomain, header, parsedEmail, sender])
                })
                .then(([reportedMail, senderDomain, header, parsedEmail, sender]) => {
                    if (reportedMail) {
                        return db.ReportedMail.update({
                            headerId: header.id,
                            body: contents.replace(/[^\x00-\x7F]/g, ""), //replace all non-ascii value with blank space
                        }, {
                            where: {
                                id: reportedMail.id
                            }
                        }).then(r => [reportedMail.id, senderDomain, header, parsedEmail, sender])
                    } else throw new Error('Reported Email not found')
                })
                .then(([reportedMailId, senderDomain, header, parsedEmail, sender]) => {
                    mailId = reportedMailId;
                    aggreagateData["reportedMailId"] = reportedMailId;
                    console.log("---------------------------------------Header Parsing Ends ---------------------------------------------")
                    console.log("---------------------------------------Links Parsing Starts ---------------------------------------------")
                    var emailUrls = {};
                    var urls = []
                    var scanner = new Scanner();
                    emailUrls = getUrls(contents);
                    emailUrls = Array.from(emailUrls)
                    var urlSet = new Set();
                    for (var i = 0; emailUrls && i < emailUrls.length; i++) {
                        var parsed = urlParse(emailUrls[i], true).hostname;
                        if (parsed && !urlSet.has(parsed)) {
                            urls.push(parsed);
                            urlSet.add(parsed)
                        }
                    }
                    return urls.reduce((p, value) => {
                        scanner.scanAttachment({ name: value, type: "url", options: {} })
                        return db.Urls.create({
                            status: "queued",
                            url: value,
                            reportedMailId: reportedMailId,
                        }).then(r => { checkDeceptive(JSON.parse(JSON.stringify(r))) });
                    }, Promise.resolve()).then(r => [reportedMailId, senderDomain, header, parsedEmail, sender])
                }).then(([reportedMailId, senderDomain, header, parsedEmail, sender]) => {
                    return db.Urls.findAll({
                        where: { reportedMailId }
                    }, {
                        attributes: ['url', 'reportedMailId']
                    }).then(r => [r, reportedMailId, senderDomain, header, parsedEmail, sender])
                }).then(([mailUrls, reportedMailId, senderDomain, header, parsedEmail, sender]) => {
                    // console.log("urls found", JSON.parse(JSON.stringify(urls)));
                    aggreagateData["totalUrlScanned"] = mailUrls.length;
                    aggreagateData["totalUrls"] = mailUrls;
                    aggreagateData["totalUrlScannedGctx"] = 0;
                    aggreagateData["urlVirusFoundGctx"] = 0;
                    var gctx = new Gctx();
                    for (var i = 0; i < mailUrls.length; i++) {
                        var total = 10;
                        aggreagateData["totalUrlScannedGctx"] += 1;
                        gctx.urlCheck({ url: mailUrls[i].url })
                            .then(gctxurlcheck => {
                                if (gctxurlcheck.url == true) {
                                    score = 10;
                                    aggreagateData["urlVirusFoundGctx"] += 1;
                                } else score = 0;
                                return db.ReportedMail.findOne({ attributes: ['score', 'total'] }, { where: { fileName: job.data.fileName } })
                            })
                            .then(whatisScore => {
                                if (whatisScore) {
                                    totalScore = total + whatisScore.total;
                                    updatedScore = whatisScore.score + score;
                                    return db.ReportedMail.update({
                                        score: updatedScore,
                                        total: totalScore,
                                    }, { where: { id: reportedMailId } }) //Here updating WRT file id and above finding the same with filename
                                }
                            })
                    }
                    return [reportedMailId, senderDomain, header, parsedEmail, sender]
                })
                .then(([reportedMailId, senderDomain, header, parsedEmail, sender]) => {
                    return db.ReportedMail.update({
                        linkStatus: "queued",
                        attachmentStatus: "queued"
                    }, {
                        where: {
                            id: reportedMailId,
                        }
                    }).then(r => [senderDomain, header, parsedEmail, sender])
                })
                .then(([senderDomain, header, parsedEmail, sender]) => {
                    return db.Header.update({
                        blockDomainId: senderDomain.id,
                        rawHeader: JSON.stringify(rawHeader),
                        mailHeader: JSON.stringify(parsedEmail.headerLines),
                    }, {
                        where: {
                            id: header.id,
                        }
                    }).then(r => [senderDomain, header, parsedEmail, sender])
                }).then(([senderDomain, header, parsedEmail, sender]) => {
                    var index = null;
                    for (var i = 0; i < parsedEmail.headerLines.length; i++) {
                        if (parsedEmail.headerLines[i]['key'] == "authentication-results") {
                            index = i;
                        }
                    }
                    if (parsedEmail.headerLines[index] == undefined) {
                        var spf = "NA",
                            dkim = "NA",
                            dmarc = "NA";
                        return db.Header.update({ dmarc: dmarc, spf: spf, dkim: dkim }, {
                            where: { id: header.id }
                        }).then(r => [r, header, sender, parsedEmail])
                    } else {
                        var string = parsedEmail.headerLines[index]['line'];
                        var spf = "",
                            dkim = "",
                            dmarc = "";
                        return stringSearcher.find(string, 'spf=pass')
                            .then(async r => {
                                // spf = (r.length) ? "pass" : "fail"
                                spf = (r.length) ? "pass" : await checkSpf(SenderIP, spfDomain)
                                return stringSearcher.find(string, 'dkim=pass')
                                    .then(async q => {
                                        // dkim = (q.length) ? "pass" : "fail"
                                        dkim = (q.length) ? "pass" : await checkDkim(rawHeader);
                                        return stringSearcher.find(string, 'dmarc=pass')
                                    }).then(async z => {
                                        // dmarc = (z.length) ? "pass" : "fail"
                                        dmarc = (z.length) ? "pass" : (spf == "pass" ? "pass" : (dkim == "pass" ? "pass" : "fail"))
                                        if (spf == "pass")
                                            aggreagateData["spfPassCount"] = 1;
                                        if (dkim == "pass")
                                            aggreagateData["dkimPassCount"] = 1;
                                        if (dmarc)
                                            aggreagateData["dmarcPassCount"] = 1;

                                        console.log("{ dmarc: dmarc, spf: spf, dkim: dkim }", { dmarc: dmarc, spf: spf, dkim: dkim })
                                        return db.Header.update({ dmarc: dmarc, spf: spf, dkim: dkim }, {
                                            where: { id: header.id }
                                        }).then(r => [r, header, sender, parsedEmail])
                                    })
                            })
                    }

                })
                .then(([r, header, sender, parsedEmail]) => {
                    console.log("---------------------------------------Email Goes For Scanning In Exchange ---------------------------------------------")
                    var realsender = sender.senderId;
                    var mailTime = header.mailTime;
                    var email = findEmails(realsender);
                    var date = new Date(mailTime);
                    var startDate = new Date(mailTime);
                    startDate.setDate(startDate.getDate() - 1)
                    var endDate = date
                    endDate.setDate(endDate.getDate() + 1)
                    startDate = String(startDate.getMonth() + 1).padStart(2, "0") + '-' + String(startDate.getDate()).padStart(2, "0") + '-' + String(startDate.getFullYear()).padStart(2, "0");
                    endDate = String(endDate.getMonth() + 1).padStart(2, "0") + '-' + String(endDate.getDate()).padStart(2, "0") + '-' + String(endDate.getFullYear()).padStart(2, "0");
                    queue.create('mail-scan-exchange-v2', { sender: JSON.stringify(email[0]), startDate: startDate, endDate: endDate, companyCode: job.data.companyCode, fileName: job.data.fileName, reportedMailId: aggreagateData["reportedMailId"], subject: parsedEmail.subject, credentials: job.data.credentials, aggreagateData: aggreagateData }).save();
                    console.log("---------------------------------------Email SPAM OR HAM Check Starts ---------------------------------------------")
                    return parsedEmail
                }).then(parsedEmail => {
                    if (parsedEmail) {
                        // console.log("parsedEmail.from.value", parsedEmail.from.value, parsedEmail.from.value.filter(address => address.address.length).length ? parsedEmail.from.value.filter(address => address.address.length)[0].address.replace("&gt", "") : "NA");
                        // return [mailer.sendReportMail(job.data.reciever, parsedEmail.subject, parsedEmail.date.toString(), parsedEmail.from.value[0].address), parsedEmail];
                        return [mailer.sendReportMail(job.data.reciever, parsedEmail.subject ? parsedEmail.subject : job.data.subject, parsedEmail.date ? parsedEmail.date.toString() : new Date().toString(), parsedEmail.from ? (parsedEmail.from.value.filter(address => address.address.length).length ? parsedEmail.from.value.filter(address => address.address.length)[0].address.replace("&gt", "") : job.data.sender) : job.data.sender), parsedEmail];
                    } else throw new Error("Mail couldnt be sent to ")
                })
                .then(([report, parsedEmail]) => {
                    if (report) {
                        var dashboard = new Dashboard();
                        return dashboard.fetchSocIds({ companyCode: job.data.companyCode }).then(r => { //fetchSocIds (what is Soc?)
                            return [r, parsedEmail]
                        })
                    } else throw new Error("Soc team Mail coudnt be fetched")
                })
                .then(([metaAuth, parsedEmail]) => {
                    // console.log("Sending email to soc team outside", job.data.reciever)
                    if (metaAuth && parsedEmail.subject && parsedEmail.date && parsedEmail.from) {
                        metaAuth.reduce((p, a) => {
                            // console.log("Sending email to soc team", a.value)
                            mailer.alertSOCTeam(a.value, job.data.reciever, parsedEmail.subject ? parsedEmail.subject : job.data.subject, parsedEmail.date ? parsedEmail.date.toString() : new Date().toString(), parsedEmail.from ? parsedEmail.from.value[0].address : job.data.sender, mailId);
                        }, Promise.resolve())
                    } else throw new Error("Mail coudnt be sent to SOC team")
                })
                .then(r => {
                    return db.ReportedMail.findOne({ where: { fileName: job.data.fileName }, include: [{ model: db.Attachment, required: true }] })
                })
                .then(r => {
                    var bodyHamorSpam = spamcheck.detect(contents)
                    // var bodyHamSpamPercentage = spamcheck.getResults(contents)
                    var hamorspam = (bodyHamorSpam && bodyHamorSpam == 'spam') ? "spam" : "ham"
                    return db.Body.create({
                        spamOrHam: hamorspam, //TODO: Move raw body to Body table 
                    })
                }).then(result => {
                    console.log("---------------------------------------Email SPAM OR HAM Check Ends ---------------------------------------------")
                    return db.ReportedMail.update({
                        bodyId: result.id
                    }, {
                        where: {
                            fileName: job.data.fileName
                        }
                    })
                })
                .then(async r => {
                    let domainDeceptive = await checkDeceptive({ url: spfDomain });
                    console.log("Domain deceptive is", domainDeceptive);
                    return db.deceptiveDomains.findOne({
                        where: { domain: spfDomain }
                    }).then(existing => {
                        if (existing)
                            db.deceptiveDomains.update({
                                deceptive: domainDeceptive
                            }, {
                                where: { id: existing.id }
                            }).catch(err => { console.log("Error while updating deceptive domain", err) })
                        else
                            db.deceptiveDomains.create({
                                domain: spfDomain,
                                deceptive: domainDeceptive
                            }).catch(err => { console.log("Error while creating deceptive domain", err) })
                        return r;
                    })
                })
                .then(r => {
                    var md5Hash;
                    var gctx = new Gctx();
                    if (r) {
                        try {
                            aggreagateData["totalAttachmentScanned"] = r.Attachments.length;
                            for (var i = 0; i < r.Attachments.length; i++) {
                                aggreagateData["totalAttachmentScannedGctx"] += 1;
                                const attachmentPath = path.join(appRoot, '..', '..', 'Content', 'Attachments', r.Attachments[i].name);
                                new Promise((resolve, reject) => {
                                    fs.readFile(attachmentPath, function (err, data) {
                                        if (err) reject(err);
                                        else resolve(data);
                                    })
                                })
                                    .then(file => {
                                        md5Hash = md5(file);
                                        return gctx.md5HashCheck({ hash: md5Hash }).then(r => [r, file])
                                            .then(([r, file]) => {
                                                if (r.md5HashCheck == true) {
                                                    score = 10;
                                                    aggreagateData["attachmentVirusFoundGctx"] += 1;
                                                } else score = 0;
                                                return db.ReportedMail.findOne({ attributes: ['score'] }, { where: { fileName: job.data.fileName } }).then(p => [p, score, file])
                                                    .then(([reportedMail, score, file]) => {
                                                        updatedScore = reportedMail.score + score;
                                                        return db.ReportedMail.update({
                                                            score: updatedScore,
                                                        }, {
                                                            where: {
                                                                fileName: job.data.fileName
                                                            }
                                                        }).then(r => [r, file])
                                                    }).then(r => [file])
                                            })
                                            .then(file => {
                                                var sha256Hash = sha256File(attachmentPath)
                                                return gctx.sha256HashCheck({ hash: sha256Hash }).then(r => [r, file])
                                                    .then(([r, file]) => {
                                                        if (r.sha256HashCheck == true) {
                                                            score = 10;
                                                            aggreagateData["attachmentVirusFoundGctx"] += 1;
                                                        } else score = 0;
                                                        return db.ReportedMail.findOne({ attributes: ['score'] }, { where: { fileName: job.data.fileName } }).then(p => [p, score, file])
                                                            .then(([reportedMail, score, file]) => {
                                                                updatedScore = reportedMail.score + score;
                                                                return db.ReportedMail.update({
                                                                    score: updatedScore,
                                                                }, {
                                                                    where: {
                                                                        fileName: job.data.fileName
                                                                    }
                                                                }).then(r => [r, file])
                                                            }).then(r => [file])
                                                    })
                                            })
                                            .then(file => {
                                                var sha1Check = sha1(attachmentPath);
                                                return gctx.sha1HashCheck({ hash: sha1Check }).then(r => [r, file])
                                                    .then(([r, file]) => {
                                                        if (r.sha1hash == true) {
                                                            score = 10;
                                                            aggreagateData["attachmentVirusFoundGctx"] += 1;
                                                        } else score = 0;
                                                        return db.ReportedMail.findOne({ attributes: ['score'] }, { where: { fileName: job.data.fileName } }).then(p => [p, score, file])
                                                            .then(([reportedMail, score, file]) => {
                                                                updatedScore = reportedMail.score + score;
                                                                return db.ReportedMail.update({
                                                                    score: updatedScore,
                                                                }, {
                                                                    where: {
                                                                        fileName: job.data.fileName
                                                                    }
                                                                }).then(r => [r, file])
                                                            }).then(r => [file])
                                                    })
                                            })
                                    })
                            }
                            console.log("---------------------------------------Email Parsing Ends ---------------------------------------------")
                            done();
                        } catch {
                            done()
                        }
                    } else {
                        done();
                    }
                    createAggregateReport(aggreagateData);
                }).then(r => {
                    queue.create('dnsbl-check', { SenderIP, mailId }).save();
                    return r;
                })
                .catch(err => {
                    console.log(err)
                    done(new Error('Error Parsing report!!'));
                })
        })

        // queue.process('report', (job, done) => {
        //     console.log("Inside report", job.data);
        //     let headerFilePath = path.join(appRoot, '..', '..', 'Content', 'Unprocessed', job.data.fileName + '-header.txt');
        //     let bodyFilePath = path.join(appRoot, '..', '..', 'Content', 'Unprocessed', job.data.fileName + '-body.txt');
        //     var contents = "";
        //     var rawHeader = "";
        //     try {
        //         contents = readTextFile.readSync(bodyFilePath);
        //         rawHeader = readTextFile.readSync(headerFilePath);
        //     } catch (err) {
        //         console.log("Error while rading header files", err);
        //     }
        //     var SenderIP = null;
        //     var spfDomain = null;
        //     var mailId = null;

        //     //Doubt - why using the below code for reading File again as above in contents the file data is already stored
        //     new Promise((resolve, reject) => {
        //         fs.readFile(headerFilePath, function (err, data) {
        //             if (err) reject(err);
        //             else {
        //                 resolve(data)
        //             };
        //         })
        //     })
        //         .then(headerFileContent => {
        //             return new Promise((reject, resolve) => {
        //                 simpleParser(headerFileContent, {}, (parsed, err) => {
        //                     if (err) return reject(err);
        //                     console.log(parsed)
        //                     resolve(parsed)
        //                 });
        //             })
        //         })
        //         .then((parsedEmail) => {
        //             if (parsedEmail && parsedEmail.from && parsedEmail.from.text) {
        //                 let senderId;
        //                 if (parsedEmail.from.text.split("&#xD")[0].endsWith("&gt, ")) senderId = parsedEmail.from.text.split(",")[1].split("&gt")[0].split(" ")[1];
        //                 else senderId = parsedEmail.from.text.split("&#xD")[0];
        //                 spfDomain = senderId.split('@')[1].replace(/[^.,a-zA-Z0-9]/g, '');
        //                 return db.SenderId.create({
        //                     senderId: senderId,
        //                     isBlocked: "0",
        //                 }).then(r => ([r, parsedEmail]))
        //             }
        //             else {
        //                 return db.SenderId.create({
        //                     senderId: job.data.from ? job.data.from : job.data.sender,
        //                     isBlocked: "0",
        //                 }).then(r => ([r, parsedEmail]))
        //             }
        //         })
        //         .then(([sender, parsedEmail]) => {
        //             // console.log("parsedEmail",parsedEmail)
        //             var index = null;
        //             for (var i = 0; i < parsedEmail.headerLines.length; i++) {
        //                 if (parsedEmail.headerLines[i]['key'] == "received-spf") {
        //                     index = i;
        //                 }
        //             }
        //             var r = /\b\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\b/;
        //             SenderIP = index ? parsedEmail.headerLines[index]['line'].match(r) : "";
        //             console.log("senderDomain", spfDomain);
        //             console.log("SenderIP", SenderIP);
        //             return [sender, parsedEmail]
        //         })
        //         .then(([sender, parsedEmail]) => {
        //             console.log("---------------------------------------Email Parsing Starts ---------------------------------------------")
        //             // console.log("parsedEmail", json(parsedEmail));
        //             // console.log("sender", sender);
        //             var subject = parsedEmail.subject ? parsedEmail.subject : job.data.subject;
        //             if (parsedEmail.to) {
        //                 if (parsedEmail.to) parsedEmail.to = json(parsedEmail.to);
        //                 if (!parsedEmail.to.value) parsedEmail.to.value = json(parsedEmail.to[0].value);
        //                 var recieverId = [], ccId = [], bccId = [];
        //                 for (var i = 0; (parsedEmail.to && parsedEmail.to.value && i < parsedEmail.to.value.length); i++) {
        //                     recieverId.push(parsedEmail.to.value[i].name + ' <' + parsedEmail.to.value[i].address + '>'); //TODO: check name
        //                 }
        //                 for (var j = 0; (parsedEmail.cc && parsedEmail.cc.value && j < parsedEmail.cc.value.length); j++) {
        //                     ccId.push(parsedEmail.cc.value[j].name + ' <' + parsedEmail.cc.value[j].address + '>'); //TODO: check name
        //                 }
        //                 for (var k = 0; (parsedEmail.bcc && parsedEmail.bcc.value && k < parsedEmail.bcc.value.length); k++) {
        //                     bccId.push(parsedEmail.bcc.value[k].name + ' <' + parsedEmail.bcc.value[k].address + '>'); //TODO: check name
        //                 }
        //             }
        //             console.log("---------------------------------------Header Parsing Starts ---------------------------------------------")
        //             return db.Header.create({
        //                 subject: subject ? subject : null,
        //                 reciever: JSON.stringify(recieverId), //TODO: Move to another table
        //                 ccId: JSON.stringify(ccId),
        //                 bccId: JSON.stringify(bccId),
        //                 mailTime: (parsedEmail && parsedEmail.date) ? parsedEmail.date : new Date(),
        //                 senderId: (sender && sender.id) ? sender.id : null,
        //             }).then(r => [r, parsedEmail, sender,])
        //         }).then(([header, parseEmail, sender]) => {
        //             var domain = job.data.sender.split('@')[1].split('>')[0].split("&")[0];
        //             domain = domain[1].split('>')
        //             return db.SenderDomains.findOne({
        //                 attributes: ['id', 'domains', 'companyCode'],
        //                 where: { domains: domain[0], companyCode: job.data.companyCode }
        //             }).then(r => [r, header, parseEmail, sender, domain[0]])
        //         }).then(([senderDomain, header, parseEmail, sender, domain]) => {
        //             if (senderDomain) {
        //                 return [senderDomain, header, parseEmail, sender]
        //             } else {
        //                 return db.SenderDomains.create({
        //                     domains: domain,
        //                     isBlocked: "0",
        //                     companyCode: job.data.companyCode,
        //                 }).then(r => [r, header, parseEmail, sender])
        //             }
        //         })
        //         .then(([senderDomain, header, parsedEmail, sender]) => {
        //             return db.ReportedMail.findOne({
        //                 where: {
        //                     fileName: job.data.fileName,
        //                 }
        //             }).then(r => [r, senderDomain, header, parsedEmail, sender])
        //         })
        //         .then(([reportedMail, senderDomain, header, parsedEmail, sender]) => {
        //             if (reportedMail) {
        //                 return db.ReportedMail.update({
        //                     headerId: header.id,
        //                     body: contents,
        //                 }, {
        //                     where: {
        //                         id: reportedMail.id
        //                     }
        //                 }).then(r => [reportedMail.id, senderDomain, header, parsedEmail, sender])
        //             } else throw new Error('Reported Email not found')
        //         })
        //         .then(([reportedMailId, senderDomain, header, parsedEmail, sender]) => {
        //             mailId = reportedMailId;
        //             console.log("---------------------------------------Header Parsing Ends ---------------------------------------------")
        //             console.log("---------------------------------------Links Parsing Starts ---------------------------------------------")
        //             var emailUrls = {};
        //             var urls = []
        //             var scanner = new Scanner();
        //             emailUrls = getUrls(contents);
        //             emailUrls = Array.from(emailUrls) //Why this?
        //             var urlSet = new Set();
        //             for (var i = 0; emailUrls && i < emailUrls.length; i++) {
        //                 var parsed = urlParse(emailUrls[i], true).hostname;
        //                 if (parsed && !urlSet.has(parsed)) { urls.push(parsed); urlSet.add(parsed) }
        //             }
        //             return urls.reduce((p, value) => {
        //                 scanner.scanAttachment({ name: value, type: "url", options: {} })
        //                 return db.Urls.create({
        //                     status: "queued", //Meaning of this?
        //                     url: value,
        //                     reportedMailId: reportedMailId,
        //                 })
        //             }, Promise.resolve()).then(r => [reportedMailId, senderDomain, header, parsedEmail, sender])
        //         }).then(([reportedMailId, senderDomain, header, parsedEmail, sender]) => {
        //             // return db.Urls.findAll({ attributes: ['url'] }, {
        //             // 	where: {
        //             // 		reportedMailId: reportedMailId
        //             // 	}
        //             // })
        //             return db.Urls.findAll({
        //                 where: { reportedMailId }
        //             }, {
        //                 attributes: ['url', 'reportedMailId']
        //             })
        //                 .then(r => [r, reportedMailId, senderDomain, header, parsedEmail, sender])
        //         }).then(([mailUrls, reportedMailId, senderDomain, header, parsedEmail, sender]) => {
        //             var gctx = new Gctx();
        //             console.log("mailUrls.length", mailUrls.length, reportedMailId, JSON.parse(JSON.stringify(mailUrls)))
        //             for (var i = 0; i < mailUrls.length; i++) {
        //                 var total = 10;
        //                 gctx.urlCheck({ url: mailUrls[i].url })
        //                     .then(gctxurlcheck => {
        //                         if (gctxurlcheck.url == true) score = 10;
        //                         else score = 0;
        //                         return db.ReportedMail.findOne({ attributes: ['score', 'total'] }, { where: { fileName: job.data.fileName } })
        //                     })
        //                     .then(whatisScore => {
        //                         if (whatisScore) {
        //                             totalScore = total + whatisScore.total;
        //                             updatedScore = whatisScore.score + score;
        //                             return db.ReportedMail.update({
        //                                 score: updatedScore,
        //                                 total: totalScore,
        //                             }, { where: { id: reportedMailId } }) //Here updating WRT file id and above finding the same with filename
        //                         }
        //                     })
        //             }
        //             return [reportedMailId, senderDomain, header, parsedEmail, sender]
        //         })
        //         .then(([reportedMailId, senderDomain, header, parsedEmail, sender]) => {
        //             return db.ReportedMail.update({
        //                 linkStatus: "queued",
        //                 attachmentStatus: "queued"
        //             }, {
        //                 where: {
        //                     id: reportedMailId,
        //                 }
        //             }).then(r => [senderDomain, header, parsedEmail, sender])
        //         })
        //         .then(([senderDomain, header, parsedEmail, sender]) => {
        //             return db.Header.update({
        //                 blockDomainId: senderDomain.id,
        //                 rawHeader: JSON.stringify(rawHeader),
        //                 mailHeader: JSON.stringify(parsedEmail.headerLines),
        //             }, {
        //                 where: {
        //                     id: header.id,
        //                 }
        //             }).then(r => [senderDomain, header, parsedEmail, sender])
        //         }).then(([senderDomain, header, parsedEmail, sender]) => {
        //             var index = null;
        //             for (var i = 0; i < parsedEmail.headerLines.length; i++) {
        //                 if (parsedEmail.headerLines[i]['key'] == "authentication-results") {
        //                     index = i;
        //                 }
        //             }
        //             if (parsedEmail.headerLines[index] == undefined) {
        //                 var spf = "NA", dkim = "NA", dmarc = "NA";
        //                 return db.Header.update({ dmarc: dmarc, spf: spf, dkim: dkim }, {
        //                     where: { id: header.id }
        //                 }).then(r => [r, header, sender, parsedEmail])
        //             }
        //             else {
        //                 var string = parsedEmail.headerLines[index]['line'];
        //                 var spf = "", dkim = "", dmarc = "";
        //                 return stringSearcher.find(string, 'spf=pass')
        //                     .then(async r => {
        //                         spf = (r.length) ? "pass" : await checkSpf(SenderIP, spfDomain)
        //                         return stringSearcher.find(string, 'dkim=pass')
        //                             .then(async q => {
        //                                 dkim = (q.length) ? "pass" : await checkDkim(rawHeader);
        //                                 return stringSearcher.find(string, 'dmarc=pass')
        //                             }).then(async z => {
        //                                 dmarc = (z.length) ? "pass" : (spf == "pass" ? "pass" : (dkim == "pass" ? "pass" : "fail"))
        //                                 console.log("{ dmarc: dmarc, spf: spf, dkim: dkim }", { dmarc: dmarc, spf: spf, dkim: dkim })
        //                                 return db.Header.update({ dmarc: dmarc, spf: spf, dkim: dkim }, {
        //                                     where: { id: header.id }
        //                                 }).then(r => [r, header, sender, parsedEmail])
        //                             })
        //                     })
        //             }

        //         })
        //         .then(([r, header, sender, parsedEmail]) => {
        //             console.log("---------------------------------------Email Goes For Scanning In Exchange ---------------------------------------------", JSON.parse(JSON.stringify(sender)))
        //             var realsender = sender.senderId;
        //             var mailTime = header.mailTime;
        //             var email = findEmails(realsender);
        //             var date = new Date(mailTime);
        //             var startDate = new Date(mailTime);
        //             startDate.setDate(startDate.getDate() - 1)
        //             var endDate = date
        //             endDate.setDate(endDate.getDate() + 1)
        //             startDate = String(startDate.getMonth() + 1).padStart(2, "0") + '-' + String(startDate.getDate()).padStart(2, "0") + '-' + String(startDate.getFullYear()).padStart(2, "0");
        //             endDate = String(endDate.getMonth() + 1).padStart(2, "0") + '-' + String(endDate.getDate()).padStart(2, "0") + '-' + String(endDate.getFullYear()).padStart(2, "0");
        //             queue.create('mail-scan-exchange-v2', { sender: JSON.stringify(email[0]), startDate: startDate, endDate: endDate, companyCode: job.data.companyCode, fileName: job.data.fileName }).save();
        //             console.log("---------------------------------------Email SPAM OR HAM Check Starts ---------------------------------------------")
        //             return parsedEmail
        //         }).then(parsedEmail => {
        //             if (parsedEmail) {
        //                 // return [mailer.sendReportMail(job.data.reciever, parsedEmail.subject, parsedEmail.date.toString(), parsedEmail.from.value[0].address), parsedEmail];
        //                 return [mailer.sendReportMail(job.data.reciever, parsedEmail.subject ? parsedEmail.subject : job.data.subject, parsedEmail.date ? parsedEmail.date.toString() : new Date().toString(), parsedEmail.from ? (parsedEmail.from.value.filter(address => address.address.length).length ? parsedEmail.from.value.filter(address => address.address.length)[0].address.replace("&gt", "") : job.data.sender) : job.data.sender), parsedEmail];
        //             }
        //             else throw new Error("Mail couldnt be sent to ")
        //         })
        //         .then(([report, parsedEmail]) => {
        //             if (report) {
        //                 var dashboard = new Dashboard();
        //                 return dashboard.fetchSocIds({ companyCode: job.data.companyCode }).then(r => { //fetchSocIds (what is Soc?)
        //                     return [r, parsedEmail]
        //                 })
        //             }
        //             else throw new Error("Soc team Mail coudnt be fetched")
        //         })
        //         .then(([metaAuth, parsedEmail]) => {
        //             if (metaAuth && (parsedEmail.subject || job.data.subject || parsedEmail.date) && (parsedEmail.from || job.data.sender)) {
        //                 metaAuth.reduce((p, a) => {
        //                     mailer.alertSOCTeam(a.value, job.data.reciever, parsedEmail.subject ? parsedEmail.subject : job.data.subject, parsedEmail.date ? parsedEmail.date.toString() : new Date().toString(), parsedEmail.from ? parsedEmail.from.value[0].address : job.data.sender,mailId);
        //                 }, Promise.resolve())
        //             }
        //             else throw new Error("Mail coudnt be sent to SOC team")
        //         })
        //         .then(r => {
        //             return db.ReportedMail.findOne({ where: { fileName: job.data.fileName }, include: [{ model: db.Attachment, required: true }] })
        //         })
        //         .then(r => {
        //             var bodyHamorSpam = spamcheck.detect(contents)
        //             // var bodyHamSpamPercentage = spamcheck.getResults(contents)
        //             var hamorspam = (bodyHamorSpam && bodyHamorSpam == 'spam') ? "spam" : "ham"
        //             return db.Body.create({
        //                 spamOrHam: hamorspam, //TODO: Move raw body to Body table 
        //             })
        //         }).then(result => {
        //             console.log("---------------------------------------Email SPAM OR HAM Check Ends ---------------------------------------------")
        //             return db.ReportedMail.update({
        //                 bodyId: result.id
        //             }, {
        //                 where: {
        //                     fileName: job.data.fileName
        //                 }
        //             })
        //         })
        //         .then(r => {
        //             var md5Hash;
        //             var gctx = new Gctx();
        //             if (r) {
        //                 try {
        //                     for (var i = 0; i < r.Attachments.length; i++) {
        //                         const attachmentPath = path.join(appRoot, '..', '..', 'Content', 'Attachments', r.Attachments[i].name);
        //                         new Promise((resolve, reject) => {
        //                             fs.readFile(attachmentPath, function (err, data) {
        //                                 if (err) reject(err);
        //                                 else resolve(data);
        //                             })
        //                         })
        //                             .then(file => {
        //                                 md5Hash = md5(file);
        //                                 return gctx.md5HashCheck({ hash: md5Hash }).then(r => [r, file])
        //                                     .then(([r, file]) => {
        //                                         if (r.md5HashCheck == true) score = 10;
        //                                         else score = 0;
        //                                         return db.ReportedMail.findOne({ attributes: ['score'] }, { where: { fileName: job.data.fileName } }).then(p => [p, score, file])
        //                                             .then(([reportedMail, score, file]) => {
        //                                                 updatedScore = reportedMail.score + score;
        //                                                 return db.ReportedMail.update({
        //                                                     score: updatedScore,
        //                                                 }, {
        //                                                     where: {
        //                                                         fileName: job.data.fileName
        //                                                     }
        //                                                 }).then(r => [r, file])
        //                                             }).then(r => [file])
        //                                     })
        //                                     .then(file => {
        //                                         var sha256Hash = sha256File(attachmentPath)
        //                                         return gctx.sha256HashCheck({ hash: sha256Hash }).then(r => [r, file])
        //                                             .then(([r, file]) => {
        //                                                 if (r.sha256HashCheck == true) score = 10;
        //                                                 else score = 0;
        //                                                 return db.ReportedMail.findOne({ attributes: ['score'] }, { where: { fileName: job.data.fileName } }).then(p => [p, score, file])
        //                                                     .then(([reportedMail, score, file]) => {
        //                                                         updatedScore = reportedMail.score + score;
        //                                                         return db.ReportedMail.update({
        //                                                             score: updatedScore,
        //                                                         }, {
        //                                                             where: {
        //                                                                 fileName: job.data.fileName
        //                                                             }
        //                                                         }).then(r => [r, file])
        //                                                     }).then(r => [file])
        //                                             })
        //                                     })
        //                                     .then(file => {
        //                                         var sha1Check = sha1(attachmentPath);
        //                                         return gctx.sha1HashCheck({ hash: sha1Check }).then(r => [r, file])
        //                                             .then(([r, file]) => {
        //                                                 if (r.sha1hash == true) score = 10;
        //                                                 else score = 0;
        //                                                 return db.ReportedMail.findOne({ attributes: ['score'] }, { where: { fileName: job.data.fileName } }).then(p => [p, score, file])
        //                                                     .then(([reportedMail, score, file]) => {
        //                                                         updatedScore = reportedMail.score + score;
        //                                                         return db.ReportedMail.update({
        //                                                             score: updatedScore,
        //                                                         }, {
        //                                                             where: {
        //                                                                 fileName: job.data.fileName
        //                                                             }
        //                                                         }).then(r => [r, file])
        //                                                     }).then(r => [file])
        //                                             })
        //                                     })
        //                             })
        //                     }
        //                     console.log("---------------------------------------Email Parsing Ends ---------------------------------------------")
        //                     done();
        //                 }
        //                 catch {
        //                     done()
        //                 }
        //             }
        //             else done();
        //         })
        //         .catch(err => {
        //             console.log(err)
        //             done(new Error('Error Parsing report!!'));
        //         })
        // })

        queue.process('mail-scan-exchange', (job, done) => {
            console.log("---------------------------------------Email Goes For Scanning In Exchange in 2nd QUEUE for Result ---------------------------------------------")
            var dashboard = new Dashboard();
            dashboard.exchangeMetaAuth({ companyCode: job.data.companyCode })
                .then(r => {
                    var exchange = new Exchange();
                    exchange.scanMails({ sender: job.data.sender, startDate: job.data.startDate, endDate: job.data.endDate, filename: job.data.fileName + '.csv', options: (r && r.data) ? r.data : null })
                    done();
                })
                .catch(err => {
                    console.log('kue', 'mail-scan-exchange', err)
                    done(new Error('Error Sending Scan command to Exchange'))
                })
        })

        queue.process('mail-scan-exchange-v2', (job, done) => {
            console.log("---------------------------------------Message Trace API Scanning Starts---------------------------------------------", job.data)
            var outlook = new Outlook();
            if (job.data.credentials && job.data.credentials.userName) {
                outlook.searchMail({ username: job.data.credentials.userName, password: job.data.credentials.password, sender: job.data.sender.replace(/"/g, ""), subject: job.data.subject })
                    .then(r => {
                        r = JSON.parse(JSON.stringify(r))
                        // console.log("Search results", r);
                        r.mails = r.mails.map(mail => {
                            mail.name = job.data.fileName;
                            mail.reportedMailId = job.data.reportedMailId
                            return mail;
                        })
                        const csvWriter = createCsvWriter({
                            path: __dirname + "/../../ExchangeAdapter/results/" + job.data.fileName + '.csv',
                            header: [
                                { id: 'organization', title: 'organization' },
                                { id: 'senderAddress', title: 'SenderAddress' },
                                { id: 'recipientAddress', title: 'RecipientAddress' },
                                { id: 'subject', title: 'Subject' },
                                { id: 'status', title: 'Status' },
                                { id: 'fromIp', title: 'FromIP' },
                                { id: 'size', title: 'Size' },
                                { id: 'recieved', title: 'Recieved' }
                            ]
                        });

                        csvWriter.writeRecords(r.mails)
                            .then(() => {
                                console.log('...Done');
                                db.ReportedMailResult.bulkCreate(r.mails)
                                    .then(r => {
                                        db.ReportedMail.update({ status: 'processed' }, { where: { id: job.data.reportedMailId } })
                                        if (r.length - 1 > 0) {
                                            job.data.aggreagateData.whoElseCount = r.length - 1;
                                            new Dashboard().updateWhoElseAggregateReports(job.data.aggreagateData);
                                        }
                                        return 1
                                    })
                            });
                        done();
                    })
                    .catch(err => {
                        console.log('kue', 'mail-scan-exchange-v2', err)
                        done(new Error('Error Sending Scan command to Exchange'))
                    })
            }
        })

        queue.process('exchange-read-who-else', (job, done) => {
            var fileName = job.data.fileName;
            var resultSet = new Set();
            var results = [];
            try {
                var filePath = path.join(appRoot, '..', '..', 'ExchangeAdapter', 'results', fileName + '.csv')
                db.ReportedMail.findOne({ where: { id: job.data.reportedMailId }, include: [{ model: db.Header, required: true }] })
                    .then(r => {
                        if (r) {
                            try {
                                new Promise((resolve, reject) => {
                                    fs.access(filePath, fs.F_OK, (err) => {
                                        if (err) {
                                            fs.writeFile(filePath, '', function (err) {
                                                if (err) reject(err);
                                                return resolve(r)
                                            });
                                        } else {
                                            return resolve(r);
                                        }
                                    })
                                })
                                    .then(r => {
                                        console.log("File accessed")
                                        try {
                                            fs.createReadStream(filePath).pipe(csvParser())
                                                .on('data', function (data) {
                                                    try {
                                                        if (data) {
                                                            var newData = {};
                                                            if (data['SenderAddress']) newData['senderAddress'] = data['SenderAddress'];
                                                            if (data['RecipientAddress']) newData['recipientAddress'] = data['RecipientAddress'];
                                                            if (data['Subject']) newData['subject'] = data['Subject'];
                                                            if (data['Size']) newData['size'] = data['Size'];
                                                            if (data['FromIP']) newData['fromIp'] = data['FromIP'];
                                                            if (data['Status']) newData['status'] = data['Status'];
                                                            if (data['Received']) newData['received'] = data['Received'];
                                                            newData['reportedMailId'] = job.data.reportedMailId;
                                                            newData['name'] = fileName;
                                                            if (data['Subject'] == r.Header.subject && !resultSet.has(data['RecipientAddress'])) {
                                                                resultSet.add(data['RecipientAddress']);
                                                                results.push(newData);
                                                            } else console.log("Subject Didnt match ", data['Subject'])
                                                        }
                                                    } catch (err) {
                                                        console.log(err)
                                                        done()
                                                    }
                                                })
                                                .on('end', function () {
                                                    console.log("results", results)
                                                    db.ReportedMailResult.bulkCreate(results)
                                                        .then(r => {
                                                            return db.ReportedMail.update({ status: 'processed' }, { where: { id: job.data.reportedMailId } })
                                                        })
                                                        .then(r => { done() })
                                                })
                                        } catch (err) {
                                            console.log("Error while reading file", err);
                                            done()
                                        }
                                    })
                            } catch (err) {
                                console.log(err)
                                done();
                            }
                        } else {
                            db.ReportedMail.update({ status: 'unprocessed' }, { where: { id: job.data.reportedMailId } })
                            done();
                        }
                    })
                    .catch(err => {
                        console.log(err)
                        done();
                    })
            } catch (err) {
                console.log(err)
                done();
            }
        })
        queue.process('dnsbl-check', async (job, done) => {
            console.log("Inside dnsbl-check", job.data);
            let checkExisting = await checkExistingDnsblRecord(job.data.SenderIP);
            if (!checkExisting) {
                let results = await new DNSBL().scanIp({ ip: job.data.SenderIP });
                console.log("Results", results);
                results.forEach(r => {
                    db.dnsblIpScanneds.create({
                        mailId: job.data.mailId,
                        scannedOn: r.blacklist,
                        ip: r.address,
                        listed: r.listed
                    }).catch(err => { console.log("error while saving dnsbl record", err) })
                })
            }
            done();
        })
    }
}