import nodemailer from 'nodemailer';
import config from './config';
import { db } from './models';
import path from 'path';
import moment from 'moment';

export default {
    sendReportMail: (to, subject, date, from) => {
        return new Promise((resolve, reject) => {
            try {
                var smtpTransport = nodemailer.createTransport({
                    host: config.mail.host,
                    port: config.mail.port,
                    auth: {
                        user: config.mail.username,
                        pass: config.mail.password
                    },
                    tls: { rejectUnauthorized: false },
                });
                smtpTransport.sendMail({
                    from: config.mail.from,
                    to: to,
                    subject: 'Your Mail has been successfully Reported',
                    html: "Dear Employee<br><br>Thank You for reporting a suspicious email.<br><br> The email was received at " + new Date(date) + "<br><br> From: " + from + "<br> Subject: " + subject + "<br><br> It has been forwaded to SOC Team for further review.<br> Your actions are helping the organization to keep the network and our data safe!<br><br> Thanks,<br> ThreatAlertButton Team",
                }, function (error, info) {
                    if (error) {
                        return reject({
                            name: "Exception",
                            msg: 'Email Sending Failed'
                        })
                    }
                    return resolve(true)
                });


            } catch (err) {
                console.log(err)
                reject(err);
            }
        });
    },

    SendApprovedMail: (email, type) => {
        return new Promise((resolve, reject) => {
            try {

                var smtpTransport = nodemailer.createTransport({
                    host: config.mail.host,
                    port: config.mail.port,
                    auth: {
                        user: config.mail.username,
                        pass: config.mail.password
                    },
                    tls: { rejectUnauthorized: false },
                });
                smtpTransport.sendMail({
                    from: config.mail.from,
                    to: email,
                    subject: 'Report',
                    html: "Hi <br>" + "Your Email has been marked as Threat. The email you reported to the security team has been recognised as malicious/threat.",
                }, function (error, info) {
                    if (error) {
                        return reject({
                            name: "Exception",
                            msg: 'Email Sending Failed'
                        })
                    }
                    return resolve(true)
                });


            } catch (err) {
                console.log(err)
                reject(err);
            }
        });
    },

    SendDeclinedMail: (email, type) => {
        return new Promise((resolve, reject) => {
            try {

                var smtpTransport = nodemailer.createTransport({
                    host: config.mail.host,
                    port: config.mail.port,
                    auth: {
                        user: config.mail.username,
                        pass: config.mail.password
                    },
                    tls: { rejectUnauthorized: false },
                });
                smtpTransport.sendMail({
                    from: config.mail.from,
                    to: email,
                    subject: 'Report',
                    html: "Hi <br>" + " Your Reported Email has been considered as Safe. The email you reported to the security team has been considered as safe.",
                }, function (error, info) {
                    if (error) {
                        return reject({
                            name: "Exception",
                            msg: 'Email Sending Failed'
                        })
                    }
                    return resolve(true)
                });


            } catch (err) {
                console.log(err)
                reject(err);
            }
        });
    },

    alertSOCTeam: (socEmail, to, subject, date, from, mailId) => {
        console.log("alertSOCTEAM", config.mail.username, config.mail.password);
        return new Promise((resolve, reject) => {
            try {
                var smtpTransport = nodemailer.createTransport({
                    host: config.mail.host,
                    port: config.mail.port,
                    auth: {
                        // user: "apikey",
                        // pass: "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"
                        user: config.mail.username,
                        pass: config.mail.password,
                    },
                    tls: { rejectUnauthorized: false },
                });
                smtpTransport.sendMail({
                    from: config.mail.from,
                    to: socEmail,
                    subject: 'A new Phishing Attack is now active!',
                    html: "Dear Manager <br><br>" + "A new phishing attack was reported by " + to + " after one of your company's employees recieved a phishing email.<br><br> The email was recieved at " + new Date(date) + "<br><br> From: " + from + "<br> Subject: " + subject + "<br><br> Please proceed to an email report page for more details.<br><br> <a href='" + config.app.url + "reported/" + mailId + "'>Email Report Page</a><br><br> Thanks,<br> ThreatAlertButton Team",
                }, function (error, info) {
                    // console.log("Mail sent to soc team", socEmail, error, info);
                    if (error) {
                        return reject({
                            name: "Exception",
                            msg: 'Email Sending Failed'
                        })
                    }
                    return resolve(true)
                });


            } catch (err) {
                console.log(err)
                reject(err);
            }
        });
    },
}